<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Util\Utilities;

class Sm00001000LancamentoPropostaFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Retorna a composição da proposta (o que será inserido na FCT_TRABSERVORCAMENTO).
     * 
     * @param integer[] $escopos
     * $param number $nrorcamentomodelo
     * 
     * @return mixed[]
     */
    public function getElementosComposicao($escopos, $composicoes, $nrorg, $idApareceProposta) {
        $sql = "SELECT DISTINCT NRCOMPSERVICO,
                       IDELEMENTO,
                       IDELEMENTOPAI,
                       IDTIPOELEMENTO,
                       CDTPMEDIDAVARIAVEL,
                       CDELEMENTO,
                       IDSERVOBRIGCOMP,
                       CDSERVORCARELAC,
                       NMSERVORCARELAC,
                       NRTIPOORCAMENTO,
                       NMTIPOORCAMENTO,
                       IDALTQTDMEDIDAPROP,
                       IDFATURACONTR,
                       CDMODPGTO,
                       NRGRUPOFATURA,
                       NVL(IDUTILIZAVARPRECO, 'N') AS IDUTILIZAVARPRECO,
                       CDTPMEDIDASERV,
                       DECODE(IDAPARECEPROPOSTA, 'N', 1, QTMEDIDASERV) AS QTMEDIDASERV,
                       DECODE(IDAPARECEPROPOSTA, 'N', 1, QTMEDIDASERV) AS QTMEDIDACALC,
                       DSTPMEDIDASERVRELAC,
                       DECODE(IDAPARECEPROPOSTA, 'N', 0, VRUNITARIO) AS VRUNITARIO,
                       DECODE(IDAPARECEPROPOSTA, 'N', 0, VRPRECO) AS VRPRECO,
                       DTFIMVALPBSERV,
                       NRORG,
                       NULL AS QTMEDIDAVARIAVEL,
                       IDELEMENTORAIZ,
                       IDMEDIDAFIXA,
                       'N' AS IDMULTIPVARIACAO,
                       DECODE(:IDAPARECEPROPOSTA, 'N', 'S', 'N') AS IDESCOLHIDO,
                       DSMEDIDAVARIAVEL,
                       CDMEDVARIAVELRELAC,
                       IDAPARECEPROPOSTA
                 FROM ( SELECT COMPOSICAO.NRCOMPSERVICO,
                           COMPOSICAO.NRITCOMPSERVICO,
                           COMPOSICAO.IDELEMENTO,
                           COMPOSICAO.IDELEMENTOPAI,
                           COMPOSICAO.IDTIPOELEMENTO,
                           COMPOSICAO.CDTPMEDIDAVARIAVEL,
                           COMPOSICAO.CDELEMENTO,
                           COMPOSICAO.IDSERVOBRIGCOMP,
                           COMPOSICAO.CDSERVORCARELAC,
                           COMPOSICAO.NMSERVORCARELAC,
                           COMPOSICAO.NRTIPOORCAMENTO,
                           COMPOSICAO.NMTIPOORCAMENTO,
                           COMPOSICAO.IDALTQTDMEDIDAPROP,
                           COMPOSICAO.IDFATURACONTR,
                           COMPOSICAO.CDMODPGTO,
                           COMPOSICAO.NRGRUPOFATURA,
                           COMPOSICAO.IDUTILIZAVARPRECO,
                           COMPOSICAO.CDTPMEDIDASERV,
                           COMPOSICAO.QTMEDIDASERV,
                           COMPOSICAO.DSTPMEDIDASERVRELAC,
                           COMPOSICAO.VRPRECO AS VRUNITARIO,
                           COMPOSICAO.VRPRECO * COMPOSICAO.QTMEDIDASERV AS VRPRECO,
                           TO_CHAR(COMPOSICAO.DTFIMVALPBSERV, 'DD/MM/YYYY') AS DTFIMVALPBSERV,
                           COMPOSICAO.NRORG,
                           COMPOSICAO.IDELEMENTORAIZ,
                           COMPOSICAO.IDMEDIDAFIXA,
                           COMPOSICAO.DSMEDIDAVARIAVEL,
                           COMPOSICAO.CDMEDVARIAVELRELAC,
                           COMPOSICAO.IDAPARECEPROPOSTA
                      FROM ( SELECT DISTINCT ITEM.NRCOMPSERVICO,
                               ITEM.NRITCOMPSERVICO,
                               ITEM.IDELEMENTO,
                               ITEM.IDELEMENTOPAI,
                               ELEMENTO.IDTIPOELEMENTO,
                               ELEMENTO.CDTPMEDIDAVARIAVEL,
                               ITEM.CDSERVORCA AS CDELEMENTO,
                               ITEM.IDSERVOBRIGCOMP,
                               SERVRELAC.CDSERVORCARELAC,
                               SERVICO.NMSERVORCA AS NMSERVORCARELAC,
                               ITESCOPO.NRTIPOORCAMENTO,
                               ESCOPO.NMTIPOORCAMENTO,
                               ESCOPO.IDALTQTDMEDIDAPROP,
                               ESCOPO.IDUTILIZAVARPRECO,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.IDFATURACONTR, PARAMSRV.IDFATURACONTR) AS IDFATURACONTR,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.CDMODPGTO, PARAMSRV.CDMODPGTO) AS CDMODPGTO,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.NRGRUPOFATURA, PARAMSRV.NRGRUPOFATURA) AS NRGRUPOFATURA,
                               SERVRELAC.CDTPMEDIDASERV AS CDMEDVARIAVELRELAC,
                               ITPRECO.CDTPMEDIDASERV,
                               MEDIDA.DSTPMEDIDASERV AS DSMEDIDAVARIAVEL,
                               MEDIDARELAC.DSTPMEDIDASERV AS DSTPMEDIDASERVRELAC,
                               DECODE(ITPRECO.VRPRECO, NULL, NULL, SERVRELAC.QTMEDIDASERV) AS QTMEDIDASERV,
                               ITPRECO.VRPRECO,
                               ITPRECO.DTFIMVALPBSERV,
                               ITEM.NRORG,
                               (SELECT ITPAI.IDELEMENTO
                                FROM FCT_ITCOMPSERVICO ITPAI
                                WHERE ITPAI.NRCOMPSERVICO = ITEM.NRCOMPSERVICO
                                    AND ITPAI.IDELEMENTOPAI IS NULL
                                    AND INSTR(ITEM.IDELEMENTO, ITPAI.IDELEMENTO) > 0) AS IDELEMENTORAIZ,
                               SERVRELAC.IDMEDIDAFIXA,
                               ITEM.IDAPARECEPROPOSTA
                           FROM FCT_ITCOMPSERVICO ITEM
                           LEFT JOIN FCT_RELACSERVICO SERVRELAC
                             ON ITEM.CDSERVORCA = SERVRELAC.CDSERVORCA
                            AND ITEM.NRORG      = SERVRELAC.NRORG
                           LEFT JOIN FCT_SERVICOORCA SERVICO
                             ON SERVRELAC.CDSERVORCARELAC = SERVICO.CDSERVORCA
                            AND SERVRELAC.NRORG           = SERVICO.NRORG
                           LEFT JOIN FCT_SERVICOORCA ELEMENTO
                             ON ITEM.CDSERVORCA      = ELEMENTO.CDSERVORCA
                            AND ITEM.NRORG           = ELEMENTO.NRORG 
                           LEFT JOIN FCT_ITESCOPOPROPOSTA ITESCOPO
                             ON SERVRELAC.CDSERVORCARELAC = ITESCOPO.CDSERVORCA
                            AND SERVRELAC.NRORG           = ITESCOPO.NRORG
                           LEFT JOIN FCT_TIPOORCAMENTO ESCOPO
                             ON ITESCOPO.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                            AND ITESCOPO.NRORG           = ESCOPO.NRORG
                           LEFT JOIN FCT_PARAMFATESCOPO PARAMESC
                             ON ESCOPO.NRTIPOORCAMENTO = PARAMESC.NRTIPOORCAMENTO
                            AND ESCOPO.NRORG           = PARAMESC.NRORG
                           LEFT JOIN FCT_PARAMFATSRVESCO PARAMSRV
                             ON ITESCOPO.NRITESCOPOPROPOSTA = PARAMSRV.NRITESCOPOPROPOSTA
                            AND ITESCOPO.NRORG              = PARAMSRV.NRORG
                           LEFT JOIN FCT_ITPRECBASESERV ITPRECO
                             ON ESCOPO.CDPRECBASESERV = ITPRECO.CDPRECBASESERV
                            AND ESCOPO.NRORG          = ITPRECO.NRORG
                            AND ITESCOPO.CDSERVORCA   = ITPRECO.CDSERVORCA
                           LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA
                             ON ELEMENTO.CDTPMEDIDAVARIAVEL = MEDIDA.CDTPMEDIDASERV
                            AND ELEMENTO.NRORG          = MEDIDA.NRORG
                          LEFT JOIN FCT_TIPOMEDIDASERV MEDIDARELAC
                             ON SERVRELAC.CDTPMEDIDASERV = MEDIDARELAC.CDTPMEDIDASERV
                            AND SERVRELAC.NRORG          = MEDIDARELAC.NRORG
                          WHERE ITEM.NRCOMPSERVICO IN ($composicoes)
                            AND ITEM.NRORG = :NRORG
                            AND (ITEM.IDAPARECEPROPOSTA = :IDAPARECEPROPOSTA OR :IDAPARECEPROPOSTA IS NULL)
                            AND TRUNC(NVL(ITPRECO.DTINIPBSERV,SYSDATE)) <= TRUNC(SYSDATE)
                            AND (ITESCOPO.NRTIPOORCAMENTO IN ($escopos) OR ITESCOPO.NRTIPOORCAMENTO IS NULL)
                           ) COMPOSICAO
                   ORDER BY COMPOSICAO.NRTIPOORCAMENTO,
                            COMPOSICAO.CDSERVORCARELAC)";
        
        $params = array(
            "NRORG" => $nrorg,
            "IDAPARECEPROPOSTA" => $idApareceProposta
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getElementosComposicaoSemCodigoComposicao($escopos, $composicoes, $nrorg, $idApareceProposta) {
        $sql = "SELECT DISTINCT
                       IDELEMENTO,
                       IDELEMENTOPAI,
                       IDTIPOELEMENTO,
                       CDTPMEDIDAVARIAVEL,
                       CDELEMENTO,
                       CDSERVORCARELAC,
                       NMSERVORCARELAC,
                       NRTIPOORCAMENTO,
                       NMTIPOORCAMENTO,
                       IDALTQTDMEDIDAPROP,
                       IDFATURACONTR,
                       CDMODPGTO,
                       NRGRUPOFATURA,
                       QTPARCELASFATU,
                       NRDIAVENCPARCELASFATU,
                       IDUTILIZAVARPRECO,
                       CDTPMEDIDASERV,
                       DECODE(IDAPARECEPROPOSTA, 'N', 0, QTMEDIDASERV) AS QTMEDIDASERV,
                       DECODE(IDAPARECEPROPOSTA, 'N', 0, QTMEDIDASERV) AS QTMEDIDACALC,
                       DSTPMEDIDASERV,
                       DECODE(IDAPARECEPROPOSTA, 'N', 0, VRUNITARIO) AS VRUNITARIO,
                       DECODE(IDAPARECEPROPOSTA, 'N', 0, VRPRECO) AS VRPRECO,
                       DTFIMVALPBSERV,
                       NRORG,
                       NULL AS QTMEDIDAVARIAVEL,
                       IDELEMENTORAIZ,
                       NMELEMENTORAIZ,
                       CDMEDVARIAVELRELAC,
                       DSTPMEDIDASERVRELAC,
                       QTMEDSERVRELAC,
                       IDMEDIDAFIXA,
                       'N' AS IDMULTIPVARIACAO,
                       DECODE(:IDAPARECEPROPOSTA, 'N', 'S', 'N') AS IDESCOLHIDO,
                       DSMEDIDAVARIAVEL,
                       CDTPMEDVARESCOPO,
                       DSTPMEDVARESCOPO,
                       IDAPARECEPROPOSTA
                 FROM (
                    SELECT COMPOSICAO.NRCOMPSERVICO,
                           COMPOSICAO.NRITCOMPSERVICO,
                           COMPOSICAO.IDELEMENTO,
                           COMPOSICAO.IDELEMENTOPAI,
                           COMPOSICAO.IDTIPOELEMENTO,
                           COMPOSICAO.CDTPMEDIDAVARIAVEL,
                           COMPOSICAO.CDELEMENTO,
                           COMPOSICAO.IDSERVOBRIGCOMP,
                           COMPOSICAO.CDSERVORCARELAC,
                           COMPOSICAO.NMSERVORCARELAC,
                           COMPOSICAO.NRTIPOORCAMENTO,
                           COMPOSICAO.NMTIPOORCAMENTO,
                           COMPOSICAO.IDALTQTDMEDIDAPROP,
                           COMPOSICAO.IDFATURACONTR,
                           COMPOSICAO.CDMODPGTO,
                           COMPOSICAO.NRGRUPOFATURA,
                           COMPOSICAO.QTPARCELASFATU,
                           COMPOSICAO.NRDIAVENCPARCELASFATU,
                           COMPOSICAO.IDUTILIZAVARPRECO,
                           COMPOSICAO.CDTPMEDIDASERV,
                           COMPOSICAO.QTMEDIDASERV,
                           COMPOSICAO.QTMEDSERVRELAC,
                           COMPOSICAO.DSTPMEDIDASERV,
                           COMPOSICAO.VRPRECO AS VRUNITARIO,
                           COMPOSICAO.VRPRECO * COMPOSICAO.QTMEDIDASERV AS VRPRECO,
                           TO_CHAR(COMPOSICAO.DTFIMVALPBSERV, 'DD/MM/YYYY') AS DTFIMVALPBSERV,
                           COMPOSICAO.NRORG,
                           COMPOSICAO.IDELEMENTORAIZ,
                           ELE.NMSERVORCA AS NMELEMENTORAIZ,
                           COMPOSICAO.CDMEDVARIAVELRELAC,
                           COMPOSICAO.DSTPMEDIDASERVRELAC,
                           COMPOSICAO.IDMEDIDAFIXA,
                           COMPOSICAO.DSMEDIDAVARIAVEL,
                           COMPOSICAO.CDTPMEDVARESCOPO,
                           COMPOSICAO.DSTPMEDVARESCOPO,
                           COMPOSICAO.IDAPARECEPROPOSTA
                      FROM (
                        SELECT DISTINCT ITEM.NRCOMPSERVICO,
                               ITEM.NRITCOMPSERVICO,
                               ITEM.IDELEMENTO,
                               ITEM.IDELEMENTOPAI,
                               ELEMENTO.IDTIPOELEMENTO,
                               ELEMENTO.CDTPMEDIDAVARIAVEL,
                               ITEM.CDSERVORCA AS CDELEMENTO,
                               ITEM.IDSERVOBRIGCOMP,
                               SERVRELAC.CDSERVORCARELAC,
                               SERVICO.NMSERVORCA AS NMSERVORCARELAC,
                               SERVRELAC.CDTPMEDIDASERV AS CDMEDVARIAVELRELAC,
                               MEDIDA.DSTPMEDIDASERV AS DSMEDIDAVARIAVEL,
                               MEDIDARELAC.DSTPMEDIDASERV AS DSTPMEDIDASERVRELAC,
                               ITESCOPO.NRTIPOORCAMENTO,
                               ESCOPO.NMTIPOORCAMENTO,
                               ESCOPO.IDALTQTDMEDIDAPROP,
                               ESCOPO.IDUTILIZAVARPRECO,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.IDFATURACONTR, PARAMSRV.IDFATURACONTR) AS IDFATURACONTR,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.CDMODPGTO, PARAMSRV.CDMODPGTO) AS CDMODPGTO,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.NRGRUPOFATURA, PARAMSRV.NRGRUPOFATURA) AS NRGRUPOFATURA,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.QTPARCELASFATU, PARAMSRV.QTPARCELASFATU) AS QTPARCELASFATU,
                               DECODE(PARAMSRV.NRSEQFATUSRVESCO, NULL, PARAMESC.NRDIAVENCPARCELASFATU, PARAMSRV.NRDIAVENCPARCELASFATU) AS NRDIAVENCPARCELASFATU,
                               ITPRECO.CDTPMEDIDASERV,
                               NVL(PFITESC.CDTPMEDIDAVARIAVEL, CAPAESC.CDTPMEDIDAVARIAVEL) AS CDTPMEDVARESCOPO,
                               NVL(MEDVARITESCOPO.DSTPMEDIDASERV, MEDVARCAPAESCOPO.DSTPMEDIDASERV) AS DSTPMEDVARESCOPO,
                               MEDIDA.DSTPMEDIDASERV,
                               DECODE(ITPRECO.VRPRECO, NULL, NULL, SERVRELAC.QTMEDIDASERV) AS QTMEDIDASERV,
                               SERVRELAC.QTMEDIDASERV AS QTMEDSERVRELAC,
                               ITPRECO.VRPRECO,
                               ITPRECO.DTFIMVALPBSERV,
                               ITEM.NRORG,
                               (SELECT DISTINCT IDELEMENTO
                                   FROM FCT_ITCOMPSERVICO
                                  WHERE NRCOMPSERVICO = ITEM.NRCOMPSERVICO
                                    AND CONNECT_BY_ISLEAF = 1
                                CONNECT BY PRIOR IDELEMENTOPAI = IDELEMENTO
                                    AND PRIOR NRCOMPSERVICO = ITEM.NRCOMPSERVICO
                                  START WITH IDELEMENTO = ITEM.IDELEMENTO) AS IDELEMENTORAIZ,
                                SERVRELAC.IDMEDIDAFIXA,
                                ITEM.IDAPARECEPROPOSTA
                           FROM FCT_ITCOMPSERVICO ITEM
                           LEFT JOIN FCT_RELACSERVICO SERVRELAC
                             ON ITEM.CDSERVORCA = SERVRELAC.CDSERVORCA
                            AND ITEM.NRORG      = SERVRELAC.NRORG
                           LEFT JOIN FCT_SERVICOORCA SERVICO
                             ON SERVRELAC.CDSERVORCARELAC = SERVICO.CDSERVORCA
                            AND SERVRELAC.NRORG           = SERVICO.NRORG
                           LEFT JOIN FCT_SERVICOORCA ELEMENTO
                             ON ITEM.CDSERVORCA      = ELEMENTO.CDSERVORCA
                            AND ITEM.NRORG           = ELEMENTO.NRORG 
                           LEFT JOIN FCT_ITESCOPOPROPOSTA ITESCOPO
                             ON SERVRELAC.CDSERVORCARELAC = ITESCOPO.CDSERVORCA
                            AND SERVRELAC.NRORG           = ITESCOPO.NRORG
                           LEFT JOIN FCT_TIPOORCAMENTO ESCOPO
                             ON ITESCOPO.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                            AND ITESCOPO.NRORG           = ESCOPO.NRORG
                           LEFT JOIN FCT_CAPAPARAMFATESC CAPAESC
                             ON ESCOPO.NRTIPOORCAMENTO = CAPAESC.NRTIPOORCAMENTO
                            AND ESCOPO.NRORG           = CAPAESC.NRORG
                           LEFT JOIN FCT_PARAMFATITESC PFITESC
                             ON ITESCOPO.NRITESCOPOPROPOSTA = PFITESC.NRITESCOPOPROPOSTA
                            AND ITESCOPO.NRORG              = PFITESC.NRORG
                           LEFT JOIN FCT_PARAMFATESCOPO PARAMESC
                             ON ESCOPO.NRTIPOORCAMENTO = PARAMESC.NRTIPOORCAMENTO
                            AND ESCOPO.NRORG           = PARAMESC.NRORG
                           LEFT JOIN FCT_PARAMFATSRVESCO PARAMSRV
                             ON ITESCOPO.NRITESCOPOPROPOSTA = PARAMSRV.NRITESCOPOPROPOSTA
                            AND ITESCOPO.NRORG              = PARAMSRV.NRORG
                           LEFT JOIN FCT_ITPRECBASESERV ITPRECO
                             ON ESCOPO.CDPRECBASESERV = ITPRECO.CDPRECBASESERV
                            AND ESCOPO.NRORG          = ITPRECO.NRORG
                            AND ITESCOPO.CDSERVORCA   = ITPRECO.CDSERVORCA
                           LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA
                             ON ELEMENTO.CDTPMEDIDAVARIAVEL = MEDIDA.CDTPMEDIDASERV
                            AND ELEMENTO.NRORG              = MEDIDA.NRORG
                           LEFT JOIN FCT_TIPOMEDIDASERV MEDVARCAPAESCOPO
                             ON CAPAESC.CDTPMEDIDAVARIAVEL = MEDVARCAPAESCOPO.CDTPMEDIDASERV
                            AND CAPAESC.NRORG              = MEDVARCAPAESCOPO.NRORG
                           LEFT JOIN FCT_TIPOMEDIDASERV MEDVARITESCOPO
                             ON PFITESC.CDTPMEDIDAVARIAVEL = MEDVARITESCOPO.CDTPMEDIDASERV
                            AND PFITESC.NRORG              = MEDVARITESCOPO.NRORG
                           LEFT JOIN FCT_TIPOMEDIDASERV MEDIDARELAC
                             ON SERVRELAC.CDTPMEDIDASERV = MEDIDARELAC.CDTPMEDIDASERV
                            AND SERVRELAC.NRORG          = MEDIDARELAC.NRORG
                          WHERE ITEM.NRCOMPSERVICO IN ($composicoes)
                            AND (ITESCOPO.NRTIPOORCAMENTO IN ($escopos) OR (ITESCOPO.NRTIPOORCAMENTO IS NULL AND SERVRELAC.CDSERVORCARELAC IS NULL))
                            AND ITEM.NRORG = :NRORG
                            AND (ITEM.IDAPARECEPROPOSTA = :IDAPARECEPROPOSTA OR :IDAPARECEPROPOSTA IS NULL)
                            AND TRUNC(NVL(ITPRECO.DTINIPBSERV,SYSDATE)) <= TRUNC(SYSDATE)
                           ) COMPOSICAO
                    LEFT JOIN FCT_SERVICOORCA ELE
                      ON COMPOSICAO.IDELEMENTORAIZ = ELE.IDELEMENTO
                     AND COMPOSICAO.NRORG          = ELE.NRORG
                   ORDER BY COMPOSICAO.NRTIPOORCAMENTO,
                            COMPOSICAO.CDSERVORCARELAC)";
        
        $params = array(
            "NRORG" => $nrorg,
            "IDAPARECEPROPOSTA" => $idApareceProposta
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Insere os Elementos da Proposta.
     * 
     * @param string[] $params
     */
    public function inserirElementosProposta(array $params) {
        $sql = "INSERT INTO FCT_TRABSERVORCAMENTO (
                    NRTRABSERVORCA,     NRSEQORCAMENTO,     NRTIPOORCAMENTO,    NRCOMPSERVICO,
                    NRITCOMPSERVICO,    IDELEMENTO,         IDELEMENTOPAI,      IDTIPOELEMENTO,
                    IDESCOLHIDO,        CDSERVORCA,         QTSERVORCA,         CDTPMEDIDASERV,     
                    QTMEDSERVORCA,      VRUNITSERVORCA,     VRTOTSERVORCA,      IDATIVO,            
                    NRORG,              CDOPERINCLUSAO,     NRORGINCLUSAO,      DTINCLUSAO,
                    IDSERVOBRIGCOMP,    QTMEDIDAVARIAVEL,   QTMEDIDACALC,       VRTOTSAJUSTE,
                    IDAPARECEPROPOSTA
                )
                VALUES (
                    :NRTRABSERVORCA,          :NRSEQORCAMENTO,             :NRTIPOORCAMENTO,         :NRCOMPSERVICO,
                    :NRITCOMPSERVICO,         :IDELEMENTO,                 :IDELEMENTOPAI,           :IDTIPOELEMENTO,
                    :IDESCOLHIDO,             :CDSERVORCA,                 ROUND(:QTSERVORCA, 2),    :CDTPMEDIDASERV,    
                    ROUND(:QTMEDSERVORCA, 2), ROUND(:VRUNITSERVORCA, 2),   ROUND(:VRTOTSERVORCA, 2), :IDATIVO,           
                    :NRORG,                   :CDOPERINCLUSAO,             :NRORGINCLUSAO,           SYSDATE,
                    :IDSERVOBRIGCOMP,         ROUND(:QTMEDIDAVARIAVEL, 2), ROUND(:QTMEDIDACALC, 2),  ROUND(:VRTOTSAJUSTE, 2),
                    :IDAPARECEPROPOSTA
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Insere os Itens da Proposta.
     * 
     * @param string[] $params
     * 
     * 
     */
    public function inserirItensProposta(array $params) {
        $sql = "INSERT INTO FCT_ITEMORCAMENTO (
                    NRITEMORCAMENTO,        NRSEQORCAMENTO,         CDSERVORCA,             NMITEMORCAMENTO,
                    DSQTITEMORCA,           QTITEMORCAMENTO,        CDTPMEDIDASERV,         QTMEDIDAORCA,
                    VRUNITITEMORCA,         VRTOTALITEMORCA,        IDITEMOBRIGORCA,        IDSOMASUBITENS,
                    IDSOMANOITEMPAI,        IDSOMANOTOTALORCA,      IDIMPRESSAOORCA,        IDPRECOLOCACAO,
                    IDMULTMEDVRTOT,         IDBLOQALTORC,           IDADICITEMORC,          IDADICCOBRITEMORC,
                    IDTPDESCORCA,           IDTRANSPORTADORA,       IDATIVO,                NRORG,
                    CDOPERINCLUSAO,         NRORGINCLUSAO,          DTINCLUSAO,             NRTIPOORCAMENTO,
                    IDFATURAORCA,           CDMODPGTO,              NRGRUPOFATURA,          QTPARCELASFATU,
                    NRDIAVENCPARCELASFATU,  VRPERCDESCORCA,         VRITEMSEMDESC,          IDELEMENTO,
                    CDTPMEDIDAVARIAVEL,     NRITEMORCAMENTOPAI,     VRUNITITEMORCATBLPRECO, IDMEDIDAFIXA,
                    NMCONTRATUALITEMORCA,   VRDESCORCA,             TXFORMULAITORCA,        CDOPERULTATU,
                    NRORGULTATU,            DTULTATU,               QTMEDIDACALCULADA,      IDAPARECEPROPOSTA
                )
                VALUES (
                    :NRITEMORCAMENTO,          :NRSEQORCAMENTO,            :CDSERVORCA,                       :NMITEMORCAMENTO,
                    :DSQTITEMORCA,             ROUND(:QTITEMORCAMENTO, 2), :CDTPMEDIDASERV,                   ROUND(:QTMEDIDAORCA, 2),
                    ROUND(:VRUNITITEMORCA, 2), ROUND(:VRTOTALITEMORCA, 2), :IDITEMOBRIGORCA,                  :IDSOMASUBITENS,
                    :IDSOMANOITEMPAI,          :IDSOMANOTOTALORCA,         :IDIMPRESSAOORCA,                  :IDPRECOLOCACAO,
                    :IDMULTMEDVRTOT,           :IDBLOQALTORC,              :IDADICITEMORC,                    :IDADICCOBRITEMORC,
                    :IDTPDESCORCA,             :IDTRANSPORTADORA,          :IDATIVO,                          :NRORG,
                    :CDOPERINCLUSAO,           :NRORGINCLUSAO,             SYSDATE,                           :NRTIPOORCAMENTO,
                    :IDFATURAORCA,             :CDMODPGTO,                 :NRGRUPOFATURA,                    :QTPARCELASFATU,
                    :NRDIAVENCPARCELASFATU,    ROUND(:VRPERCDESCORCA, 2),  ROUND(:VRITEMSEMDESC, 2),          :IDELEMENTO,
                    :CDTPMEDIDAVARIAVEL,       :NRITEMORCAMENTOPAI,        ROUND(:VRUNITITEMORCATBLPRECO, 2), :IDMEDIDAFIXA,
                    :NMCONTRATUALITEMORCA,     ROUND(:VRDESCORCA, 2),      :TXFORMULAITORCA,                  :CDOPERULTATU,
                    :NRORGULTATU,              :DTULTATU,                  ROUND(:QTMEDIDACALCULADA, 2),      :IDAPARECEPROPOSTA
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna as propostas filhas da proposta de código informado.
     * 
     * @param mixed[] $params
     * 
     * @return mixed[]
     */
    public function getPropostasFilhas(array $params) {
        $sql = "SELECT NRSEQORCAMENTO
                  FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTOPAI = :NRSEQORCAMENTOPAI
                   AND NRORG             = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna os dados que irar compor o assunto e corpo do email  
     * 
     * @param mixed[] $params
     * 
     * @return mixed[]
     */
    public function getCorpoParaEmailProposta(array $params) {
        $sql = "SELECT ORC.NRORCAMENTO, ORC.DSREFERENCIAORCA, ORC.VRORCAMENTO, ORC.DTREFORCAMENTO, PARC.NMPRINCIPALPARC 
                FROM FCT_ORCAMENTO ORC
                INNER JOIN PARCNEGOCIO PARC
                ON PARC.NRPARCNEGOCIO = ORC.NRPARCNEGOCIO
                AND PARC.NRORG = ORC.NRORG
                WHERE ORC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                AND ORC.NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Atualiza a situação das propostas informadas.
     * 
     * @param string  $situacao
     * @param string  $nrseqorcamento
     * @param integer $nrorg
     */
    public function atualizarSituacaoPropostas($situacao, $nrseqorcamento, $nrorg) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO
                 WHERE NRSEQORCAMENTO IN (".$nrseqorcamento.")
                   AND NRORG          = :NRORG";
        
        $params = array(
            "IDSITUACAOORCAMENTO" => $situacao,
            "NRORG"               => $nrorg
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Atualiza a Proposta ao aceitá-la.
     * 
     * @param mixed[] $params
     */
    public function atualizarPropostaAceite(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO,
                       DTACEITEORCA        = :DTACEITEORCA,
                       CDOPERULTATU        = :CDOPERULTATU,
                       DTULTATU            = SYSDATE
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Deleta as estruturas ligadas a uma proposta
     */
    public function excluirRelacionamentoEstruturaAprop(array $params) {
        $sql = "DELETE FROM FCT_RELACESTRUTURAPROP
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna os Documentos da Proposta.
     * 
     * @param mixed[] $params
     * @return mixed[]
     */
    public function getDocumentosProposta(array $params) {
        $sql = "SELECT LISTAGG(NRSEQORCADOC, ',') WITHIN GROUP (ORDER BY NRSEQORCADOC) AS NRSEQORCADOC,
                    NMARQUIVO,
                    DSCAMINHOARQ,
                    DSTIPO,
                    VRTAMANHO,
                    NRORG
                FROM FCT_ORCAMENTODOC
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG        = :NRORG
                GROUP BY NMARQUIVO,
                    DSCAMINHOARQ,
                    DSTIPO,
                    VRTAMANHO,
                    NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o e-mail do Vendedor da Proposta.
     * 
     * @param mixed[] $params
     * @return mixed[]
     */
    public function getEmailVendedorProposta(array $params) {
        $sql = "SELECT C.CDCOMUNICAPARC
                  FROM COMUNICAPARC C
                 INNER JOIN FCT_VENDEDORORCA V
                    ON C.NRPARCNEGOCIO = V.NRPARCVENDEDOR
                   AND C.NRORG         = V.NRORG
                 WHERE V.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND V.NRORG          = :NRORG
                   AND V.IDVENPRIORCA   = 'S'
                   AND C.CDFORMACOMU    = '05'
                   AND C.IDATIVO        = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
     /**
     * Retorna os dados do e-mail da Parametrização SM
     * 
     * @param integer $nrorg
     */
    public function getEmailParametrizado(array $params) {
        $sql = "SELECT CDFILIAL,
                   DSREMETENTEEMAIL,
                   DSASSUNTOEMAIL,
                   DSCORPOEMAIL FROM FCT_PARAMEMAILFILIAL
                   WHERE NRORG  = :NRORG
                   AND (CDFILIAL = :CDFILIAL OR CDFILIAL IS NULL)";
                            
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o e-mail da Empresa da Proposta.
     * 
     * @param mixed[] $params
     * @return mixed[]
     */
    public function getEmailEmpresaProposta(array $params) {
        $sql = "SELECT E.DSEMAILEMPRESA
                  FROM EMPRESA E
                 INNER JOIN FILIAL F
                    ON E.CDEMPRESA = F.CDEMPRESA
                   AND E.NRORG     = F.NRORG
                 WHERE F.CDFILIAL = (SELECT CDFILIAL
                                       FROM FCT_RELACESCOPOPROPOSTA
                                      WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                        AND NRORG          = :NRORG
                                        AND ROWNUM         = 1)
                   AND F.NRORG    = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o e-mail do Interlocutor da Proposta.
     * 
     * @param mixed[] $params
     * @return mixed[]
     */
    public function getEmailInterlocutorProposta(array $params) {
        $sql = "SELECT DSEMAIL
                 FROM FCT_ORCAMENTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Atualiza os elementos da proposta (tabela FCT_TRABSERVORCAMENTO).
     */
    public function atualizarElementosComposicaoProposta(array $params) {
        $sql = "UPDATE FCT_TRABSERVORCAMENTO
                   SET IDESCOLHIDO  = :IDESCOLHIDO,
                       DTULTATU     = SYSDATE,
                       CDOPERULTATU = :CDOPERULTATU,
                       NRORGULTATU  = :NRORGULTATU
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND IDELEMENTO     = :IDELEMENTO
                   AND NRCOMPSERVICO  = :NRCOMPSERVICO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Exclui os itens da proposta.
     */
    public function excluirItensProposta(array $params) {
        $sql = "DELETE FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Atualiza o valor total da proposta com a soma dos itens.
     */
    public function atualizarValorProposta(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET VRORCAMENTO    = ROUND(:VRORCAMENTO, 2)
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
   
    /**
     * Retorna a soma dos itens da proposta.
     */
    public function getSomaValorItensProposta(array $params) {
        $sql = "SELECT NVL(SUM(VRTOTALITEMORCA), 0) AS VRTOTALITEMORCA
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG
                   AND IDELEMENTO IS NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Insere os tipos de medida variáveis da proposta.
     */
    public function inserirTipoDeMedidaProposta(array $params) {
        $sql = "INSERT INTO FCT_RELACMEDIDAPROPOSTA (
                    NRRELACMEDIDAPROPOSTA,  NRSEQORCAMENTO,  CDTPMEDIDAVARIAVEL,  QTMEDIDAVARIAVEL,
                    NRORG,                  DTINCLUSAO,      NRORGINCLUSAO,       CDOPERINCLUSAO,
                    IDATIVO,                IDELEMENTORAIZ,  QTFRANQUIA,          NRTIPOORCAMENTO,
                    IDAPARECEPROPOSTA
                )
                VALUES (
                    :NRRELACMEDIDAPROPOSTA, :NRSEQORCAMENTO, :CDTPMEDIDAVARIAVEL, ROUND(:QTMEDIDAVARIAVEL, 2),
                    :NRORG,                 SYSDATE,         :NRORGINCLUSAO,      :CDOPERINCLUSAO,
                    :IDATIVO,               :IDELEMENTORAIZ, :QTFRANQUIA,         :NRTIPOORCAMENTO,
                    :IDAPARECEPROPOSTA
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Exclui a proposta.
     */
    public function excluirProposta(array $params) {
        $sql = "DELETE FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }


    public function validaOrcamentoValidos(array $params) {
        $sql = "SELECT NRORCAMENTOMODELO 
                FROM FCT_ORCAMENTOMODELO 
                WHERE IDATIVO            = 'N'
                   AND NRORG             = :NRORG
                   AND NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Exclui os escopos da proposta.
     */
    public function excluirEscopos(array $params) {
        $sql = "DELETE FROM FCT_RELACESCOPOPROPOSTA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna todas as variações de preço para as tabelas de preço ligadas aos escopos informados.
     * 
     * @param string $escopos
     * @param mixed[] $params
     * 
     * @return mixed[]
     */
    public function getVariacoesPreco($escopos, $params) {
        $sql = "SELECT VARIACAO.CDTPMEDIDASERV,
                       VARIACAO.CDTPMEDIDAVARIAVEL,
                       VARIACAO.QTMEDIDAINIVARIACAO,
                       VARIACAO.QTMEDIDAFIMVARIACAO,
                       VARIACAO.VRPRECOVARIACAO,
                       VARIACAO.IDMULTIPVARIACAO,
                       TO_CHAR(VARIACAO.DTFIMVALPBSERV, 'DD/MM/YYYY') AS DTFIMVALPBSERV,
                       ITPRECO.CDSERVORCA,
                       ESCOPO.NRTIPOORCAMENTO
                  FROM FCT_VARIACAOPRECBASESERV VARIACAO
                 INNER JOIN FCT_ITPRECBASESERV ITPRECO
                    ON VARIACAO.NRSEQITPRECBSERV = ITPRECO.NRSEQITPRECBSERV
                   AND VARIACAO.NRORG            = ITPRECO.NRORG
                 INNER JOIN FCT_PRECBASESERV PRECO
                    ON ITPRECO.CDPRECBASESERV = PRECO.CDPRECBASESERV
                   AND ITPRECO.NRORG          = PRECO.NRORG
                 INNER JOIN FCT_TIPOORCAMENTO ESCOPO
                    ON PRECO.CDPRECBASESERV = ESCOPO.CDPRECBASESERV
                   AND PRECO.NRORG          = ESCOPO.NRORG
                 WHERE ESCOPO.NRTIPOORCAMENTO                   IN ($escopos)
                   AND ESCOPO.IDUTILIZAVARPRECO                  = 'S'
                   AND ITPRECO.NRORG                             = :NRORG
                   AND TRUNC(NVL(VARIACAO.DTINIPBSERV,SYSDATE)) <= TRUNC(SYSDATE)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna as variações de preço ligadas ao orçamento após a sua conclusao
    **/
    public function getVariacaoPrecoLigadaAoOrca(array $params){
        $sql = "SELECT VARIACAO.CDTPMEDIDASERV,
                           MEDIDA.DSTPMEDIDASERV,
                           VARIACAO.CDTPMEDIDAVARIAVEL,
                           MEDIDAVAR.DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL,
                           VARIACAO.QTMEDIDAINIVARIACAO,
                           VARIACAO.QTMEDIDAFIMVARIACAO,
                           VARIACAO.VRPRECOVARIACAO,
                           NVL(VARIACAO.IDMULTIPVARIACAO, 'N') AS IDMULTIPVARIACAO,
                           VARIACAO.CDSERVORCA,
                           ESCOPO.NRTIPOORCAMENTO,
                           ESCOPO.NMTIPOORCAMENTO,
                           NULL AS DTFIMVALPBSERV
                    FROM FCT_VARPRECBASESERVORCA VARIACAO
                    INNER JOIN FCT_RELACESCOPOPROPOSTA RELACESC
                      ON VARIACAO.NRSEQORCAMENTO = RELACESC.NRSEQORCAMENTO
                     AND VARIACAO.NRORG          = RELACESC.NRORG
                    INNER JOIN FCT_TIPOORCAMENTO ESCOPO
                      ON RELACESC.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                     AND RELACESC.NRORG           = ESCOPO.NRORG
                    INNER JOIN FCT_TIPOMEDIDASERV MEDIDA
                      ON VARIACAO.CDTPMEDIDASERV = MEDIDA.CDTPMEDIDASERV
                     AND VARIACAO.NRORG          = MEDIDA.NRORG
                    INNER JOIN FCT_TIPOMEDIDASERV MEDIDAVAR
                      ON VARIACAO.CDTPMEDIDAVARIAVEL = MEDIDAVAR.CDTPMEDIDASERV
                     AND VARIACAO.NRORG              = MEDIDAVAR.NRORG
                    WHERE ESCOPO.IDUTILIZAVARPRECO = 'S'
                       AND VARIACAO.NRORG          = :NRORG
                       AND VARIACAO.NRSEQORCAMENTO = :NRSEQORCAMENTO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
   
    public function getConfiguracoesEmail($params) {
        $sql = "SELECT DSHOSTEMAIL, DSPORTAEMAIL, DSUSUARIOEMAIL, DSSENHAEMAIL, IDSMTPSECURE
                  FROM FCT_PARAMETROGERAL
                 WHERE NRORG = :nrorg";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna alguns dados da proposta para inserir nos itens.
     */
    public function getDadosPropostaParaItens(array $params) {
        $sql = "SELECT IDFATURAORCA,
                       CDMODPGTO,
                       NRGRUPOFATURA,
                       QTPARCELASFATU,
                       NRDIAVENCPARCELASFATU
                  FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna as variações de preço da proposta.
     */
    public function getVariacaoPreco(array $params) {
        $sql = "SELECT TRAB.NRTRABSERVORCA,
                       TRAB.NRORG,
                       PRECO.VRPRECOVARIACAO
                  FROM FCT_TRABSERVORCAMENTO TRAB
                 INNER JOIN (SELECT RELMEDIDA.NRSEQORCAMENTO,
                                    ELEMENTO.IDELEMENTO,
                                    ITPRECO.CDSERVORCA,
                                    VARIACAO.VRPRECOVARIACAO,
                                    RELMEDIDA.NRORG
                               FROM FCT_VARIACAOPRECBASESERV VARIACAO
                              INNER JOIN FCT_ITPRECBASESERV ITPRECO
                                 ON VARIACAO.NRSEQITPRECBSERV = ITPRECO.NRSEQITPRECBSERV
                                AND VARIACAO.NRORG            = ITPRECO.NRORG
                              INNER JOIN FCT_RELACSERVICO RELSERVICO
                                 ON ITPRECO.CDSERVORCA = RELSERVICO.CDSERVORCARELAC
                                AND ITPRECO.NRORG      = RELSERVICO.NRORG
                              INNER JOIN FCT_SERVICOORCA ELEMENTO
                                 ON RELSERVICO.CDSERVORCA = ELEMENTO.CDSERVORCA
                                AND RELSERVICO.NRORG      = ELEMENTO.NRORG
                              INNER JOIN FCT_RELACMEDIDAPROPOSTA RELMEDIDA
                                 ON ELEMENTO.CDTPMEDIDAVARIAVEL = RELMEDIDA.CDTPMEDIDAVARIAVEL
                                AND ELEMENTO.NRORG             = RELMEDIDA.NRORG
                              WHERE RELMEDIDA.NRSEQORCAMENTO = :NRSEQORCAMENTO
                                AND RELMEDIDA.NRORG          = :NRORG
                                AND RELMEDIDA.QTMEDIDAVARIAVEL BETWEEN VARIACAO.QTMEDIDAINIVARIACAO AND VARIACAO.QTMEDIDAFIMVARIACAO
                                AND ITPRECO.CDPRECBASESERV IN (SELECT CDPRECBASESERV
                                                                 FROM FCT_RELACESCOPOMODELO
                                                                WHERE NRSEQORCAMENTO = RELMEDIDA.NRSEQORCAMENTO
                                                                  AND NRORG          = RELMEDIDA.NRORG)) PRECO
                     ON TRAB.NRSEQORCAMENTO = PRECO.NRSEQORCAMENTO
                    AND TRAB.IDELEMENTO     = PRECO.IDELEMENTO
                    AND TRAB.CDSERVORCA     = PRECO.CDSERVORCA
                    AND TRAB.NRORG          = PRECO.NRORG
                  WHERE TRAB.IDESCOLHIDO    = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Atualiza os preços dos elementos da proposta
     */
    public function atualizarPrecos(array $params) {
        $sql = "UPDATE FCT_TRABSERVORCAMENTO
                   SET VRUNITSERVORCA = ROUND(:VRUNITSERVORCA, 2),
                       VRTOTSERVORCA  = ROUND(:VRUNITSERVORCA * QTMEDSERVORCA, 2)
                 WHERE NRTRABSERVORCA = :NRTRABSERVORCA
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna as variações de preço da proposta
     */
    public function getVariacoesPrecoProposta(array $params) {
        $sql = "SELECT REL.NRRELACMEDIDAPROPOSTA,
                       REL.CDTPMEDIDAVARIAVEL AS CDTPMEDIDASERV,
                       REL.QTMEDIDAVARIAVEL AS QUANTIDADE,
                       REL.IDELEMENTORAIZ,
                       ELE.NMSERVORCA AS NMELEMENTORAIZ,
                       REL.NRSEQORCAMENTO,
                       REL.NRORG,
                       REL.QTFRANQUIA,
                       REL.NRTIPOORCAMENTO
                  FROM FCT_RELACMEDIDAPROPOSTA REL
                 INNER JOIN FCT_SERVICOORCA ELE
                     ON REL.IDELEMENTORAIZ = ELE.IDELEMENTO
                    AND REL.NRORG          = ELE.NRORG
                 WHERE REL.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND REL.NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Exclui os elementos da proposta
     */
    public function excluirElementosProposta(array $params) {
        $sql = "DELETE
                  FROM FCT_TRABSERVORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna os Elementos escolhidos.
     */
    public function getElementosEscolhidos(array $params) {
        $sql = "SELECT DISTINCT IDELEMENTO, NRCOMPSERVICO
                  FROM FCT_TRABSERVORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG
                   AND IDESCOLHIDO    = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os Itens da Proposta.
     */
    public function getItensProposta(array $params) {
        $sql = "SELECT NRITEMORCAMENTO,
                       NRITEMORCAMENTOPAI,
                       NMITEMORCAMENTO,
                       CDSERVORCA,
                       NRTIPOORCAMENTO,
                       QTITEMORCAMENTO,
                       VRUNITITEMORCA,
                       VRTOTALITEMORCA,
                       IDTPDESCORCA,
                       NVL(VRPERCDESCORCA, 0) AS VRPERCDESCORCA,
                       NVL(VRDESCORCA, 0) AS VRDESCORCA,
                       QTMEDIDAORCA,
                       IDELEMENTO,
                       CDTPMEDIDAVARIAVEL,
                       CDTPMEDIDASERV,
                       NMCONTRATUALITEMORCA,
                       VRUNITITEMORCATBLPRECO,
                       QTMEDIDACALCULADA,
                       CDOPERINCLUSAO,
                       DECODE(IDELEMENTO, NULL, 0, VRUNITITEMORCA - VRUNITITEMORCATBLPRECO) AS VRACRESCIMOITEM
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "verificaDesconto" verifica se algum item do orcaçamento possui desconto, 
     * para chamar o relatório de prévia corretamente.
     */
    public function verificaDesconto(array $params) {
        
        $sql = "SELECT COUNT(*) AS DESCONTO
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    /* A função "buscaModeloDocumento" busca o modelo de documentos para cada escopo a fim
       de ser inserido na tabela FCT_RELACESCOPOPROPOSTA*/
    
    public function buscaModeloDocumento(array $params) {
        $sql = "SELECT ESCOPODOC.NRSEQMODDOCFCT, 
                       PARAMFAT.CDFILIALFATURA AS CDFILIAL
                FROM FCT_ORCAESCOPODOC ESCOPODOC
                INNER JOIN FCT_RELACESCOPOMODELO REL
                  ON ESCOPODOC.NRRELACESCOPOMODELO = REL.NRRELACESCOPOMODELO
                  AND ESCOPODOC.NRORG              = REL.NRORG
                INNER JOIN FCT_CAPAPARAMFATESC PARAMFAT
                  ON REL.NRTIPOORCAMENTO = PARAMFAT.NRTIPOORCAMENTO
                  AND REL.NRORG          = PARAMFAT.NRORG
                WHERE REL.NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                 AND REL.NRTIPOORCAMENTO    = :NRTIPOORCAMENTO
                 AND REL.NRORG              = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    /**
     * A função "buscaFilialHierarquia" busca a filial vinculado ao operador na hierarquiia de operadores (CRM_HIERARQUIAOPER)
     * a fim de ser inserido na tabela FCT_RELACESCOPOPROPOSTA
     */
     
    public function buscaFilialHierarquia(array $params) {
        $sql = "SELECT CDFILIAL
                  FROM CRM_HIERARQUIAOPER
                  WHERE CDOPERADOR = :CDOPERADOR
                    AND NRORG = :NRORG
                    AND ROWNUM = 1
                ORDER BY DTULTATU DESC";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Verifica se o parceiro de negócio é um vendedor.
     */
    public function verificarSeEVendedor(array $params) {
        $sql = "SELECT COUNT(*) AS EXISTE
                  FROM PARCNEGOCIO PARC
                 INNER JOIN PARCERIA
                    ON PARC.NRPARCNEGOCIO = PARCERIA.NRPARCNEGOCIO
                   AND PARC.NRORG         = PARCERIA.NRORG
                 INNER JOIN VENDEDOR VEN
                    ON PARC.NRPARCNEGOCIO = VEN.NRPARCNEGOCIO
                   AND PARC.NRORG         = VEN.NRORG
                 WHERE PARCERIA.CDTIPOPARCEIRO = 'VENDEDOR'
                   AND PARCERIA.IDATIVO        = 'S'
                   AND PARC.IDATIVO            = 'S'
                   AND PARC.NRPARCNEGOCIO      = :NRPARCNEGOCIO
                   AND PARC.NRORG              = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
   
    /**
     * Retorna o parametro que define se a medida do elemento está fixada ou não
     * juntamente com o elemento raiz
    **/
    /*public function getElementosIdMedidaFixada($idelemento, $nrorg) {
        $sql = "SELECT DISTINCT SERV.IDELEMENTO, 
                       RELAC.CDSERVORCARELAC, 
                       RELAC.CDTPMEDIDASERV AS CDMEDVARIAVELRELAC,
                       RELAC.QTMEDIDASERV AS QTMEDSERVRELAC,
                       TPMEDIDA.DSTPMEDIDASERV AS DSTPMEDIDASERVRELAC,
                       CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTORAIZ
                  FROM FCT_SERVICOORCA SERV
                  LEFT JOIN FCT_RELACSERVICO RELAC
                    ON SERV.CDSERVORCA = RELAC.CDSERVORCA
                   AND SERV.NRORG      = RELAC.NRORG
                  LEFT JOIN FCT_TIPOMEDIDASERV TPMEDIDA
                    ON RELAC.CDTPMEDIDASERV = TPMEDIDA.CDTPMEDIDASERV
                   AND RELAC.NRORG          = TPMEDIDA.NRORG
                 WHERE SERV.NRORG           = $nrorg
                   AND RELAC.IDMEDIDAFIXA   = 'S'
                   AND CDSERVORCARELAC IN (SELECT DISTINCT CDSERVORCARELAC
                                             FROM FCT_SERVICOORCA ELEMENTO
                                             LEFT JOIN FCT_RELACSERVICO SERVRELAC
                                               ON ELEMENTO.CDSERVORCA = SERVRELAC.CDSERVORCA
                                              AND ELEMENTO.NRORG      = SERVRELAC.NRORG
                                            WHERE SERVRELAC.IDMEDIDAFIXA = 'S'
                                          CONNECT BY PRIOR ELEMENTO.IDELEMENTO = ELEMENTO.IDELEMENTOPAI
                                            START WITH ELEMENTO.IDELEMENTO IN ('$idelemento'))
                CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                  START WITH IDELEMENTO IN ('$idelemento')";
        
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }*/

    /**
     * Retorna o parametro que define se a medida do elemento está fixada ou não
     * juntamente com o elemento raiz
    **/
    /*public function getElementosIdMedidaFixadaTela($idelemento, $nrorg){
        $sql = "SELECT CDSERVORCARELAC, 
                  CDMEDVARIAVELRELAC,
                  SUM(QTMEDSERVRELAC)AS QTMEDSERVRELAC,
                  DSTPMEDIDASERVRELAC,
                  IDELEMENTORAIZ
                FROM (SELECT DISTINCT SERV.IDELEMENTO, 
                          RELAC.CDSERVORCARELAC, 
                          RELAC.CDTPMEDIDASERV AS CDMEDVARIAVELRELAC,
                          RELAC.QTMEDIDASERV AS QTMEDSERVRELAC,
                          TPMEDIDA.DSTPMEDIDASERV AS DSTPMEDIDASERVRELAC,
                          CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTORAIZ
                        FROM FCT_SERVICOORCA SERV
                        LEFT JOIN FCT_RELACSERVICO RELAC
                          ON  SERV.CDSERVORCA = RELAC.CDSERVORCA
                          AND SERV.NRORG      = RELAC.NRORG
                        LEFT JOIN FCT_TIPOMEDIDASERV TPMEDIDA
                          ON RELAC.CDTPMEDIDASERV =  TPMEDIDA.CDTPMEDIDASERV
                          AND RELAC.NRORG =  TPMEDIDA.NRORG
                        WHERE SERV.NRORG = $nrorg
                            AND CDSERVORCARELAC IN ((SELECT DISTINCT CDSERVORCARELAC
                                                      FROM FCT_SERVICOORCA ELEMENTO
                                                      LEFT JOIN FCT_RELACSERVICO SERVRELAC
                                                        ON ELEMENTO.CDSERVORCA = SERVRELAC.CDSERVORCA
                                                        AND ELEMENTO.NRORG     = SERVRELAC.NRORG
                                                      WHERE IDMEDIDAFIXA = 'S'
                                                      CONNECT BY PRIOR ELEMENTO.IDELEMENTO = ELEMENTO.IDELEMENTOPAI
                                                      START WITH ELEMENTO.IDELEMENTO IN('$idelemento')))
                        CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                        START WITH IDELEMENTO IN('$idelemento'))
                GROUP BY CDSERVORCARELAC, 
                  CDMEDVARIAVELRELAC,
                  DSTPMEDIDASERVRELAC,
                  IDELEMENTORAIZ";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }*/
   
    /**
     * Retorna o parametro que define se a medida do elemento está fixada ou não
     * juntamente com o elemento raiz
    **/
    public function getIdUsaMedidaFixada($idelemento, $nrorg, $cdservorca){
        $sql = "SELECT DISTINCT IDMEDIDAFIXA, 
                    CDSERVORCARELAC,
                    CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTORAIZ
                FROM FCT_SERVICOORCA SERV
                LEFT JOIN FCT_RELACSERVICO RELAC
                  ON  SERV.CDSERVORCA = RELAC.CDSERVORCA
                  AND SERV.NRORG      = RELAC.NRORG
                WHERE SERV.NRORG     = $nrorg
                    AND IDMEDIDAFIXA = 'S'
                    AND CDSERVORCARELAC IN($cdservorca)
                CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                START WITH IDELEMENTO IN('$idelemento')";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
   
    /**
     * Retorna os tipos de medida variáveis relacionados aos elementos informados.
     */
    public function getTiposMedidaVariaveisDosElementos(array $idElemento, $nrtipoorcamento, $nrorg) {
        $condicaoIdelementoIn = Utilities::getCondicaoIn999($idElemento, "IDELEMENTO", true);
        
        $sql = "SELECT MEDIDA.CDTPMEDIDASERV,
                       MEDIDA.DSTPMEDIDASERV,
                       MEDIDA.QUANTIDADE,
                       MEDIDA.QTFRANQUIA,
                       MEDIDA.IDELEMENTO,
                       MEDIDA.IDELEMENTORAIZ,
                       SERV.NMSERVORCA AS NMELEMENTORAIZ
                  FROM (SELECT DISTINCT TIPO.CDTPMEDIDASERV,
                               TIPO.DSTPMEDIDASERV,
                               1 AS QUANTIDADE,
                               0 AS QTFRANQUIA,
                               ELEMENTO.IDELEMENTORAIZ AS IDELEMENTO,
                               ELEMENTO.IDELEMENTORAIZ,
                               ELEMENTO.NRORG
                          FROM (
                            SELECT DISTINCT
                                   CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTORAIZ,
                                   CDTPMEDIDAVARIAVEL,
                                   NRORG
                              FROM FCT_SERVICOORCA
                           CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                             START WITH IDELEMENTO IN (SELECT IDELEMENTO
                                                         FROM FCT_SERVICOORCA
                                                        WHERE $condicaoIdelementoIn
                                                          AND NRORG         = :NRORG
                                                          AND IDELEMENTOPAI IS NULL)) ELEMENTO
                          LEFT JOIN FCT_TIPOMEDIDASERV TIPO
                            ON TIPO.CDTPMEDIDASERV = ELEMENTO.CDTPMEDIDAVARIAVEL
                           AND TIPO.NRORG          = ELEMENTO.NRORG 
                         WHERE TIPO.CDTPMEDIDASERV IS NOT NULL
                           AND TIPO.CDTPMEDIDASERV NOT IN (SELECT DISTINCT CDTPMEDIDAVARIAVEL FROM FCT_VARIACAOPRECBASESERV)
                         UNION ALL
                        SELECT DISTINCT TIPO.CDTPMEDIDASERV,
                               TIPO.DSTPMEDIDASERV,
                               NVL(VAR.QTFRANQUIA, 1) AS QUANTIDADE,
                               NVL(VAR.QTFRANQUIA, 0) AS QTFRANQUIA,
                               ELEMENTO.IDELEMENTORAIZ AS IDELEMENTO,
                               ELEMENTO.IDELEMENTORAIZ,
                               ELEMENTO.NRORG
                          FROM (
                            SELECT DISTINCT
                                   CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTORAIZ,
                                   CDTPMEDIDAVARIAVEL,
                                   NRORG
                              FROM FCT_SERVICOORCA
                           CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                             START WITH IDELEMENTO IN (SELECT IDELEMENTO
                                                         FROM FCT_SERVICOORCA
                                                        WHERE $condicaoIdelementoIn
                                                          AND NRORG         = :NRORG
                                                          AND IDELEMENTOPAI IS NULL)) ELEMENTO
                         INNER JOIN FCT_TIPOMEDIDASERV TIPO
                            ON TIPO.CDTPMEDIDASERV = ELEMENTO.CDTPMEDIDAVARIAVEL
                           AND TIPO.NRORG          = ELEMENTO.NRORG 
                         INNER JOIN FCT_VARIACAOPRECBASESERV VAR
                            ON TIPO.CDTPMEDIDASERV = VAR.CDTPMEDIDAVARIAVEL
                           AND TIPO.NRORG          = VAR.NRORG
                         INNER JOIN FCT_ITPRECBASESERV ITPREC
                            ON VAR.NRSEQITPRECBSERV = ITPREC.NRSEQITPRECBSERV
                           AND VAR.NRORG            = ITPREC.NRORG
                         WHERE TIPO.CDTPMEDIDASERV IS NOT NULL
                           AND ITPREC.CDPRECBASESERV IN (SELECT DISTINCT CDPRECBASESERV FROM FCT_TIPOORCAMENTO WHERE NRTIPOORCAMENTO IN ($nrtipoorcamento))
                           AND ITPREC.CDSERVORCA IN (SELECT DISTINCT CDSERVORCA FROM FCT_ITESCOPOPROPOSTA WHERE NRTIPOORCAMENTO IN ($nrtipoorcamento))) MEDIDA
                  LEFT JOIN FCT_SERVICOORCA SERV
                    ON MEDIDA.IDELEMENTORAIZ = SERV.IDELEMENTO
                   AND MEDIDA.NRORG          = SERV.NRORG";
        
        $params = array("NRORG" => $nrorg);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    

    /**
     * busca o modelo de documento.
     */
    public function getModeloDocumento($params) {
        $sql = "SELECT MODELO.NRSEQMODDOCFCT,
                       MODELO.TXMODELODOCORCA
                  FROM FCT_MODELODOC MODELO
                 INNER JOIN FCT_RELACESCOPOPROPOSTA RELAC
                    ON MODELO.NRSEQMODDOCFCT = RELAC.NRSEQMODDOCFCT
                   AND MODELO.NRORG          = RELAC.NRORG
                 WHERE RELAC.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                   AND RELAC.NRORG             = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }    
    
    /**
     * Insere os modelo de documento nos documentos da proposta.
     */
    public function inserirDocumentoProposta($params) {
        $sql = "INSERT INTO FCT_ORCAMENTODOC(
                    NRSEQORCADOC, NRSEQORCAMENTO, NRSEQMODDOCFCT, TXMODELOORCADOC, IDPADRAOORCADOC,
                    NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO
                )
                VALUES (
                    :NRSEQORCADOC, :NRSEQORCAMENTO, :NRSEQMODDOCFCT, :TXMODELOORCADOC, 'S',
                    :NRORG, 'S', SYSDATE, :NRORG, :OPERADOR
                )";
                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Insere as Composições da Proposta.
     * 
     * @param mixed[] $params
     */
    public function inserirComposicoesProposta(array $params) {
        $sql = "INSERT INTO FCT_COMPSERVORCAMENTO (
                    NRCOMPSERVORCAMENTO,   NRSEQORCAMENTO,   NRCOMPSERVICO,   NMCOMPSERVICO,  NRCOMPSERVICOPAI,
                    IDCOMPOBRIGATORIA,     IDESCOLHIDO,      IDATIVO,         NRORG,
                    CDOPERINCLUSAO,        NRORGINCLUSAO,    DTINCLUSAO
                )
                VALUES (
                    :NRCOMPSERVORCAMENTO,  :NRSEQORCAMENTO,  :NRCOMPSERVICO,  :NMCOMPSERVICO, :NRCOMPSERVICOPAI,
                    :IDCOMPOBRIGATORIA,    :IDESCOLHIDO,     :IDATIVO,        :NRORG,
                    :CDOPERINCLUSAO,       :NRORGINCLUSAO,   SYSDATE
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Atualiza as composições da proposta.
     */
    public function atualizarComposicoesProposta(array $params) {
        $sql = "UPDATE FCT_COMPSERVORCAMENTO
                   SET IDESCOLHIDO  = :IDESCOLHIDO,
                       DTULTATU     = SYSDATE,
                       CDOPERULTATU = :CDOPERULTATU,
                       NRORGULTATU  = :NRORGULTATU
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRCOMPSERVICO  = :NRCOMPSERVICO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getComposicoesProposta(array $params) {
        $sql = "SELECT NRCOMPSERVICO,
                       NMCOMPSERVICO,
                       NRCOMPSERVICOPAI,
                       IDCOMPOBRIGATORIA,
                       IDESCOLHIDO
                  FROM FCT_COMPSERVORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getComposicoesInseridasProposta($nrseqorcamento, $nrorg, $composicao) {
        $sql = "SELECT NRCOMPSERVICO,
                       NMCOMPSERVICO,
                       NRCOMPSERVICOPAI,
                       IDCOMPOBRIGATORIA,
                       IDESCOLHIDO
                  FROM FCT_COMPSERVORCAMENTO
                 WHERE NRSEQORCAMENTO = $nrseqorcamento
                   AND NRORG          = $nrorg
                   AND NRCOMPSERVICO  = $composicao";
        
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    /**
     * Exclui as composições da proposta
     */
    public function excluirComposicoesProposta(array $params) {
        $sql = "DELETE
                  FROM FCT_COMPSERVORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * busca o caminho do modelo do documento anexado na proposta
     */
    public function buscaDocAnexado(array $params) {
       $sql = "SELECT LISTAGG(NRSEQORCADOC, ',') WITHIN GROUP (ORDER BY NRSEQORCADOC) AS NRSEQORCADOC,
                    NMARQUIVO,
                    DSCAMINHOARQ,
                    DSTIPO,
                    VRTAMANHO 
                FROM FCT_ORCAMENTODOC 
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG
                GROUP BY NMARQUIVO,
                    DSCAMINHOARQ,
                    DSTIPO,
                    VRTAMANHO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
        
    }
    
    /**
     * Atualiza o documento anexado na proposta
     */
    public function AnexaOrcamentoDoc(array $params, $nrorcadoc) {
       $sql = "UPDATE FCT_ORCAMENTODOC
                  SET NMARQUIVO    = :NMARQUIVO,
                      DSCAMINHOARQ = :DSCAMINHOARQ,
                      VRTAMANHO    = :VRTAMANHO,
                      DSTIPO       = :DSTIPO,
                      DTULTATU     = SYSDATE,
                      CDOPERULTATU = :CDOPERULTATU,
                      NRORGULTATU  = :NRORGULTATU
                WHERE NRSEQORCADOC   IN($nrorcadoc)
                  AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna os escopos da proposta.
     */
    public function getEscoposProposta(array $params) {
        $sql = "SELECT NRTIPOORCAMENTO
                  FROM FCT_RELACESCOPOPROPOSTA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getDocumentosContratosGerados(array $params) {
        $sql = "SELECT DOC.NRSEQCONTRATO,
                       DOC.NRSEQCONTDOC,
                       DOC.NMARQUIVO,
                       DOC.DSCAMINHOARQ,
                       DOC.DSTIPO,
                       DOC.VRTAMANHO
                  FROM FCT_CONTRATODOC DOC
                 INNER JOIN FCT_REFEORCACONT REFE
                    ON DOC.NRSEQCONTRATO = REFE.NRSEQCONTRATO
                   AND DOC.NRORG         = REFE.NRORG
                 WHERE REFE.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND REFE.NRORG          = :NRORG
                   AND REFE.IDATIVO        = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function atualizarCaminhoDocumentoContrato(array $params) {
        $sql = "UPDATE FCT_CONTRATODOC
                   SET DSCAMINHOARQ = :DSCAMINHOARQ
                 WHERE NRSEQCONTDOC  = :NRSEQCONTDOC
                   AND NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function atualizarCaminhoDocumentoProposta(array $params, $nrorcadoc) {
        $sql = "UPDATE FCT_ORCAMENTODOC
                   SET DSCAMINHOARQ = :DSCAMINHOARQ
                 WHERE NRSEQORCADOC   IN ($nrorcadoc)
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function reverterAceiteProposta(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET DTACEITEORCA        = NULL,
                       IDSITUACAOORCAMENTO = 'CONCLUIDO'
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function excluirEnderecoCliente(array $params) {
        $sql = "DELETE FROM ENDECLIE
                 WHERE CDCLIENTE = (SELECT CDCLIENTE
                                      FROM CLIENTE
                                     WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                                       AND NRORG         = :NRORG)
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function excluirInterlocutorCliente(array $params) {
        $sql = "DELETE FROM INTECLIE
                 WHERE CDCLIENTE = (SELECT CDCLIENTE
                                      FROM CLIENTE
                                     WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                                       AND NRORG         = :NRORG)
                   AND NRORG     = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function excluirParceriaCliente(array $params) {
        $sql = "DELETE FROM PARCERIA
                 WHERE CDTIPOPARCEIRO = 'CLIENTE'
                   AND NRPARCNEGOCIO  = :NRPARCNEGOCIO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function ativarParceriaProspect(array $params) {
        $sql = "UPDATE PARCERIA
                   SET IDATIVO = 'S'
                 WHERE CDTIPOPARCEIRO = 'PROSPECT'
                   AND NRPARCNEGOCIO  = :NRPARCNEGOCIO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function atualizarTipoPrincipalParceiro(array $params) {
        $sql = "UPDATE PARCNEGOCIO
                   SET CDTIPOPARCPRINCIPAL = 'PROSPECT'
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function excluirCliente(array $params) {
        $sql = "DELETE FROM CLIENTE
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna a Url de integração com o crm
    **/
    public function getUrlIntegracaoCrm(array $params){
        $sql = "SELECT URLINTEGRACAOCRM 
                FROM FCT_PARAMETROGERAL
                WHERE NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();        
    }
    
    /**
     * Retorna o elemento que o usuário deseja atualizar o valor
    **/
    public function getElementoAlteradoPeloUsuarioNaComposicaoServico(array $params) {
        $sql = "SELECT NRTRABSERVORCA,
                       QTMEDSERVORCA,
                       VRUNITSERVORCA,
                       VRTOTSAJUSTE
                  FROM FCT_TRABSERVORCAMENTO
                 WHERE NRSEQORCAMENTO  = :NRSEQORCAMENTO
                   AND IDELEMENTOPAI   = :IDELEMENTOPAI
                   AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND CDSERVORCA      = :CDSERVORCA
                   AND NRORG           = :NRORG
                   AND IDELEMENTO      = 'USUARIO'
                   AND (CDTPMEDIDASERV = :CDTPMEDIDASERV OR CDTPMEDIDASERV IS NULL)" ;
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Atualiza o valor do elemento
    **/
    public function atualizarElementoAlteradoPeloUsuarioNaComposicaoServico(array $params, $multiplicaVariacao, $vrpercajuste, $idtpajuste, $oldTotalSajuste, $countElemento) {
        $percdescorca = (float)$vrpercajuste;
        $vrpercajuste = ($idtpajuste == 'P' || $idtpajuste == 'Y') ? (($oldTotalSajuste * (float)$vrpercajuste / 100)/$countElemento) : (float)$vrpercajuste/$countElemento;
        $ajuste  = ($idtpajuste == 'P' || $idtpajuste == 'V') ? "-" : "+";
        $vrtotal = "(VRUNITSERVORCA * :QTMEDSERVORCA)";
        $sql = "UPDATE FCT_TRABSERVORCAMENTO
                SET IDTPDESCORCA   = '$idtpajuste',
                    VRPERCDESCORCA = ROUND($percdescorca/$countElemento, 2),
                    VRTOTSAJUSTE   = ROUND(VRUNITSERVORCA * :QTMEDSERVORCA, 2),
                    VRTOTSERVORCA  = ROUND($vrtotal $ajuste $vrpercajuste, 2),
                    QTMEDSERVORCA  = ROUND(:QTMEDSERVORCA, 2),
                    CDOPERULTATU   = :CDOPERULTATU,
                    DTULTATU       = SYSDATE
                WHERE NRTRABSERVORCA  = :NRTRABSERVORCA
                    AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                    AND CDSERVORCA      = :CDSERVORCA
                    AND NRSEQORCAMENTO  = :NRSEQORCAMENTO
                    AND IDELEMENTOPAI   = :IDELEMENTOPAI
                    AND NRORG           = :NRORG
                    AND IDATIVO         = 'S'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * retorna a quantidade de elementos distintos de uma árvore de elementos
    **/
    public function getCountElementos(array $params){
        $sql = "SELECT COUNT(IDELEMENTO) QTELEMENTOS, 
                  SUM(NVL(VRTOTSAJUSTE, 0)) AS VRTOTSAJUSTE
                FROM (SELECT DISTINCT IDELEMENTO, VRTOTSAJUSTE
                  FROM FCT_TRABSERVORCAMENTO
                  WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                     AND (IDELEMENTOPAI  = :IDELEMENTOPAI
                          OR IDELEMENTOPAI LIKE :IDELEMENTOPAI||'%')
                     AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                     AND CDSERVORCA      = :CDSERVORCA
                     AND NRORG           = :NRORG
                     AND IDESCOLHIDO     = 'S'
                     AND IDATIVO         = 'S'
                     AND (CDTPMEDIDASERV  = :CDTPMEDIDASERV OR CDTPMEDIDASERV IS NULL))";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $quantidade = $std->fetchAll();
        return $quantidade[0];
    }
    
    /**
     * Inativa o elemento após criar uma cópia com o tipo
     * Alterado pelo usuário
    **/
    public function inativaElemento(array $params){
        $sql = "UPDATE FCT_TRABSERVORCAMENTO
                SET IDATIVO = 'N'
                WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                   AND IDELEMENTOPAI   = :IDELEMENTOPAI
                   AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND CDSERVORCA      = :CDSERVORCA
                   AND NRORG           = :NRORG
                   AND (CDTPMEDIDASERV = :CDTPMEDIDASERV OR CDTPMEDIDASERV IS NULL)
                   AND IDELEMENTO      <> 'USUARIO'";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getItemPropostaPorCodServico(array $params) {
        $sql = "SELECT NRITEMORCAMENTO,
                       QTMEDIDAORCA,
                       VRUNITITEMORCA
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO  = :NRSEQORCAMENTO
                   AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND CDSERVORCA      = :CDSERVORCA
                   AND NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getSomatoriaElementosComposicaoPropostaPorElementoRaiz(array $params) {
        $sql = "SELECT MEDIDAS.IDELEMENTORAIZ || '_' || TO_CHAR(MEDIDAS.CDSERVORCA) || '_' || MEDIDAS.NRTIPOORCAMENTO AS IDARVORE,
                  TO_CHAR(MEDIDAS.CDSERVORCA) || '_' || MEDIDAS.NRTIPOORCAMENTO AS IDARVOREPAI,
                  'M'                        AS IDTIPO,
                  MEDIDAS.NRSEQORCAMENTO,
                  NULL                                                                               AS NRITEMORCAMENTO,
                  NULL                                                                               AS CDPRODSERV,
                  NULL                                                                               AS TIPOPRODSERV,
                  MEDIDAS.CDSERVORCA                                                                 AS CDSERVORCA,
                  NULL                                                                               AS CDPRODUTO,
                  DECODE(MEDIDAS.IDELEMENTORAIZ, 'USUARIO', 'Alterado pelo Usuário', ELE.NMSERVORCA) AS NMITEMORCAMENTO,
                  1                                                                                  AS QTITEMORCAMENTO,
                  MEDIDAS.CDTPMEDIDASERV,
                  MED.DSTPMEDIDASERV,
                  MEDIDAS.QTMEDSERVORCA  AS QTMEDIDAORCA,
                  'S'                    AS IDMULTMEDVRTOT,
                  MEDIDAS.VRUNITSERVORCA AS VRUNITITEMORCA,
                  NULL                   AS IDTPDESCORCA,
                  NULL                   AS VRPERCDESCORCA,
                  NULL                   AS VRITEMSEMDESC,
                  MEDIDAS.VRTOTSERVORCA  AS VRTOTALITEMORCA,
                  NULL                   AS IDFATURAORCA,
                  MEDIDAS.NRTIPOORCAMENTO,
                  ESCOPO.NMTIPOORCAMENTO,
                  NULL AS CDMODPGTO,
                  NULL AS DSMODPGTO,
                  NULL AS QTTEMPOLOCACAO,
                  NULL AS TXFORMULAITORCA,
                  NULL AS NMREDUITEMORCA,
                  NULL AS NMMNEMOITEMORCA,
                  NULL AS DSQTITEMORCA,
                  NULL AS DSQTITEMORCAADIC,
                  NULL AS QTITEMORCAADIC,
                  NULL AS IDTRANSPORTADORA,
                  NULL AS QTDIASCONTRATO,
                  NULL AS IDITEMOBRIGORCA,
                  NULL AS IDSOMASUBITENS,
                  NULL AS IDSOMANOITEMPAI,
                  NULL AS DSAGRUPAITEMORCA,
                  NULL AS NRORG,
                  NULL AS IDATIVO,
                  NULL AS DTINCLUSAO,
                  NULL AS DTULTATU,
                  NULL AS NRORGINCLUSAO,
                  NULL AS CDOPERINCLUSAO,
                  NULL AS NMOPERINCLUSAO,
                  NULL AS NRORGULTATU,
                  NULL AS CDOPERULTATU,
                  NULL AS NMOPERULTATU,
                  NULL AS NRPARCFORNECEDOR,
                  NULL AS IDSOMANOTOTALORCA,
                  NULL AS IDUNIDTEMPOLOCACAO,
                  NULL AS IDPRECOLOCACAO,
                  NULL AS IDADICCOBRITEMORC,
                  'N'  AS IDADICITEMORC,
                  NULL AS IDBLOQALTORC,
                  NULL AS NRGRUPOFATURA,
                  NULL AS NMGRUPOFATURA,
                  NULL AS DTINICFATORC,
                  NULL AS IDIMPRESSAOORCA,
                  NULL AS NRITEMORCAMENTOPAI,
                  NULL AS SGUNIDADEADICIONAL,
                  NULL AS CDBEMATIVO,
                  NULL AS CDEMPRESA,
                  NULL AS SGUNIDADE,
                  NULL AS QTPARCELASFATU,
                  NULL AS NRDIAVENCPARCELASFATU
                FROM
                  (SELECT ELEMENTOS.NRSEQORCAMENTO,
                    ELEMENTOS.NRTIPOORCAMENTO,
                    ELEMENTOS.CDSERVORCA,
                    ELEMENTOS.QTSERVORCA,
                    ELEMENTOS.CDTPMEDIDASERV,
                    SUM(ELEMENTOS.QTMEDSERVORCA) AS QTMEDSERVORCA,
                    ELEMENTOS.VRUNITSERVORCA,
                    SUM(ELEMENTOS.VRTOTSERVORCA) AS VRTOTSERVORCA,
                    ELEMENTOS.IDELEMENTORAIZ,
                    ELEMENTOS.NRORG
                  FROM
                    (SELECT TRAB.NRSEQORCAMENTO,
                      TRAB.NRTIPOORCAMENTO,
                      TRAB.CDSERVORCA,
                      TRAB.QTSERVORCA,
                      TRAB.CDTPMEDIDASERV,
                      TRAB.QTMEDSERVORCA,
                      TRAB.VRUNITSERVORCA,
                      TRAB.VRTOTSERVORCA,
                      CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTORAIZ,
                      TRAB.NRORG
                    FROM
                      ( SELECT DISTINCT NRSEQORCAMENTO,
                        NRTIPOORCAMENTO,
                        IDESCOLHIDO,
                        CDSERVORCA,
                        QTSERVORCA,
                        CDTPMEDIDASERV,
                        QTMEDSERVORCA,
                        NVL(QTMEDIDAVARIAVEL, DECODE(VRUNITSERVORCA, NULL, NULL, 1)) QTMEDIDAVARIAVEL,
                        VRUNITSERVORCA,
                        VRTOTSERVORCA,
                        IDELEMENTO,
                        IDELEMENTOPAI,
                        NRORG
                      FROM FCT_TRABSERVORCAMENTO
                      WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                      AND NRORG            = :NRORG
                      AND IDESCOLHIDO      = 'S'
                      ) TRAB
                    WHERE CDSERVORCA             IS NOT NULL
                    AND VRUNITSERVORCA           IS NOT NULL
                      CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                      START WITH IDELEMENTO      IN
                      (SELECT DISTINCT IDELEMENTO
                      FROM FCT_TRABSERVORCAMENTO
                      WHERE NRSEQORCAMENTO = TRAB.NRSEQORCAMENTO
                      AND NRORG            = TRAB.NRORG
                      AND IDESCOLHIDO      = 'S'
                      AND IDELEMENTOPAI   IS NULL
                      )
                    ) ELEMENTOS
                  INNER JOIN FCT_SERVICOORCA
                  ON ELEMENTOS.CDSERVORCA = FCT_SERVICOORCA.CDSERVORCA
                  AND ELEMENTOS.NRORG     = FCT_SERVICOORCA.NRORG
                  GROUP BY ELEMENTOS.NRSEQORCAMENTO,
                    ELEMENTOS.NRTIPOORCAMENTO,
                    ELEMENTOS.CDSERVORCA,
                    ELEMENTOS.QTSERVORCA,
                    ELEMENTOS.CDTPMEDIDASERV,
                    ELEMENTOS.VRUNITSERVORCA,
                    ELEMENTOS.IDELEMENTORAIZ,
                    ELEMENTOS.NRORG
                  ORDER BY ELEMENTOS.IDELEMENTORAIZ,
                    ELEMENTOS.CDSERVORCA
                  ) MEDIDAS
                LEFT JOIN FCT_TIPOMEDIDASERV MED
                ON MEDIDAS.CDTPMEDIDASERV = MED.CDTPMEDIDASERV
                AND MEDIDAS.NRORG         = MED.NRORG
                LEFT JOIN FCT_SERVICOORCA ELE
                ON MEDIDAS.IDELEMENTORAIZ = ELE.IDELEMENTO
                AND MEDIDAS.NRORG         = ELE.NRORG
                LEFT JOIN FCT_TIPOORCAMENTO ESCOPO
                ON MEDIDAS.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                AND MEDIDAS.NRORG          = ESCOPO.NRORG
                WHERE MEDIDAS.CDSERVORCA   = :CDSERVORCA";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function excluirElementoAlteradoPeloUsuarioNaComposicaoServico(array $params) {
        $sql = "DELETE FROM FCT_TRABSERVORCAMENTO
                 WHERE NRSEQORCAMENTO  = :NRSEQORCAMENTO
                   AND IDELEMENTOPAI   = :IDELEMENTOPAI
                   AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND CDSERVORCA      = :CDSERVORCA
                   AND NRTRABSERVORCA  = :NRTRABSERVORCA
                   AND NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getElementosAlteradosPeloUsuarioNaComposicaoServico(array $params) {
        $sql = "SELECT NRTIPOORCAMENTO,
                       NRCOMPSERVICO,
                       NRITCOMPSERVICO,
                       IDESCOLHIDO,
                       CDSERVORCA,
                       QTSERVORCA,
                       CDTPMEDIDASERV,
                       QTMEDSERVORCA,
                       VRUNITSERVORCA,
                       VRTOTSERVORCA,
                       NRORG,
                       IDATIVO,
                       IDSERVOBRIGCOMP,
                       IDELEMENTO,
                       IDELEMENTOPAI,
                       IDTIPOELEMENTO,
                       QTMEDIDAVARIAVEL
                  FROM FCT_TRABSERVORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG
                   AND IDELEMENTO     = 'USUARIO'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function atualizarCaminhoDocumentoGeradoPDFProposta(array $params) {
        $sql = "UPDATE FCT_ORCAMENTODOC
                   SET DSCAMINHODOCGERADOPDF = :DSCAMINHODOCGERADOPDF
                 WHERE NRSEQORCADOC          = :NRSEQORCADOC
                   AND NRORG                 = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getCaminhoDocumentoGeradoPDFProposta(array $params) {
        $sql = "SELECT DSCAMINHODOCGERADOPDF
                  FROM FCT_ORCAMENTODOC
                 WHERE NRSEQORCADOC = :NRSEQORCADOC
                   AND NRORG        = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Busca o código da filial do primeiro escopo 
    **/
    public function buscaFilialEscopoDocumento(array $params, $nrtipoorcamento){
        $sql = "SELECT DISTINCT CDFILIAL
                FROM FCT_RELACESCOPOPROPOSTA
                WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND NRTIPOORCAMENTO  IN (". $nrtipoorcamento .")
                    AND NRORG            = :NRORG";
       
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Altera o operador responsável de N propostas
    **/
    /*public function salvaAlteracaoMassaOper(array $params, $nrseqorcamento){
        $sql = "UPDATE FCT_ORCAMENTO
                SET CDOPERRESPONSAVEL = :CDOPERRESPONSAVEL
                WHERE NRSEQORCAMENTO IN (".$nrseqorcamento.")
                    AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }*/
    
    /**
     * retorna o código do prospect a partir do parceiro de negócio
    **/
    public function getProspectParceiro(array $params){
        $sql = "SELECT CDPROSPECT, CDCLIENTE FROM PROSPECT 
                INNER JOIN PARCNEGOCIO 
                    ON PROSPECT.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                    AND PROSPECT.NRORG        = PARCNEGOCIO.NRORG
                WHERE PARCNEGOCIO.NRPARCNEGOCIO = :NRPARCNEGOCIO
                    AND PARCNEGOCIO.NRORG       = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * gera um contato a para uma proposta que não está ligada a uma oportunidade
    **/
    public function geraContato(array $params){
        $sql = "INSERT INTO CONTATO (CDFILIAL,           NRCONTATO,      CDCLIENTE,      DTHRCONTATO,
                    DSREFECONT,         IDSTATUSCONT,   CDPROSPECT,     IDTPCONTATO, 
                    NRORG,              DTINCLUSAO,     NRORGINCLUSAO,  CDOPERINCLUSAO, 
                    NRPARCNEGOCIO,      IDATIVO,        NRSEQCONTATO,   IDGERAFATURA, 
                    NRPARCINTERLOCUTOR, IDTIPOCONTATO,  NRPRIORIDADE,   IDCANCELADO, 
                    IDNOVOTICKET,       IDORIGEMTICKET, IDARQUIVADO )
                VALUES(:CDFILIAL,             :NRSEQCONTATO,      :CDCLIENTE,  SYSDATE,       
                        :DSREFECONT,           'A',            :CDPROSPECT,    'C',
                        :NRORG,                SYSDATE,        :NRORG,         :CDOPERADOR,
                        :NRPARCNEGOCIO,       'S',            :NRSEQCONTATO,   'NAO',
                        :NRPARCINTERLOCUTOR,  'OPORTUNIDADE', 1,               'N',
                        'N',                  'SM',           'N' )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * altera a tabela de proposta com o código do novo contato gerado
    **/
    public function alteraContatoProposta(array $params){
        $sql = "UPDATE FCT_ORCAMENTO
                SET NRCONTATO     = :NRCONTATO,
                    NRSEQCONTATO  = :NRCONTATO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG        = :NRORG";
        
        $std = $this->connection->prepare($sql); 
        $std->execute($params);
    }
    
    /**
     * Retorna o código da proposta
     */
    public function getCodigoOrcamento(array $params) {
        $sql = "SELECT NRORCAMENTO
                  FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * deleta um contato gerado para uma proposta não ligada a uma oportunidade
     */
    public function deletaContatoProposta(array $params) {
        $sql = "DELETE FROM CONTATO 
                    WHERE NRSEQCONTATO     = :NRSEQCONTATO
                        AND NRORG          = :NRORG
                        AND IDORIGEMTICKET = 'SM'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Inativa um contato
    **/
    public function inativaContato(array $params){
        $sql = "UPDATE CONTATO
                SET IDATIVO = 'N'
                WHERE NRSEQCONTATO     = :NRSEQCONTATO
                    AND NRORG          = :NRORG
                    AND IDORIGEMTICKET = 'SM'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Inativa as evoluções de um contato
    **/
    public function inativaEvolCont(array $params){
        $sql = "UPDATE EVOLCONT
                SET IDATIVO = 'N'
                WHERE NRSEQCONTATO     = :NRSEQCONTATO
                    AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Inativa uma proposta
    **/
    public function inativaProposta(array $params){
        $sql = "UPDATE FCT_ORCAMENTO
                SET IDATIVO = 'N'
                WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND NRORG            = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getIdConverteProspectEmCliente(array $params) {
        $sql = "SELECT IDCONVCLIENTE
                  FROM FCT_PARAMETROGERAL
                 WHERE NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getCountClienteProspect(array $params) {
        $sql = "SELECT COUNT(*) AS QTDE
                  FROM CLIENTE
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Retorna o numero dos contratos e  
     **/
    public function getItemContratos(array $params){
        $sql = "SELECT ITCON.NRITEMOCONTRATO, 
                    ITCON.NRSEQCONTRATO
                FROM FCT_ITEMCONTRATO ITCON
                INNER JOIN FCT_REFEORCACONT REF
                    ON REF.NRSEQCONTRATO = ITCON.NRSEQCONTRATO
                    AND REF.NRORG        = ITCON.NRORG
                WHERE REF.NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND ITCON.CDSERVORCA = :CDSERVORCA
                    AND ITCON.NRORG      = :NRORG
                    AND REF.IDATIVO     = 'S'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }


    public function getPerfisParamModelo(array $params, $perfis){
        $sql = "SELECT DISTINCT NRORCAMENTOMODELO AS MODELO FROM FCT_RELORCAMODPERFACESS
                 WHERE NRPERFILACESSO NOT IN ($perfis)
                   AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getPerfisParamContrato(array $params, $perfis){
        $sql = "SELECT DISTINCT NRSEQMODCONTRA AS MODELO FROM FCT_RELMODCONTPERFACESS
                 WHERE NRPERFILACESSO NOT IN ($perfis)
                  AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getPerfisOperador(array $params){
        $sql = "SELECT NRPERFILACESSO FROM PERFACESSPARC PERF
                  WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                    AND  NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Retorna o parametro de autorização de 
     * geração de documentos em DOCX
    **/
    public function getParametroExtensaoDoc(array $params){
      $sql = "SELECT IDPERMITEDOCX, 
                MODELO.NRTIPOORCAMENTO
              FROM FCT_ORCAMENTO ORC
              LEFT JOIN FCT_RELACESCOPOMODELO MODELO
                ON ORC.NRORCAMENTOMODELO = MODELO.NRORCAMENTOMODELO
                --AND ORC.NRTIPOORCAMENTO   = MODELO.NRTIPOORCAMENTO
                AND ORC.NRORG             = MODELO.NRORG
              LEFT JOIN FCT_ORCAESCOPODOC ORCDOC
                ON MODELO.NRRELACESCOPOMODELO = ORCDOC.NRRELACESCOPOMODELO
                AND MODELO.NRORG              = ORCDOC.NRORG
              WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                AND ORC.NRORG      = :NRORG
                AND (ORCDOC.IDSITUACAOORCAMENTO = ORC.IDSITUACAOORCAMENTO
                OR ORCDOC.IDSITUACAOORCAMENTO  IS NULL)";
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      return $std->fetchAll();
    }
    
    /* Atualiza a conexão do banco de acordo com o connection enviado.
       Atualmente  utilizado para funcionamento da rotina 
       "atualizarMedVariavelOM" do service "Sm00012000MedicaoFaturamentoService"
    */
    public function atualizaConnection($connection){
        $this->connection = $connection;
    }
    
    /**
     * Retorna os dados do documento da proposta de acordo com a situação
    **/
    public function getDadosDocumentoSituacao(array $params){
        $sql = "SELECT ESCOPODOC.NRSEQMODDOCFCT,
                     DOC.NMARQUIVO,
                     DOC.DSCAMINHOARQ,
                     DOC.DSTIPO,
                     DOC.VRTAMANHO,
                     REL.NRTIPOORCAMENTO,
                     ESCOPODOC.IDSITUACAOORCAMENTO
                FROM FCT_ORCAESCOPODOC ESCOPODOC
                INNER JOIN FCT_MODELODOC DOC
                  ON ESCOPODOC.NRSEQMODDOCFCT  = DOC.NRSEQMODDOCFCT
                  AND ESCOPODOC.NRORG          = DOC.NRORG
                INNER JOIN FCT_RELACESCOPOMODELO REL
                  ON REL.NRRELACESCOPOMODELO = ESCOPODOC.NRRELACESCOPOMODELO
                  AND REL.NRORG              = ESCOPODOC.NRORG
                WHERE REL.NRORCAMENTOMODELO = (SELECT NRORCAMENTOMODELO FROM FCT_ORCAMENTO WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO)
                  AND REL.NRORG             = :NRORG
                  AND (IDSITUACAOORCAMENTO  = :IDSITUACAOORCAMENTO
                        OR (IDSITUACAOORCAMENTO IS NULL AND (SELECT COUNT(IDSITUACAOORCAMENTO)
                                                              FROM FCT_ORCAESCOPODOC
                                                              WHERE IDSITUACAOORCAMENTO   = :IDSITUACAOORCAMENTO
                                                                AND NRRELACESCOPOMODELO   = ESCOPODOC.NRRELACESCOPOMODELO) = 0))";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Atualiza a tabela de documento da proposta quando a situação da mesma 
     * muda 
    **/
    public function atualizaDocumentoSituacao(array $params){
        $sql = "UPDATE FCT_ORCAMENTODOC
                SET NRSEQMODDOCFCT 	    = :NRSEQMODDOCFCT,
                    NMARQUIVO      	    = :NMARQUIVO,
                    DSCAMINHOARQ   	    = :DSCAMINHOARQ,
                    DSTIPO         	    = :DSTIPO,
                    VRTAMANHO           = :VRTAMANHO,
                    IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                  AND NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                  AND NRORG             = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Atualiza os operadores responáveis da proposta através da ação
     * atualizar op. responsáveis das telas de contrato e prospect
    **/
    public function atualizaOperadorResponsavel(array $params, $query){
        $sql = "UPDATE FCT_ORCAMENTO
                  SET CDOPERRESPONSAVEL = :NEWCDOPERADOR
                WHERE $query
                  AND NRORG             = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function getCapaFaturamento(array $params) {
        $sql = "SELECT NRCAPAPARAMFATESC, NRTIPOORCAMENTO, NRGRUPOFATURA, CDPRODFATURA, CDFILIALFATURA,
                       NRPARCFATURA, IDTPFATURA, QTPARCELASFATU, IDTPCALCPARCINC, IDGERARATEIO,
                       IDAPURAAPONTAMENTO
                  FROM FCT_CAPAPARAMFATESC
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG = :NRORG ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function setCapaFaturamento(array $params){
        $sql = "INSERT INTO FCT_CAPAPARAMFATPROP 
                    (NRCAPAPARAMFATPROP, NRSEQORCAMENTO, NRGRUPOFATURA, CDPRODFATURA, CDFILIALFATURA,
                     NRPARCFATURA, IDTPFATURA, QTPARCELASFATU, IDTPCALCPARCINC, IDGERARATEIO,
                     IDAPURAAPONTAMENTO, IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                VALUES
                    (:NRCAPAPARAMFATPROP, :NRSEQORCAMENTO, :NRGRUPOFATURA, :CDPRODFATURA, :CDFILIALFATURA,
                     :NRPARCFATURA, :IDTPFATURA, :QTPARCELASFATU, :IDTPCALCPARCINC, :IDGERARATEIO,
                     :IDAPURAAPONTAMENTO, 'S', :NRORG, SYSDATE, :NRORG, :OPERADOR)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function getDadosFaturamento(array $params) {
        $sql = "SELECT NRPARAMFATESC, NRTIPOORCAMENTO, NRDIACOMPETINI, NRDIACOMPETFIN, IDMESCOMPETINI,
                       IDMESCOMPETFIN, QTDIASAPURACAO, IDTPCALCVENC, NRDIACALCVENC, IDMESCALCVENC,
                       QTDIASFATUVENC
                  FROM FCT_PARAMFATESC
                  WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Retorna se um elemento está inserido na tabela fct_trabservorcamento
    **/
    public function getElementosProposta($nrseqorcamento, $nrorg, $idelemento, $nrcompservico){
        $sql = "SELECT * 
                FROM FCT_TRABSERVORCAMENTO
                WHERE NRSEQORCAMENTO  = $nrseqorcamento
                    AND NRORG         = $nrorg
                    AND IDELEMENTO    = '$idelemento'
                    AND NRCOMPSERVICO = $nrcompservico"; 
                    
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    
    public function setDadosFaturamento(array $params){
        $sql = "INSERT INTO FCT_PARAMFATPROP 
                    (NRPARAMFATPROP, NRSEQORCAMENTO, NRDIACOMPETINI, NRDIACOMPETFIN, IDMESCOMPETINI,
                     IDMESCOMPETFIN, QTDIASAPURACAO, IDTPCALCVENC, NRDIACALCVENC, IDMESCALCVENC,
                     QTDIASFATUVENC, IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                VALUES
                    (:NRPARAMFATPROP, :NRSEQORCAMENTO, :NRDIACOMPETINI, :NRDIACOMPETFIN, :IDMESCOMPETINI,
                     :IDMESCOMPETFIN, :QTDIASAPURACAO, :IDTPCALCVENC, :NRDIACALCVENC, :IDMESCALCVENC,
                     :QTDIASFATUVENC, 'S', :NRORG, SYSDATE, :NRORG, :OPERADOR)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }    
    

    public function getDadosItFaturamento(array $params) {
        $sql = "SELECT NRSEQFATUSRVESCO, NRITESCOPOPROPOSTA , NRTIPOORCAMENTO, CDPRODFATURA, CDFILIALFATURA, 
                       NRGRUPOFATURA, NRPARCFATURA, IDFATURACONTR, CDMODPGTO, IDBLOQVRITMEDIC
                       QTPARCELASFATU, NRDIAVENCPARCELASFATU
                  FROM FCT_PARAMFATSRVESCO
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRITESCOPOPROPOSTA = :NRITESCOPOPROPOSTA
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    } 
    
    
    public function setDadosItFaturamento(array $params){
        $sql = "INSERT INTO FCT_PARAMFATITPROP 
                    (NRPARAMFATITPROP, NRITEMORCAMENTO, NRSEQORCAMENTO, NRGRUPOFATURA, CDPRODFATURA,
                     CDFILIALFATURA, NRPARCFATURA, IDTPFATURA, QTPARCELASFATU, IDATIVO,
                     NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                VALUES
                    (:NRPARAMFATITPROP, :NRITEMORCAMENTO, :NRSEQORCAMENTO, :NRGRUPOFATURA, :CDPRODFATURA,
                     :CDFILIALFATURA, :NRPARCFATURA, :IDTPFATURA, :QTPARCELASFATU, 'S',
                     :NRORG, SYSDATE, :NRORG, :OPERADOR)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    } 
    
    
    public function getServicosEscopo(array $params) {
        $sql = "SELECT NRITESCOPOPROPOSTA, CDSERVORCA 
                  FROM FCT_ITESCOPOPROPOSTA
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    } 
    
    
    public function getItProposta(array $params) {
        $sql = "SELECT NRITEMORCAMENTO, NMITEMORCAMENTO
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND CDSERVORCA = :CDSERVORCA
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
   
    /**
     * Retorna as informações da variação de preço ligada a proposta
    **/
    public function getInformacoesVariacoesPreco(array $params, $nrtipoorcamento){
        $sql = "SELECT DISTINCT DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.DTINIPBSERV, NVL(VAR.DTINIPBSERV, ITVAR.DTINIPBSERV)) AS DTINIPBSERV,
                    DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.CDTPMEDIDASERV,          NVL(VAR.CDTPMEDIDASERV, ITVAR.CDTPMEDIDASERV)) AS CDTPMEDIDASERV,
                    DECODE(RELACSERV.IDMEDIDAFIXA, 'S', NULL,                          VAR.CDTPMEDIDAVARIAVEL) AS CDTPMEDIDAVARIAVEL,
                    DECODE(RELACSERV.IDMEDIDAFIXA, 'S', 0,                             NVL(VAR.QTMEDIDAINIVARIACAO, 0)) AS QTMEDIDAINIVARIACAO,
                    DECODE(RELACSERV.IDMEDIDAFIXA, 'S', 99999999,                      NVL(VAR.QTMEDIDAFIMVARIACAO, 99999999)) AS QTMEDIDAFIMVARIACAO,
                    DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.VRPRECO,                 NVL(VAR.VRPRECOVARIACAO, ITVAR.VRPRECO)) AS VRPRECOVARIACAO,
                    DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.DTFIMVALPBSERV,          NVL(VAR.DTFIMVALPBSERV, ITVAR.DTFIMVALPBSERV)) AS DTFIMVALPBSERV,
                    NVL(DECODE(RELACSERV.IDMEDIDAFIXA, 'S', NULL,                      VAR.IDMULTIPVARIACAO), 'N') AS IDMULTIPVARIACAO,
                    TRAB.CDSERVORCA, VAR.QTFRANQUIA
                FROM FCT_TRABSERVORCAMENTO TRAB
                INNER JOIN FCT_ITPRECBASESERV ITVAR
                  ON TRAB.CDSERVORCA       = ITVAR.CDSERVORCA
                  AND TRAB.NRORG           = ITVAR.NRORG
                INNER JOIN FCT_TIPOORCAMENTO TIPO
                  ON ITVAR.CDPRECBASESERV = TIPO.CDPRECBASESERV
                  AND ITVAR.NRORG         = TIPO.NRORG
                INNER JOIN FCT_ITESCOPOPROPOSTA ITESC
                  ON TIPO.NRTIPOORCAMENTO = ITESC.NRTIPOORCAMENTO
                  AND TIPO.NRORG = ITESC.NRORG
                  AND TRAB.CDSERVORCA = ITESC.CDSERVORCA
                LEFT JOIN FCT_RELACMEDIDAPROPOSTA REL
                  ON TRAB.NRSEQORCAMENTO  = REL.NRSEQORCAMENTO
                  AND TRAB.NRORG          = REL.NRORG
                LEFT JOIN FCT_VARIACAOPRECBASESERV VAR
                  ON ITVAR.NRSEQITPRECBSERV  = VAR.NRSEQITPRECBSERV
                  AND ITVAR.NRORG            = VAR.NRORG
                  AND REL.CDTPMEDIDAVARIAVEL = VAR.CDTPMEDIDAVARIAVEL
                INNER JOIN FCT_RELACSERVICO RELACSERV
                  ON ITVAR.CDSERVORCA = RELACSERV.CDSERVORCARELAC
                  AND ITVAR.NRORG     = RELACSERV.NRORG
                WHERE TRAB.NRSEQORCAMENTO  = :NRSEQORCAMENTO
                  AND TRAB.NRORG           = :NRORG
                  AND TIPO.NRTIPOORCAMENTO IN ($nrtipoorcamento)
                ORDER BY CDSERVORCA";
        
        /*
        AND REL.IDELEMENTORAIZ IN (SELECT DISTINCT IDELEMENTO
                                               FROM FCT_TRABSERVORCAMENTO 
                                              WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                                AND NRORG          = :NRORG
                                                AND IDATIVO        = 'S'
                                                AND IDESCOLHIDO    = 'S'
                                                AND IDELEMENTOPAI IS NULL)
        */
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
   
    /**
     * Salva as informações das variações de preço ligadas a um orcamento 
    **/
    public function salvarVarPrecoOrcaConcluida(array $params){
        $sql = "INSERT INTO FCT_VARPRECBASESERVORCA (
                    NRVARPRECBASESERVORCA, NRSEQORCAMENTO,      CDSERVORCA,          CDTPMEDIDASERV, 
                    CDTPMEDIDAVARIAVEL,    QTMEDIDAINIVARIACAO, QTMEDIDAFIMVARIACAO, VRPRECOVARIACAO,
                    IDMULTIPVARIACAO,      IDATIVO,             NRORG,               NRORGINCLUSAO, 
                    CDOPERINCLUSAO,        CDOPERULTATU,        NRORGULTATU,         DTINCLUSAO, 
                    DTULTATU,              QTFRANQUIA
                ) VALUES(
                    :NRVARPRECBASESERVORCA, :NRSEQORCAMENTO,                :CDSERVORCA,                    :CDTPMEDIDASERV, 
                    :CDTPMEDIDAVARIAVEL,    ROUND(:QTMEDIDAINIVARIACAO, 2), ROUND(:QTMEDIDAFIMVARIACAO, 2), ROUND(:VRPRECOVARIACAO, 2),
                    :IDMULTIPVARIACAO,      :IDATIVO,                       :NRORG,                         :NRORGINCLUSAO, 
                    :CDOPERINCLUSAO,        NULL,                           NULL,                            SYSDATE, 
                    NULL,                   :QTFRANQUIA
                )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna a variação de preço ligada a proposta no momento em que a validação é executada
     * as tabelas muda de acordo com a ação
    **/
    public function getVariacoesLigadasAoOrcamento(array $params, $cdservorca){
        $sql = "SELECT NVL(QTMEDIDAINIVARIACAO, 1) AS QTMEDIDAINIVARIACAO,
                       NVL(QTMEDIDAFIMVARIACAO, 99999) AS QTMEDIDAFIMVARIACAO,
                       NVL(VRPRECOVARIACAO, ITVAR.VRPRECO) AS VRPRECOVARIACAO,
                       IDMULTIPVARIACAO
                FROM FCT_ITPRECBASESERV ITVAR
                LEFT JOIN FCT_VARIACAOPRECBASESERV VAR
                  ON VAR.NRSEQITPRECBSERV  = ITVAR.NRSEQITPRECBSERV
                  AND VAR.NRORG            = ITVAR.NRORG
                WHERE :IDSITUACAOORCAMENTO IN('ELABORACAO', 'ENVIADO', 'CANCELADO')
                  AND ITVAR.NRORG        = :NRORG
                  AND ITVAR.CDSERVORCA IN($cdservorca)
                  AND (NVL(VAR.IDMULTIPVARIACAO, 'N') = 'N'
                        OR (VAR.IDMULTIPVARIACAO = 'S' 
                            AND (VAR.CDTPMEDIDASERV        = :CDTPMEDIDASERV  
                                 OR VAR.CDTPMEDIDAVARIAVEL = :CDTPMEDIDASERV 
                                 OR ITVAR.CDTPMEDIDASERV   = :CDTPMEDIDASERV)))
                UNION ALL
                SELECT QTMEDIDAINIVARIACAO,
                       QTMEDIDAFIMVARIACAO,
                       VRPRECOVARIACAO,
                       IDMULTIPVARIACAO
                FROM FCT_VARPRECBASESERVORCA
                WHERE :IDSITUACAOORCAMENTO IN('CONCLUIDO', 'ACEITO')
                  AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG
                  AND CDSERVORCA IN($cdservorca)
                  AND (NVL(IDMULTIPVARIACAO, 'N') = 'N'
                        OR (IDMULTIPVARIACAO = 'S' 
                            AND ((CDTPMEDIDASERV          = :CDTPMEDIDASERV 
                                    OR CDTPMEDIDAVARIAVEL = :CDTPMEDIDASERV))))";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    } 
    
    /**
     * Retorna todos os servicos ligados a um orcamento que utilizam variação de medida 
    **/
    public function getServicosRelacionadosOrca($nrseqorcamento, $nrorg){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg 
        );
        
        $sql = "SELECT DISTINCT LISTAGG(CDSERVORCA, ',') WITHIN GROUP (ORDER BY CDSERVORCA) CDSERVORCA
                FROM FCT_ITEMORCAMENTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll()[0]["CDSERVORCA"];
    }
    
    /**
     * Retorna as informações do servico relacionado a proposta durante a conclusao
    **/
    public function getInformacoesServicoMedida(array $params, $nrtipoorcamento){
        $sql = "SELECT DISTINCT RELAC.CDSERVORCA,
                    RELAC.CDSERVORCARELAC,
                    RELAC.CDTPMEDIDASERV,
                    RELAC.QTMEDIDASERV,
                    RELAC.IDMEDIDAFIXA
                  FROM FCT_TRABSERVORCAMENTO TRAB
                  INNER JOIN FCT_ITCOMPSERVICO COMP
                      ON TRAB.NRCOMPSERVICO = COMP.NRCOMPSERVICO
                      AND TRAB.NRORG        = COMP.NRORG
                  INNER JOIN FCT_RELACESCOPOPROPOSTA TIPO
                      ON TRAB.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                      AND TRAB.NRORG          = TIPO.NRORG
                      AND TRAB.NRSEQORCAMENTO = TIPO.NRSEQORCAMENTO
                  INNER JOIN FCT_RELACSERVICO RELAC
                      ON TRAB.CDSERVORCA  = RELAC.CDSERVORCARELAC
                      AND COMP.CDSERVORCA = RELAC.CDSERVORCA
                      AND TRAB.NRORG      = RELAC.NRORG
                  WHERE TRAB.NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND TRAB.NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
   
    /**
     * Salva as informações do servico ligado a proposta no momento em que ela é concluída
    **/
    public function salvarServicoMedidaOrca(array $params){
        $sql = "INSERT INTO FCT_RELACSERVMEDIDAORCA (
                    NRSERVMEDORCA,  NRSEQORCAMENTO, CDSERVORCA,   CDSERVORCARELAC, 
                    CDTPMEDIDASERV, QTMEDIDASERV,   IDMEDIDAFIXA, NRORG, 
                    IDATIVO,        DTINCLUSAO,     DTULTATU,     NRORGINCLUSAO, 
                    CDOPERINCLUSAO
                ) VALUES(
                    :NRSERVMEDORCA,  :NRSEQORCAMENTO,         :CDSERVORCA,   :CDSERVORCARELAC, 
                    :CDTPMEDIDASERV, ROUND(:QTMEDIDASERV, 2), :IDMEDIDAFIXA, :NRORG, 
                    :IDATIVO,        SYSDATE,                 SYSDATE,       :NRORGINCLUSAO, 
                    :CDOPERINCLUSAO
                )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna os tipos de medida não inseridos na proposta
    **/
    /*public function getMedidasNaoInseridasNaProposta($idElementos, array $params){
        $sql = "SELECT DISTINCT SERV.*, 
                    NVL(QTFRANQUIA, 0) AS QTFRANQUIA,
                    NVL(QTFRANQUIA, 1) AS QUANTIDADE
                FROM (SELECT DISTINCT NRORG,
                        CDTPMEDIDAVARIAVEL AS CDTPMEDIDASERV, 
                        CONNECT_BY_ROOT IDELEMENTO AS IDELEMENTORAIZ
                    FROM FCT_SERVICOORCA
                    CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                    START WITH IDELEMENTO IN('$idElementos')) SERV
                INNER JOIN FCT_VARIACAOPRECBASESERV VAR
                    ON SERV.CDTPMEDIDASERV  = VAR.CDTPMEDIDAVARIAVEL
                    AND SERV.NRORG          = VAR.NRORG
                WHERE SERV.CDTPMEDIDASERV NOT IN(SELECT DISTINCT CDTPMEDIDAVARIAVEL 
                                                  FROM FCT_RELACMEDIDAPROPOSTA
                                                  WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                                    AND NRORG          = :NRORG
                                                    AND IDELEMENTORAIZ = SERV.IDELEMENTORAIZ)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }*/

    /**
     * deleta a medida váriavel de um elemento retirado da proposta
    **/
    public function deletarTiposMedidaDaProposta(array $params){
        $sql = "DELETE FROM FCT_RELACMEDIDAPROPOSTA
                WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                    AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Busca os serviços relacionados que não estão inseridos 
     * na tabela de relacionamento com a proposta
    **/
    public function getInfoServicoMedidaPropAceita($composicoes, $nrseqorcamento){
        $sql = "SELECT RELAC.CDSERVORCA,
                  RELAC.CDSERVORCARELAC,
                  RELAC.CDTPMEDIDASERV,
                  RELAC.QTMEDIDASERV,
                  RELAC.IDMEDIDAFIXA
                FROM FCT_ITCOMPSERVICO COMP
                INNER JOIN FCT_RELACSERVICO RELAC
                  ON COMP.CDSERVORCA = RELAC.CDSERVORCA
                  AND COMP.NRORG = RELAC.NRORG
                WHERE NRCOMPSERVICO IN($composicoes)
                  AND RELAC.CDSERVORCA NOT IN (SELECT RELACPROP.CDSERVORCA 
                                                FROM FCT_RELACSERVMEDIDAORCA RELACPROP
                                                WHERE NRSEQORCAMENTO = $nrseqorcamento)";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    /**
     * Busca as variações que não estão inseridos 
     * na tabela de relacionamento com a proposta
    **/
    public function getInfoVariacaoPropAceita($composicoes, $nrseqorcamento, $nrtipoorcamento){
        $sql = "SELECT DISTINCT DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.DTINIPBSERV, NVL(VAR.DTINIPBSERV, ITVAR.DTINIPBSERV)) AS DTINIPBSERV,
                        DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.CDTPMEDIDASERV,          NVL(VAR.CDTPMEDIDASERV, ITVAR.CDTPMEDIDASERV)) AS CDTPMEDIDASERV,
                        DECODE(RELACSERV.IDMEDIDAFIXA, 'S', NULL,                          VAR.CDTPMEDIDAVARIAVEL) AS CDTPMEDIDAVARIAVEL,
                        DECODE(RELACSERV.IDMEDIDAFIXA, 'S', 1,                             NVL(VAR.QTMEDIDAINIVARIACAO, 1)) AS QTMEDIDAINIVARIACAO,
                        DECODE(RELACSERV.IDMEDIDAFIXA, 'S', 99999,                         NVL(VAR.QTMEDIDAFIMVARIACAO, 99999)) AS QTMEDIDAFIMVARIACAO,
                        DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.VRPRECO,                 NVL(VAR.VRPRECOVARIACAO, ITVAR.VRPRECO)) AS VRPRECOVARIACAO,
                        DECODE(RELACSERV.IDMEDIDAFIXA, 'S', ITVAR.DTFIMVALPBSERV,          NVL(VAR.DTFIMVALPBSERV, ITVAR.DTFIMVALPBSERV)) AS DTFIMVALPBSERV,
                        NVL(DECODE(RELACSERV.IDMEDIDAFIXA, 'S', NULL,                      VAR.IDMULTIPVARIACAO), 'N') AS IDMULTIPVARIACAO,
                        RELACSERV.CDSERVORCARELAC AS CDSERVORCA, VAR.QTFRANQUIA
                FROM FCT_ITCOMPSERVICO COMP
                INNER JOIN FCT_RELACSERVICO RELACSERV
                  ON COMP.CDSERVORCA = RELACSERV.CDSERVORCA
                  AND COMP.NRORG     = RELACSERV.NRORG
                INNER JOIN FCT_ITPRECBASESERV ITVAR
                  ON RELACSERV.CDSERVORCARELAC  = ITVAR.CDSERVORCA
                  AND RELACSERV.NRORG           = ITVAR.NRORG
                INNER JOIN FCT_TIPOORCAMENTO TIPO
                  ON ITVAR.CDPRECBASESERV = TIPO.CDPRECBASESERV
                  AND ITVAR.NRORG         = TIPO.NRORG
                LEFT JOIN FCT_RELACMEDIDAPROPOSTA REL
                  ON REL.NRSEQORCAMENTO    = $nrseqorcamento
                  AND ITVAR.NRORG          = REL.NRORG
                INNER JOIN FCT_VARIACAOPRECBASESERV VAR
                  ON ITVAR.NRSEQITPRECBSERV  = VAR.NRSEQITPRECBSERV
                  AND ITVAR.NRORG            = VAR.NRORG
                  AND REL.CDTPMEDIDAVARIAVEL = VAR.CDTPMEDIDAVARIAVEL
                WHERE COMP.NRCOMPSERVICO IN($composicoes)
                  AND TIPO.NRTIPOORCAMENTO IN($nrtipoorcamento)
                  AND REL.CDTPMEDIDAVARIAVEL NOT IN (SELECT DISTINCT NVL(RELACPROP.CDTPMEDIDAVARIAVEL, -1)
                                                       FROM FCT_VARPRECBASESERVORCA RELACPROP
                                                      WHERE NRSEQORCAMENTO = $nrseqorcamento)";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    /**
     * Atualiza o nome contratual do item
    **/
    public function atualizarNomeContratualItens($params){
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET NMCONTRATUALITEMORCA = :NMCONTRATUALITEMORCA
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG        = :NRORG
                    AND IDELEMENTO   = :IDELEMENTO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna o somatorio do elementos de um determinado servico
    **/
    public function getValorTotalItensTipoElemento(array $params){
        $sql = "SELECT SUM(VRTOTALITEMORCA) AS VRTOTALITEMORCA
                FROM FCT_ITEMORCAMENTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND CDSERVORCA   = :CDSERVORCA
                    AND NRORG        = :NRORG
                    AND IDELEMENTO IS NOT NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll()[0]["VRTOTALITEMORCA"];
    }
    /**
     * Altera o valor total do item após as alterações no valor de um elemento
    **/
    public function atualizarValorItemPai(array $params){
        $vrtotservorca = $this->getValorTotalItensTipoElemento($params);
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET VRUNITITEMORCA  = $vrtotservorca,
                    VRTOTALITEMORCA = $vrtotservorca,
                    VRITEMSEMDESC   = $vrtotservorca
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND CDSERVORCA   = :CDSERVORCA
                    AND NRORG        = :NRORG
                    AND IDELEMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna todos os contratos ligados a um orcamento 
    **/
    public function getContratos(array $params){
        $sql = "SELECT REFE.NRSEQCONTRATO,
                    ESCOPO.NRTIPOORCAMENTO
                FROM FCT_REFEORCACONT REFE
                INNER JOIN FCT_RELACESCOPOPROPOSTA ESCOPO
                    ON REFE.NRSEQORCAMENTO = ESCOPO.NRSEQORCAMENTO
                    AND REFE.NRORG         = ESCOPO.NRORG
                WHERE REFE.NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND REFE.NRORG        = :NRORG
                    AND IDATIVO           = 'S'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Deleta os itens do contrato
    **/
    public function deletarItensContrato(array $params){
        $sql = "DELETE FROM FCT_ITEMCONTRATO
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRORG       = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * insere itens no contrato
    **/
    public function inserirItensContrato(array $params){
        $sql = "INSERT INTO FCT_ITEMCONTRATO (
                 NRORG,          NRITEMOCONTRATO,       NRSEQCONTRATO,   CDPRODUTO, 
                 CDSERVORCA,     SGUNIDADE,             NMITEMCONTRATO,  QTITEMCONTRATO, 
                 CDTPMEDIDASERV, QTMEDIDACONTR,         IDUNIDTEMPOSERV, QTTEMPOSERV, 
                 IDITEMATIVO,    VRUNITITEMCONT,        VRTOTITEMCONT,   DTINICFATCONTR, 
                 IDMULTMEDVRTOT, IDADICITEMCONT,        IDADICCOBRCONT,  IDFATURAITEM, 
                 QTPARCELASFATU, NRDIAVENCPARCELASFATU, VRDESCONTO,      IDTPDESCONTO
                ) VALUES (
                 :NRORG,          :NRITEMOCONTRATO,          :NRSEQCONTRATO,           :CDPRODUTO, 
                 :CDSERVORCA,     :SGUNIDADE,                :NMITEMCONTRATO,          ROUND(:QTITEMCONTRATO, 2), 
                 :CDTPMEDIDASERV, ROUND(:QTMEDIDACONTR, 2),  :IDUNIDTEMPOSERV,         :QTTEMPOSERV, 
                 :IDITEMATIVO,    ROUND(:VRUNITITEMCONT, 2), ROUND(:VRTOTITEMCONT, 2), :DTINICFATCONTR, 
                 :IDMULTMEDVRTOT, :IDADICITEMCONT,           :IDADICCOBRCONT,          :IDFATURAITEM, 
                 :QTPARCELASFATU, :NRDIAVENCPARCELASFATU,    ROUND(:VRDESCONTO, 2),    :IDTPDESCONTO
                )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Atualiza a medida nos itens do contrato
    **/
    public function editarMedidaItensCont($paramsSelect, $vrtotal, $qtmedida){
        $sql = "UPDATE FCT_ITEMCONTRATO
                SET QTMEDIDACONTR  = ROUND($qtmedida, 2),
                    VRTOTITEMCONT  = ROUND($vrtotal, 2)
                    VRUNITITEMCONT = ROUND($vrtotal, 2)
                WHERE CDSERVORCA      = :CDSERVORCA
                    AND NRORG         = :NRORG
                    AND NRSEQCONTRATO IN (SELECT NRSEQCONTRATO
                                            FROM FCT_REFEORCACONT
                                            WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO)";
        $std = $this->connection->prepare($sql);
        $std->execute($paramsSelect);
    }

    public function getItensPaiProposta(array $params) {
        $sql = "SELECT NRITEMORCAMENTO, CDSERVORCA, NRTIPOORCAMENTO
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   and NRORG          = :NRORG
                   AND IDELEMENTO     IS NULL";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getParametrosFaturamentoServicoEscopo(array $params) {
        $sql = "SELECT PA.NRGRUPOFATURA,
                       PA.CDPRODFATURA,
                       PA.CDFILIALFATURA,
                       PA.NRPARCFATURA,
                       PA.IDTPFATURA,
                       PA.QTPARCELASFATU
                  FROM FCT_PARAMFATITESC PA
                 INNER JOIN FCT_ITESCOPOPROPOSTA IT
                    ON PA.NRITESCOPOPROPOSTA = IT.NRITESCOPOPROPOSTA
                   AND PA.NRORG              = IT.NRORG
                 WHERE IT.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND IT.CDSERVORCA      = :CDSERVORCA
                   AND IT.NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function inserirParametrosFaturamentoItemProposta(array $params) {
        $sql = "INSERT INTO FCT_PARAMFATITPROP (
                    NRPARAMFATITPROP,    NRITEMORCAMENTO,    NRSEQORCAMENTO,
                    NRGRUPOFATURA,       CDPRODFATURA,       CDFILIALFATURA,
                    NRPARCFATURA,        IDTPFATURA,         QTPARCELASFATU,
                    IDATIVO,             NRORG,              NRORGINCLUSAO,
                    CDOPERINCLUSAO
                ) VALUES (
                    :NRPARAMFATITPROP,   :NRITEMORCAMENTO,   :NRSEQORCAMENTO,
                    :NRGRUPOFATURA,      :CDPRODFATURA,      :CDFILIALFATURA,
                    :NRPARCFATURA,       :IDTPFATURA,        :QTPARCELASFATU,
                    :IDATIVO,            :NRORG,             :NRORGINCLUSAO,
                    :CDOPERINCLUSAO
                )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getParametrosFaturamentoParaItensNegociacaoProposta(array $params, $escopos, $servicos) {
        $sql = "SELECT NRTIPOORCAMENTO,
                       CDSERVORCA,
                       NRITEMORCAMENTO,
                       NRITEMORCAMENTOPAI,
                       NMITEMORCAMENTO,
                       VRTOTALITEMORCA,
                       IDTPFATURA,
                       CDTIPOOPERFATURA,
                       CDSERIENF,
                       CDEMPRESA,
                       CDINSCESTA,
                       CDINSCMUNI,
                       CDPRODFATURA,
                       CDFILIALFATURA,
                       NRPARCFATURA,
                       DECODE(IDTPFATURA, 'P', QTPARCELASFATU, NULL) AS QTPARCELASFATU,
                       DECODE(IDTPFATURA, 'P', QTDIASAPURACAO, NULL) AS QTDIASAPURACAO,
                       DECODE(IDTPFATURA, 'P', NRDIACALCVENC, NULL) AS NRDIACALCVENC,
                       DECODE(IDTPFATURA, 'P', QTDIASFATUVENC, NULL) AS QTDIASFATUVENC,
                       DECODE(IDTPFATURA, 'P', IDTPCALCVENC, NULL) AS IDTPCALCVENC,
                       DECODE(IDTPFATURA, 'P', IDMESCALCVENC, NULL) AS IDMESCALCVENC,
                       DECODE(IDTPFATURA, 'P', 0, (SELECT COUNT(CIT.NRPARAMFATITESCSERV)
                                                     FROM FCT_PARAMFATITESCSERV CIT
                                                    INNER JOIN FCT_ITESCOPOPROPOSTA ITESC
                                                       ON CIT.NRITESCOPOPROPOSTA = ITESC.NRITESCOPOPROPOSTA
                                                      AND CIT.NRORG = ITESC.NRORG
                                                    WHERE ITESC.NRTIPOORCAMENTO = TBL.NRTIPOORCAMENTO
                                                      AND ITESC.CDSERVORCA = TBL.CDSERVORCA)) AS QTCOMPETENCIA
                  FROM (
                    SELECT ITESC.NRTIPOORCAMENTO,
                           ITESC.CDSERVORCA,
                           ITORC.NRITEMORCAMENTO,
                           ITORC.NRITEMORCAMENTOPAI,
                           ITORC.NMITEMORCAMENTO,
                           ITORC.VRTOTALITEMORCA,
                           NVL(PFIT.IDTPFATURA, PFESC.IDTPFATURA) AS IDTPFATURA,
                           NVL(PFIT.CDTIPOOPERFATURA, PFESC.CDTIPOOPERFATURA) AS CDTIPOOPERFATURA,
                           NVL(PFPARCIT.CDSERIENF, NVL(PFPARCESC.CDSERIENF, NVL(PFPARCORC.CDSERIENF, NVL(PFIT.CDSERIENF, PFESC.CDSERIENF)))) AS CDSERIENF,
                           NVL(PFPARCIT.CDEMPRESA, NVL(PFPARCESC.CDEMPRESA, NVL(PFPARCORC.CDEMPRESA, NVL(PFIT.CDEMPRESA, PFESC.CDEMPRESA)))) AS CDEMPRESA,
                           NVL(PFPARCIT.CDINSCESTA, NVL(PFPARCESC.CDINSCESTA, NVL(PFPARCORC.CDINSCESTA, NVL(PFIT.CDINSCESTA, PFESC.CDINSCESTA)))) AS CDINSCESTA,
                           NVL(PFPARCIT.CDINSCMUNI, NVL(PFPARCESC.CDINSCMUNI, NVL(PFPARCORC.CDINSCMUNI, NVL(PFIT.CDINSCMUNI, PFESC.CDINSCMUNI)))) AS CDINSCMUNI,
                           NVL(PFIT.CDPRODFATURA, NVL(PFESC.CDPRODFATURA, SERV.CDPRODFATSERV)) AS CDPRODFATURA,
                           NVL(PFPARCIT.CDFILIALFATURA, NVL(PFPARCESC.CDFILIALFATURA, NVL(PFPARCORC.CDFILIALFATURA, NVL(PFIT.CDFILIALFATURA, NVL(PFESC.CDFILIALFATURA, ESCMOD.CDFILIAL))))) AS CDFILIALFATURA,
                           NVL(PFIT.NRPARCFATURA, NVL(PFESC.NRPARCFATURA, ORC.NRPARCNEGOCIO)) AS NRPARCFATURA,
                           NVL(PFIT.QTPARCELASFATU, PFESC.QTPARCELASFATU) AS QTPARCELASFATU,
                           NVL(PFIT.QTDIASAPURACAO, PFESC.QTDIASAPURACAO) AS QTDIASAPURACAO,
                           NVL(PFIT.NRDIACALCVENC, PFESC.NRDIACALCVENC) AS NRDIACALCVENC,
                           NVL(PFIT.QTDIASFATUVENC, PFESC.QTDIASFATUVENC) AS QTDIASFATUVENC,
                           NVL(PFIT.IDTPCALCVENC, PFESC.IDTPCALCVENC) AS IDTPCALCVENC,
                           NVL(PFIT.IDMESCALCVENC, PFESC.IDMESCALCVENC) AS IDMESCALCVENC
                      FROM FCT_ITESCOPOPROPOSTA ITESC
                     INNER JOIN FCT_SERVICOORCA SERV
                        ON ITESC.CDSERVORCA = SERV.CDSERVORCA
                       AND ITESC.NRORG      = SERV.NRORG
                      LEFT JOIN FCT_PARAMFATITESC PFIT
                        ON ITESC.NRTIPOORCAMENTO    = PFIT.NRTIPOORCAMENTO
                       AND ITESC.NRITESCOPOPROPOSTA = PFIT.NRITESCOPOPROPOSTA
                       AND ITESC.NRORG = PFIT.NRORG
                      LEFT JOIN PAR_PARAMFATURA PFPARCIT
                        ON PFPARCIT.NRPARCNEGOCIO = PFIT.NRPARCFATURA
                       AND PFPARCIT.NRORG = PFIT.NRORG
                      LEFT JOIN FCT_CAPAPARAMFATESC PFESC
                        ON ITESC.NRTIPOORCAMENTO = PFESC.NRTIPOORCAMENTO
                       AND ITESC.NRORG           = PFESC.NRORG
                      LEFT JOIN PAR_PARAMFATURA PFPARCESC
                        ON PFPARCESC.NRPARCNEGOCIO = PFESC.NRPARCFATURA
                       AND PFPARCESC.NRORG = PFESC.NRORG
                     INNER JOIN FCT_RELACESCOPOMODELO ESCMOD
                        ON ITESC.NRTIPOORCAMENTO    = ESCMOD.NRTIPOORCAMENTO
                       AND ITESC.NRORG              = ESCMOD.NRORG
                       AND ESCMOD.NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                     INNER JOIN FCT_ORCAMENTO ORC
                        ON ORC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                       AND ITESC.NRORG        = ORC.NRORG
                     INNER JOIN FCT_ITEMORCAMENTO ITORC
                        ON ORC.NRSEQORCAMENTO  = ITORC.NRSEQORCAMENTO
                       AND ORC.NRORG           = ITORC.NRORG
                       AND ITESC.CDSERVORCA    = ITORC.CDSERVORCA
                       AND ITORC.IDADICITEMORC = 'N'
                       AND ITORC.IDELEMENTO    IS NULL
                      LEFT JOIN PAR_PARAMFATURA PFPARCORC
                        ON PFPARCORC.NRPARCNEGOCIO = ORC.NRPARCNEGOCIO
                       AND PFPARCORC.NRORG = ORC.NRORG
                     WHERE ITESC.NRTIPOORCAMENTO IN ($escopos)
                       AND ITESC.NRORG           = :NRORG
                       AND ITESC.CDSERVORCA      IN ($servicos)
                ) TBL
                ORDER BY NRTIPOORCAMENTO,
                         IDTPFATURA,
                         CDTIPOOPERFATURA,
                         CDSERIENF,
                         CDEMPRESA,
                         CDINSCESTA,
                         CDINSCMUNI,
                         CDFILIALFATURA,
                         NRPARCFATURA,
                         QTPARCELASFATU,
                         QTDIASAPURACAO,
                         NRDIACALCVENC,
                         QTDIASFATUVENC,
                         IDTPCALCVENC,
                         IDMESCALCVENC,
                         QTCOMPETENCIA";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    /**
     * Retorna o total de modelos de contrato ativos na proposta
    **/
    public function getTotalModelosAtivos(array $params) {
        $sql = "SELECT COUNT(*) MODELOS
                FROM FCT_ORCAMENTO ORC
                JOIN FCT_ORCAMENTOMODELO MODORC
                  ON ORC.NRORCAMENTOMODELO = MODORC.NRORCAMENTOMODELO
                  AND ORC.NRORG = MODORC.NRORG
                JOIN FCT_MODELOCONTRATO MODCONT
                  ON MODORC.NRSEQMODCONTRA = MODCONT.NRSEQMODCONTRA
                  AND MODORC.NRORG = MODCONT.NRORG
                WHERE NRSEQORCAMENTO  = :NRSEQORCAMENTO
                  AND MODCONT.IDATIVO = 'S'
                  AND MODCONT.NRORG   = :NRORG";
 
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function inserirNegociacaoProposta(array $params) {
        $sql = "INSERT INTO FCT_NEGOCIACAOPROPOSTA (
            NRNEGOCIACAOPROPOSTA, NRSEQORCAMENTO, NRTIPOORCAMENTO,
            IDTPFATURA,           CDFILIALFATURA, NRPARCFATURA,
            CDTIPOOPERFATURA,     CDSERIENF,      CDEMPRESA,
            CDINSCESTA,           CDINSCMUNI,     VRTOTITEM,
            VRPERCDESCITEM,       VRDESCITEM,     VRTOTITEMCOMDESC,
            IDATIVO,              NRORG,          NRORGINCLUSAO,
            CDOPERINCLUSAO,       DTINCLUSAO
        ) VALUES (
            :NRNEGOCIACAOPROPOSTA,     :NRSEQORCAMENTO,       :NRTIPOORCAMENTO,
            :IDTPFATURA,               :CDFILIALFATURA,       :NRPARCFATURA,
            :CDTIPOOPERFATURA,         :CDSERIENF,            :CDEMPRESA,
            :CDINSCESTA,               :CDINSCMUNI,           ROUND(:VRTOTITEM, 2),
            ROUND(:VRPERCDESCITEM, 2), ROUND(:VRDESCITEM, 2), ROUND(:VRTOTITEMCOMDESC, 2),
            :IDATIVO,                  :NRORG,                :NRORGINCLUSAO,
            :CDOPERINCLUSAO,           SYSDATE
        )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function inserirItemNegociacaoProposta(array $params) {
        $sql = "INSERT INTO FCT_ITNEGOCIACAOPROPOSTA (
            NRITNEGOCIACAOPROPOSTA, NRNEGOCIACAOPROPOSTA, NRITEMORCAMENTO,
            NRITEMORCAMENTOPAI,     VRTOTITEM,            VRPERCDESCITEM,
            VRDESCITEM,             VRTOTITEMCOMDESC,     CDSERVORCA,
            CDPRODFATURA,           NRPARCFATURA,         IDATIVO,
            NRORG,                  NRORGINCLUSAO,        CDOPERINCLUSAO,
            DTINCLUSAO
        ) VALUES (
            :NRITNEGOCIACAOPROPOSTA, :NRNEGOCIACAOPROPOSTA,       :NRITEMORCAMENTO,
            :NRITEMORCAMENTOPAI,     ROUND(:VRTOTITEM, 2),        ROUND(:VRPERCDESCITEM, 2),
            ROUND(:VRDESCITEM, 2),   ROUND(:VRTOTITEMCOMDESC, 2), :CDSERVORCA,
            :CDPRODFATURA,           :NRPARCFATURA,               :IDATIVO,
            :NRORG,                  :NRORGINCLUSAO,              :CDOPERINCLUSAO,
            SYSDATE
        )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function excluirParcelasNegociacao(array $params) {
        $sql = "DELETE FROM FCT_PARCELANEGOPROP
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function inserirParcelaNegociacao(array $params) {
        $sql = "INSERT INTO FCT_PARCELANEGOPROP (
            NRPARCELANEGOPROP, NRNEGOCIACAOPROPOSTA, QTDIASFATURAMENTO,
            NRPARCELA,         NRDIAVENCIMENTO,      VRPARCELA,            
            IDATIVO,           NRORG,                DTINCLUSAO,           
            NRORGINCLUSAO,     CDOPERINCLUSAO
        ) VALUES (
            :NRPARCELANEGOPROP, :NRNEGOCIACAOPROPOSTA, :QTDIASFATURAMENTO,
            :NRPARCELA,         :NRDIAVENCIMENTO,      ROUND(:VRPARCELA, 2),  
            :IDATIVO,           :NRORG,             SYSDATE,               
            :NRORGINCLUSAO,     :CDOPERINCLUSAO
        )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getNegociacoesValoresComSomaDasParcelas(array $params) {
        $sql = "SELECT NEG.NRNEGOCIACAOPROPOSTA,
                       NEG.VRTOTITEMCOMDESC,
                       (SELECT SUM(VRPARCELA)
                          FROM FCT_PARCELANEGOPROP
                         WHERE NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                           AND NRORG                = NEG.NRORG) AS SOMA_PARCELAS
                  FROM FCT_NEGOCIACAOPROPOSTA NEG
                 WHERE NEG.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NEG.NRORG          = :NRORG
                   AND NEG.IDTPFATURA     = 'P'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    /* Busca o endereço principal do parceiro */
    public function buscaEnderecoProposta(array $params) {
        $sql = "SELECT NRENDERECOPARC
                  FROM (
                    SELECT NRENDERECOPARC
                      FROM ENDERECOPARC
                     WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                       AND NRORG          = :NRORG
                       AND CDTIPOENDERECO = 'PRINCIPAL'
                       AND IDATIVO        = 'S'
                     ORDER BY DSENDERECO)
                WHERE ROWNUM = 1";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    /*public function atualizarDatasFinaisDescontoPorPeriodo(array $params) {
        $sql = "UPDATE FCT_DESCPERIODONEGOPROP
                   SET DTFINALVALIDADEDESC  = ADD_MONTHS(TO_DATE(:DTACEITE, 'DD/MM/YYYY'), QTMESESDESC)
                 WHERE NRORG                = :NRORG
                   AND NRNEGOCIACAOPROPOSTA IN (SELECT NRNEGOCIACAOPROPOSTA
                                                  FROM FCT_NEGOCIACAOPROPOSTA
                                                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                                   AND NRORG          = :NRORG)";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }*/

    /* Insere o endereço principal do parceiro de negócio na proposta */
    public function inserirEnderecoProposta(array $params) {
        $sql = "INSERT INTO FCT_ENDERECOORCA (
                    NRENDERECOORCA,   NRSEQORCAMENTO,     CDTIPOENDERECO,     CDPAIS,
                    SGESTADO,         CDMUNICIPIO,        CDBAIRRO,           CDLOGRADOURO,
                    DSENDERECO,       NRIMOVELENDERECO,   NMBAIRROENDERECO,   DSCOMPLEENDERECO,
                    NRCEPENDERECO,    DSREFERENCIAENDE,   NRORG,              IDATIVO,
                    DTINCLUSAO,       NRORGINCLUSAO,      CDOPERINCLUSAO)
                (SELECT :NRENDERECOORCA AS NRENDERECOORCA,
                        :NRSEQORCAMENTO AS NRSEQORCAMENTO,
                        'PRINCIPAL' AS CDTIPOENDERECO,
                        CDPAIS,
                        SGESTADO,
                        CDMUNICIPIO,
                        CDBAIRRO,
                        CDLOGRADOURO,
                        DSENDERECO,
                        NRIMOVELENDERECO,
                        NMBAIRROENDERECO,
                        DSCOMPLEENDERECO,
                        NRCEPENDERECO,
                        DSREFERENCIAENDE,
                        :NRORG AS NRORG,
                        IDATIVO,
                        SYSDATE AS DTINCLUSAO,
                        :NRORG AS NRORGINCLUSAO,
                        :CDOPERADOR AS CDOPERINCLUSAO
                   FROM ENDERECOPARC
                  WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                    AND NRORG          = :NRORG
                    AND NRENDERECOPARC = :NRENDERECOPARC)";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /* Busca se o modelo utiliza composição de serviços */
    public function validaUtilizaComposicao(array $params) {
        $sql = "SELECT IDUTILIZACOMP
                  FROM FCT_ORCAMENTOMODELO
                 WHERE NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                   AND NRORG = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getElementosItensNegociacao(array $params) {
        $sql = "SELECT NRITNEGOCIACAOPROPOSTA,
                       NRNEGOCIACAOPROPOSTA,
                       NRITEMORCAMENTO,
                       VRTOTITEM
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG
                   AND NRITEMORCAMENTOPAI IS NOT NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function atualizarValorItemNegociacao(array $params) {
        $sql = "UPDATE FCT_ITNEGOCIACAOPROPOSTA
                   SET VRTOTITEM        = ROUND(:VRTOTITEM, 2),
                       VRDESCITEM       = ROUND(:VRTOTITEM * (NVL(VRPERCDESCITEM, 0) / 100), 2),
                       VRTOTITEMCOMDESC = ROUND(:VRTOTITEM - (:VRTOTITEM * (NVL(VRPERCDESCITEM, 0) / 100)), 2),
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU,
                       NRORGULTATU      = :NRORGULTATU
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRITEMORCAMENTO      = :NRITEMORCAMENTO
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getCompetenciaItemNegociacao(array $params) {
        $sql = "SELECT NRDIACOMPETINI,
                       NRDIACOMPETFIN,
                       IDMESCOMPETINI,
                       IDMESCOMPETFIN,
                       QTDIASAPURACAO,
                       IDTPCALCVENC,
                       NRDIACALCVENC,
                       IDMESCALCVENC,
                       QTDIASFATUVENC
                  FROM (
                    SELECT PFIT.NRDIACOMPETINI,
                           PFIT.NRDIACOMPETFIN,
                           PFIT.IDMESCOMPETINI,
                           PFIT.IDMESCOMPETFIN,
                           PFIT.QTDIASAPURACAO,
                           PFIT.IDTPCALCVENC,
                           PFIT.NRDIACALCVENC,
                           PFIT.IDMESCALCVENC,
                           PFIT.QTDIASFATUVENC
                      FROM FCT_PARAMFATITESCSERV PFIT
                     INNER JOIN FCT_ITESCOPOPROPOSTA IT
                        ON PFIT.NRITESCOPOPROPOSTA = IT.NRITESCOPOPROPOSTA
                       AND PFIT.NRORG              = IT.NRORG
                     WHERE IT.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                       AND IT.CDSERVORCA      = :CDSERVORCA
                       AND IT.NRORG           = :NRORG
                     UNION ALL
                    SELECT NRDIACOMPETINI,
                           NRDIACOMPETFIN,
                           IDMESCOMPETINI,
                           IDMESCOMPETFIN,
                           QTDIASAPURACAO,
                           IDTPCALCVENC,
                           NRDIACALCVENC,
                           IDMESCALCVENC,
                           QTDIASFATUVENC
                      FROM FCT_PARAMFATESC
                     WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                       AND NRORG           = :NRORG
                       AND NOT EXISTS (SELECT 1
                                         FROM FCT_PARAMFATITESCSERV
                                        WHERE NRITESCOPOPROPOSTA = (SELECT NRITESCOPOPROPOSTA
                                                                      FROM FCT_ITESCOPOPROPOSTA
                                                                     WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                                                                       AND CDSERVORCA      = :CDSERVORCA))
                )
                ORDER BY IDMESCOMPETINI, NRDIACOMPETINI";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    /* Lista os itens padrões principais do modelo de proposta */
    public function ListaItensPadroesPai(array $params) {
        $sql = "SELECT NRITEMORCAMODELO,
                       CDSERVORCA,
                       CDPRODUTO,
                       NMITEMORCAMENTO,
                       NVL(QTITEMORCAMENTO, 1) AS QTITEMORCAMENTO,
                       CDTPMEDIDASERV,
                       QTMEDIDAORCA,
                       NVL(VRUNITITEMORCA, 0) AS VRUNITITEMORCA,
                       NVL(VRTOTALITEMORCA, 0) AS VRTOTALITEMORCA,
                       IDFATURAORCA,
                       CDMODPGTO,
                       DSQTITEMORCA,
                       IDITEMOBRIGORCA,
                       IDSOMASUBITENS,
                       IDSOMANOITEMPAI,
                       IDSOMANOTOTALORCA,
                       NRITEMORCAMODELOPAI,
                       NRGRUPOFATURA,
                       IDIMPRESSAOORCA,
                       SGUNIDADE,
                       IDTRANSPORTADORA,
                       IDADICCOBRITEMORC,
                       IDADICITEMORC,
                       IDBLOQALTORC,
                       IDMULTMEDVRTOT,
                       NRTIPOHIERARQSERV,
                       QTDIASINICFAT,
                       DECODE(QTDIASINICFAT, NULL, NULL, SYSDATE+QTDIASINICFAT) AS DTINIFATURAMENTO,
                       QTPARCELASFATU,
                       NRDIAVENCPARCELASFATU
                  FROM FCT_ITEMORCAMODELO
                 WHERE NRORCAMENTOMODELO   = :NRORCAMENTOMODELO
                   AND NRORG               = :NRORG
                   AND IDITEMESPELHO       = 'NAO'
                   AND IDADICITEMORC       = 'N'
                   AND NRITEMORCAMODELOPAI IS NULL";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function inserirCompetenciaNegociacaoProposta(array $params) {
        $sql = "INSERT INTO FCT_COMPETENCIANEGOPROP (
            NRCOMPETENCIANEGOPROP, NRNEGOCIACAOPROPOSTA, NRDIACOMPETINI,
            NRDIACOMPETFIN,        IDMESCOMPETINI,       IDMESCOMPETFIN,
            QTDIASAPURACAO,        IDTPCALCVENC,         NRDIACALCVENC,
            IDMESCALCVENC,         QTDIASFATUVENC,       IDATIVO,
            NRORG,                 DTINCLUSAO,           NRORGINCLUSAO,
            CDOPERINCLUSAO
        )
        VALUES (
            :NRCOMPETENCIANEGOPROP, :NRNEGOCIACAOPROPOSTA, :NRDIACOMPETINI,
            :NRDIACOMPETFIN,        :IDMESCOMPETINI,       :IDMESCOMPETFIN,
            :QTDIASAPURACAO,        :IDTPCALCVENC,         :NRDIACALCVENC,
            :IDMESCALCVENC,         :QTDIASFATUVENC,       :IDATIVO,
            :NRORG,                 SYSDATE,               :NRORGINCLUSAO,
            :CDOPERINCLUSAO
        )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getSomaValorItensNegociacao(array $params) {
        $sql = "SELECT SUM(VRTOTITEMCOMDESC) AS VRTOTAL
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG
                   AND NRITEMORCAMENTOPAI   IS NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    /* Lista os itens padrões adicionais do modelo de proposta */
    public function ListaItensPadroesAdic(array $params) {
        $sql = "SELECT NRITEMORCAMODELO,
                           CDSERVORCA,
                           CDPRODUTO,
                           NMITEMORCAMENTO,
                           NVL(QTITEMORCAMENTO, 1) AS QTITEMORCAMENTO,
                           CDTPMEDIDASERV,
                           QTMEDIDAORCA,
                           NVL(VRUNITITEMORCA, 0) AS VRUNITITEMORCA,
                           NVL(VRTOTALITEMORCA, 0) AS VRTOTALITEMORCA,
                           IDFATURAORCA,
                           CDMODPGTO,
                           DSQTITEMORCA,
                           IDITEMOBRIGORCA,
                           IDSOMASUBITENS,
                           IDSOMANOITEMPAI,
                           IDSOMANOTOTALORCA,
                           NRITEMORCAMODELOPAI,
                           NRGRUPOFATURA,
                           IDIMPRESSAOORCA,
                           SGUNIDADE,
                           IDTRANSPORTADORA,
                           IDADICCOBRITEMORC,
                           IDADICITEMORC,
                           IDBLOQALTORC,
                           IDMULTMEDVRTOT,
                           NRTIPOHIERARQSERV,
                           QTDIASINICFAT,
                           DECODE(QTDIASINICFAT, NULL, NULL, SYSDATE+QTDIASINICFAT) AS DTINIFATURAMENTO,
                           QTPARCELASFATU,
                           NRDIAVENCPARCELASFATU
                      FROM FCT_ITEMORCAMODELO
                     WHERE NRORCAMENTOMODELO   = :NRORCAMENTOMODELO
                       AND NRORG               = :NRORG
                       AND IDITEMESPELHO       = 'NAO'
                       AND IDADICITEMORC       = 'S'
                       AND NRITEMORCAMODELOPAI = :NRITEMORCAMODELO";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function atualizarValorNegociacaoComSomaDosItens(array $params) {
        $sql = "UPDATE FCT_NEGOCIACAOPROPOSTA
                   SET VRTOTITEM        = ROUND(:VRTOTAL, 2),
                       VRDESCITEM       = ROUND((:VRTOTAL * NVL(VRPERCDESCITEM, 0) / 100), 2),
                       VRTOTITEMCOMDESC = ROUND(:VRTOTAL - (:VRTOTAL * NVL(VRPERCDESCITEM, 0) / 100), 2),
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU,
                       NRORGULTATU      = :NRORGULTATU
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    /* Insere os itens padrões do modelo de proposta */
    public function insereItensPadrao(array $params) {
        $sql = "INSERT INTO FCT_ITEMORCAMENTO (
                    CDSERVORCA,           CDPRODUTO,            NMITEMORCAMENTO,      QTITEMORCAMENTO,
                    CDTPMEDIDASERV,       QTMEDIDAORCA,         VRUNITITEMORCA,       VRTOTALITEMORCA,
                    IDFATURAORCA,         CDMODPGTO,            DSQTITEMORCA,         IDITEMOBRIGORCA,
                    IDSOMASUBITENS,       IDSOMANOITEMPAI,      NRORG,                IDATIVO,
                    DTINCLUSAO,           CDOPERINCLUSAO,       IDSOMANOTOTALORCA,    NRITEMORCAMENTOPAI,
                    NRSEQORCAMENTO,       NRITEMORCAMENTO,      NRGRUPOFATURA,        IDIMPRESSAOORCA,
                    SGUNIDADE,            IDTRANSPORTADORA,     IDADICCOBRITEMORC,    IDADICITEMORC,
                    IDBLOQALTORC,         IDMULTMEDVRTOT,       DTINICFATORC,         VRITEMSEMDESC,
                    NRTIPOHIERARQSERV,    IDTPDESCORCA,         QTPARCELASFATU,       NRDIAVENCPARCELASFATU
                ) VALUES (
                   :CDSERVORCA,          :CDPRODUTO,              :NMITEMORCAMENTO,       ROUND(:QTITEMORCAMENTO, 2),
                   :CDTPMEDIDASERV,      ROUND(:QTMEDIDAORCA, 2), ROUND(:VRUNITITEM, 2),  ROUND(:VRTOTALITEM, 2),
                   :IDFATURAORCA,        :CDMODPGTO,              'Referente ao Produto', :IDITEMOBRIGORCA,
                   :IDSOMASUBITENS,      :IDSOMANOITEMPAI,        :NRORG,                 'S',
                   SYSDATE,              :CDOPERADOR,             :IDSOMANOTOTALORCA,     :NRITEMPAI,
                   :NRSEQORCAMENTO,      :NRITEMORCAMENTO,        :NRGRUPOFATURA,         :IDIMPRESSAOORCA,
                   :SGUNIDADE,           :IDTRANSPORTADORA,       :IDADICCOBRITEMORC,     :IDADICITEMORC,
                   :IDBLOQALTORC,        :IDMULTMEDVRTOT,         :DTINIFATURAMENTO,      ROUND(:VRTOTALITEM, 2),
                   :NRTIPOHIERARQSERV,   'N',                     :QTPARCELASFATU,        :NRDIAVENCPARCELASFATU
                )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getItensFilhosItemProposta(array $params) {
        $sql = "SELECT NRITEMORCAMENTO,
                       NRITEMORCAMENTOPAI,
                       VRTOTALITEMORCA,
                       CDSERVORCA
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                   AND NRITEMORCAMENTOPAI = :NRITEMORCAMENTOPAI
                   AND NRORG              = :NRORG
                   AND IDADICITEMORC      = 'N'";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getSomaValorElementosItemNegociacao(array $params) {
        $sql = "SELECT SUM(VRTOTITEMCOMDESC) AS VRTOTAL
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRITEMORCAMENTOPAI   = :NRITEMORCAMENTOPAI
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    /* Atualiza o valor da proposta */
    public function atualizaValorProposta(array $params){
        $sql = "UPDATE FCT_ORCAMENTO
                   SET VRORCAMENTO = ROUND((SELECT SUM(VRTOTALITEMORCA)
                                              FROM FCT_ITEMORCAMENTO
                                             WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                               AND NRORG          =:NRORG), 2)
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /* Busca os documentos vinculados aos escopos do modelo da proposta */
    public function getDocumentosModProposta(array $params) {
        $sql = "SELECT ESCOPODOC.NRSEQMODDOCFCT,
                    DOC.NMARQUIVO,
                    DOC.DSCAMINHOARQ,
                    DOC.DSTIPO,
                    DOC.VRTAMANHO,
                    DOC.VRTAMANHOLOGO,
                    REL.NRTIPOORCAMENTO,
                    ESCOPODOC.IDSITUACAOORCAMENTO
                FROM FCT_ORCAESCOPODOC ESCOPODOC
                INNER JOIN FCT_MODELODOC DOC
                    ON ESCOPODOC.NRSEQMODDOCFCT = DOC.NRSEQMODDOCFCT
                    AND ESCOPODOC.NRORG         = DOC.NRORG
                INNER JOIN FCT_RELACESCOPOMODELO REL
                    ON REL.NRRELACESCOPOMODELO = ESCOPODOC.NRRELACESCOPOMODELO
                    AND REL.NRORG              = ESCOPODOC.NRORG
               WHERE REL.NRORCAMENTOMODELO    = :NRORCAMENTOMODELO
                    AND REL.NRORG             = :NRORG
                    AND DOC.IDDOCRENEGOCIACAO = 'N'";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getNegociacoesProposta(array $params) {
        $sql = "SELECT NRNEGOCIACAOPROPOSTA,
                       NRTIPOORCAMENTO,
                       IDTPFATURA,
                       VRTOTITEMCOMDESC
                  FROM FCT_NEGOCIACAOPROPOSTA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoNegociacaoDoItem(array $params) {
        $sql = "SELECT NRNEGOCIACAOPROPOSTA
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRITEMORCAMENTO = :NRITEMORCAMENTO
                   AND NRORG           = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    /**
     * Retorna os descontos por período da negociação
     * Obs: É necessário manter a ordenação.
    **/
    public function getDescontosPorPeriodoDaNegociacao(array $params) {
        $sql = "SELECT NRDESCPERIODONEGOPROP,
                       VRPERCDESCITEM,
                       VRDESCITEM,
                       VRTOTITEMCOMDESC,
                       QTMESESDESC,
                       DTFINALVALIDADEDESC,
                       IDATIVO,
                       NRORDEMAPLICADESC,
                       NRNEGOCIACAOPROPOSTA,
                       DTINCLUSAO,
                       NRORGINCLUSAO,
                       CDOPERINCLUSAO,
                       DTULTATU,
                       NRORGULTATU,
                       CDOPERULTATU,
                       IDTPDESCONTO
                FROM FCT_DESCPERIODONEGOPROP
                WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG               = :NRORG
                ORDER BY NRORDEMAPLICADESC";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function atualizarValorDescontoPorPeriodoDaNegociacao(array $params) {
        $sql = "UPDATE FCT_DESCPERIODONEGOPROP
                   SET VRDESCITEM       = ROUND((:VRTOTITEM * VRPERCDESCITEM / 100), 2),
                       VRTOTITEMCOMDESC = ROUND(:VRTOTITEM - (:VRTOTITEM * VRPERCDESCITEM / 100), 2),
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU,
                       NRORGULTATU      = :NRORGULTATU
                 WHERE NRDESCPERIODONEGOPROP = :NRDESCPERIODONEGOPROP
                   AND NRORG                 = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function excluirItemNegociacaoComFilhos(array $params) {
        $sql = "DELETE FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND (NRITEMORCAMENTO     = :NRITEMORCAMENTO
                    OR NRITEMORCAMENTOPAI   = :NRITEMORCAMENTO)
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    /* Insere os documentos da proposta */
    public function inserirDocumentosProposta(array $params) {
        $sql = "INSERT INTO FCT_ORCAMENTODOC (
                    NRSEQORCADOC,     NRSEQORCAMENTO,   NRSEQMODDOCFCT,  IDIDIOMA,
                    TXMODELOORCADOC,  IDPADRAOORCADOC,  NRORG,           IDATIVO,
                    DTINCLUSAO,       NRORGINCLUSAO,    CDOPERINCLUSAO,  NMARQUIVO,
                    DSCAMINHOARQ,     DSTIPO,           VRTAMANHO,       IDSITUACAOORCAMENTO,
                    NRTIPOORCAMENTO,  VRTAMANHOLOGO)
                VALUES (
                    :NRSEQORCADOC,     :NRSEQORCAMENTO,  :NRSEQMODDOCFCT, NULL,
                    NULL,              :IDMODPADRAO,     :NRORG,          'S',
                    SYSDATE,           :NRORG,           :CDOPERADOR,     :NMARQUIVO,
                    :DSCAMINHOARQ,     :DSTIPO,          :VRTAMANHO,      :IDSITUACAOORCAMENTO,
                    :NRTIPOORCAMENTO,  :VRTAMANHOLOGO)";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getQuantidadeItensNegociacaoProposta(array $params) {
        $sql = "SELECT COUNT(*) AS QTDE
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG
                   AND NRITEMORCAMENTOPAI   IS NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCompetenciaNegociacaoProposta(array $params) {
        $sql = "SELECT NRCOMPETENCIANEGOPROP,
                       NRDIACOMPETINI,
                       NRDIACOMPETFIN,
                       IDMESCOMPETINI,
                       IDMESCOMPETFIN,
                       QTDIASAPURACAO,
                       IDTPCALCVENC,
                       NRDIACALCVENC,
                       IDMESCALCVENC,
                       QTDIASFATUVENC,
                       IDATIVO
                  FROM FCT_COMPETENCIANEGOPROP
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG
                 ORDER BY IDMESCOMPETINI, NRDIACOMPETINI";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function atualizarDataFinalValidadeDescontoPorPeriodo(array $params) {
        $sql = "UPDATE FCT_DESCPERIODONEGOPROP
                   SET DTFINALVALIDADEDESC   = :DTFINALVALIDADEDESC
                 WHERE NRDESCPERIODONEGOPROP = :NRDESCPERIODONEGOPROP
                   AND NRORG                 = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function atualizarNegociacaoProposta(array $params) {
        $sql = "UPDATE FCT_NEGOCIACAOPROPOSTA
                   SET IDTPFATURA       = :IDTPFATURA,
                       CDFILIALFATURA   = :CDFILIALFATURA,
                       NRPARCFATURA     = :NRPARCFATURA,
                       CDTIPOOPERFATURA = :CDTIPOOPERFATURA,
                       CDSERIENF        = :CDSERIENF,
                       CDEMPRESA        = :CDEMPRESA,
                       CDINSCESTA       = :CDINSCESTA,
                       CDINSCMUNI       = :CDINSCMUNI,
                       VRTOTITEM        = ROUND(:VRTOTITEM, 2),
                       VRPERCDESCITEM   = ROUND(:VRPERCDESCITEM, 2),
                       VRDESCITEM       = ROUND(:VRDESCITEM, 2),
                       VRTOTITEMCOMDESC = ROUND(:VRTOTITEMCOMDESC, 2)
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function atualizarItemNegociacaoProposta(array $params) {
        $sql = "UPDATE FCT_ITNEGOCIACAOPROPOSTA
                   SET VRTOTITEM              = ROUND(:VRTOTITEM, 2),
                       VRPERCDESCITEM         = ROUND(:VRPERCDESCITEM, 2),
                       VRDESCITEM             = ROUND(:VRDESCITEM, 2),
                       VRTOTITEMCOMDESC       = ROUND(:VRTOTITEMCOMDESC, 2),
                       NRPARCFATURA           = :NRPARCFATURA
                 WHERE NRITNEGOCIACAOPROPOSTA = :NRITNEGOCIACAOPROPOSTA
                   AND NRORG                  = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function atualizarParceiroNegociacoesProposta(array $params) {
        $sql = "UPDATE FCT_NEGOCIACAOPROPOSTA
                   SET NRPARCFATURA = :NRPARCFATURA,
                       DTULTATU     = SYSDATE,
                       NRORGULTATU  = :NRORGULTATU,
                       CDOPERULTATU = :CDOPERULTATU
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function atualizarParceiroItensNegociacoesProposta(array $params) {
        $sql = "UPDATE FCT_ITNEGOCIACAOPROPOSTA
                   SET NRPARCFATURA = :NRPARCFATURA,
                       DTULTATU     = SYSDATE,
                       NRORGULTATU  = :NRORGULTATU,
                       CDOPERULTATU = :CDOPERULTATU
                 WHERE NRNEGOCIACAOPROPOSTA IN (SELECT NRNEGOCIACAOPROPOSTA
                                                  FROM FCT_NEGOCIACAOPROPOSTA
                                                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                                   AND NRORG          = :NRORG)
                  AND NRORG = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function validaPropReneg(array $params) {
        $sql = "SELECT COUNT(*) AS VALIDA
                  FROM FCT_ORCAMENTO
                  JOIN FCT_REFEORCACONT
                    ON FCT_REFEORCACONT.NRSEQORCAMENTO = FCT_ORCAMENTO.NRSEQORCAMENTOPAI
                   AND FCT_REFEORCACONT.NRORG = FCT_ORCAMENTO.NRORG
                  JOIN FCT_CONTRATO
                    ON FCT_CONTRATO.NRSEQCONTRATO = FCT_REFEORCACONT.NRSEQCONTRATO
                   AND FCT_CONTRATO.NRORG = FCT_REFEORCACONT.NRORG
                   AND FCT_CONTRATO.IDSITUACAOCONTRATO IN ('E', 'R')
                 WHERE FCT_ORCAMENTO.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND FCT_ORCAMENTO.NRORG = :NRORG
                   AND FCT_ORCAMENTO.NRSEQORCAMENTOPAI IS NOT NULL
                   AND FCT_REFEORCACONT.IDATIVO = 'S'";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function excluirTodasNegociacoesProposta(array $params) {
        $sql = "DELETE FROM FCT_NEGOCIACAOPROPOSTA 
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getSomaValorNegociacoesProposta(array $params) {
        $sql = "SELECT NVL(SUM(VRTOTITEMCOMDESC), 0) AS VRTOTITEMCOMDESC
                  FROM FCT_NEGOCIACAOPROPOSTA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    /**
     * Busca a quantidade máxima de parcelas permitida a um operador
    **/
    public function getQuantidadeMaxParcelasOper(array $params) {
        $sql = "SELECT QTPARCMAXIMAOPER AS QTPARCMAXIMA
                FROM FCT_QTPARCMAXIMAOPER
                WHERE CDOPERADOR = :CDOPERADOR 
                  AND NRORG      = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    /**
     * Busca a quantidade máxima de parcelas permitida a um perfil de acesso
    **/
    public function getQuantidadeMaxParcelasPerfil(array $params) {
        $sql = "SELECT MIN(QTPARCMAXPERFACESS) AS QTPARCMAXIMA
                FROM FCT_QTPARCMAXPERFACESS QTPARC
                JOIN PERFACESSPARC PERFIL
                  ON QTPARC.NRPERFILACESSO = PERFIL.NRPERFILACESSO
                  AND QTPARC.NRORG         = PERFIL.NRORG
                WHERE QTPARC.NRORG = :NRORG
                  AND PERFIL.NRPARCNEGOCIO IN(SELECT NRPARCNEGOCIO 
                                              FROM OPERADOR 
                                              WHERE CDOPERADOR = :CDOPERADOR 
                                                AND NRORG      = :NRORG)";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function atualizarValorNegociacaoComDesconto(array $params) {
        $sql = "UPDATE FCT_NEGOCIACAOPROPOSTA
                   SET VRPERCDESCITEM   = ROUND(:VRPERCDESCITEM, 2),
                       VRDESCITEM       = ROUND((VRTOTITEM * :VRPERCDESCITEM / 100), 2),
                       VRTOTITEMCOMDESC = ROUND(VRTOTITEM - (VRTOTITEM * :VRPERCDESCITEM / 100), 2),
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU,
                       NRORGULTATU      = :NRORGULTATU,
                       IDTPDESCONTO     = 'P'
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    /**
     * Retorna as parcelas da negociação
    **/
    public function getParcelasNegociacao(array $params) {
        $sql = "SELECT NRPARCELANEGOPROP,
                    QTDIASFATURAMENTO,
                    NRDIAVENCIMENTO,
                    VRPARCELA
                FROM FCT_PARCELANEGOPROP
                WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG               = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    /**
     * Retorna a quantidade máxima de parcelas da 
     * tabela de parametro de faturamento dos itens do escopo 
     * (caso haja mais de um serviço será retornado a menor quantidade)
    **/
    public function getQtMaxParcelasItemEscopo(array $params) {
        $sql = "SELECT MIN(PA.QTPARCELASFATU) AS QTPARCELASFATU
                FROM FCT_PARAMFATITESC PA
                INNER JOIN FCT_ITESCOPOPROPOSTA IT
                  ON PA.NRITESCOPOPROPOSTA = IT.NRITESCOPOPROPOSTA
                  AND PA.NRORG             = IT.NRORG
                WHERE IT.NRTIPOORCAMENTO = :NRTIPOORCAMENTO 
                  AND IT.NRORG           = :NRORG
                  AND IT.CDSERVORCA IN(SELECT CDSERVORCA
                                        FROM FCT_ITNEGOCIACAOPROPOSTA
                                        WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                                          AND NRORG                = :NRORG)";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    /**
     * Retorna a quantidade máxima de parcelas da 
     * tabela de parametro de faturamento dos itens do escopo 
     * (caso haja mais de um serviço será retornado a menor quantidade)
    **/
    public function getQtMaxParcelasEscopo(array $params) {
        $sql = "SELECT NVL(QTPARCELASFATU, 0) AS QTPARCELASFATU
                  FROM FCT_CAPAPARAMFATESC
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO 
                   AND NRORG           = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function excluirNegociacaoProposta(array $params) {
        $sql = "DELETE FROM FCT_NEGOCIACAOPROPOSTA 
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function salvarVendedor(array $params) {
        $sql = "INSERT INTO FCT_VENDEDORORCA (
                    NRVENDEDORORCA, NRSEQORCAMENTO,     NRPARCVENDEDOR,
                    NRPARCGERENTE,  VRPEPARTICIPAVENDA, IDVENPRIORCA,
                    NRORG,          IDATIVO,            DTINCLUSAO,
                    NRORGINCLUSAO,  CDOPERINCLUSAO
                )
                VALUES (
                    :NRVENDEDORORCA, :NRSEQORCAMENTO,               :NRPARCVENDEDOR,
                    :NRPARCGERENTE,  ROUND(:VRPEPARTICIPAVENDA, 2), :IDVENPRIORCA,
                    :NRORG,          :IDATIVO,                      SYSDATE,
                    :NRORGINCLUSAO,  :CDOPERINCLUSAO
                )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getValorNegociacaoComSomaParcelas(array $params) {
        $sql = "SELECT NEG.NRNEGOCIACAOPROPOSTA,
                       NEG.VRTOTITEMCOMDESC,
                       (SELECT SUM(VRPARCELA)
                          FROM FCT_PARCELANEGOPROP
                         WHERE NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                           AND NRORG                = NEG.NRORG) AS SOMA_PARCELAS
                  FROM FCT_NEGOCIACAOPROPOSTA NEG
                 WHERE NEG.NRSEQORCAMENTO       = :NRSEQORCAMENTO
                   AND NEG.NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NEG.NRORG                = :NRORG
                   AND NEG.IDTPFATURA           = 'P'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function reajustarParcelasNegociacao(array $params) {
        $sql = "UPDATE FCT_PARCELANEGOPROP
                   SET VRPARCELA      = ROUND(VRPARCELA + (VRPARCELA * :VRPERCENTUAL / 100), 2),
                       DTULTATU       = SYSDATE,
                       NRORGULTATU    = :NRORGULTATU,
                       CDOPERULTATU   = :CDOPERULTATU
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getElementosItemNegociacao(array $params) {
        $sql = "SELECT NRITNEGOCIACAOPROPOSTA,
                       NRNEGOCIACAOPROPOSTA,
                       NRITEMORCAMENTO,
                       NRITEMORCAMENTOPAI,
                       VRTOTITEM,
                       VRPERCDESCITEM,
                       VRDESCITEM,
                       VRTOTITEMCOMDESC,
                       CDSERVORCA,
                       CDPRODFATURA,
                       NRPARCFATURA
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG
                   AND NRITEMORCAMENTOPAI   = :NRITEMORCAMENTOPAI";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    /**
     * Busca as informações da tabela de operadores com permissões de 
     * alteração da medida da proposta
    **/
    public function getOperadorReducaoQtMedida($params) {
        $sql = "SELECT NRALTERAQTMEDOPER,
                       CDOPERADOR
                FROM FCT_ALTERAQTMEDOPERADOR
                WHERE CDOPERADOR = :CDOPERADOR
                  AND NRORG      = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    /**
     * Busca as informações da tabela de perfis de acesso com permissões de 
     * alteração da medida da proposta
    **/
    public function getPerfAcessReducaoQtMedida($params) {
        $sql = "SELECT ALTPERF.NRALTERAQTMEDPERF,
                       ALTPERF.NRPERFILACESSO
                FROM FCT_ALTERAQTMEDPERFACESS ALTPERF
                INNER JOIN PERFACESSPARC PERF
                  ON  ALTPERF.NRPERFILACESSO = PERF.NRPERFILACESSO
                  AND ALTPERF.NRORG          = PERF.NRORG
                WHERE ALTPERF.NRORG          = :NRORG
                  AND PERF.NRPARCNEGOCIO IN (SELECT NRPARCNEGOCIO
                                             FROM OPERADOR
                                             WHERE CDOPERADOR = :CDOPERADOR)";
    
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getParametrosFaturaNegociacoesProposta(array $params) {
        $sql = "SELECT NEG.NRNEGOCIACAOPROPOSTA,
                     NEG.IDTPFATURA,
                     NEG.CDFILIALFATURA,
                     NEG.CDTIPOOPERFATURA,
                     NEG.CDSERIENF,
                     (SELECT SUM(VRPARCELA)
                        FROM FCT_PARCELANEGOPROP
                        WHERE NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                            AND NRORG              = NEG.NRORG) AS SOMA_PARCELAS
                FROM FCT_NEGOCIACAOPROPOSTA NEG
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
                  
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Retorna todos os descontos da negociação e seus itens
    **/
    public function getDescontosNegociacaoValores(array $params) {
        $sql = "SELECT DISTINCT *
                FROM (
                    SELECT NEGO.NRTIPOORCAMENTO,
                      NEGO.NRSEQORCAMENTO,
                      NEGO.NRNEGOCIACAOPROPOSTA,
                      NEGO.NRORG,
                      DECODE(NVL(NEGO.VRPERCDESCITEM, 0), 0, ITNEGO.IDTPDESCONTO, NEGO.IDTPDESCONTO) AS IDTPDESCONTO, 
                      DECODE(NVL(NEGO.VRPERCDESCITEM, 0), 0, NVL(ITNEGO.VRPERCDESCITEM, 0), NVL(NEGO.VRPERCDESCITEM, 0)) AS VRPERCDESCITEM,
                      DECODE(NVL(NEGO.VRPERCDESCITEM, 0), 0, NVL(ITNEGO.VRDESCITEM, 0), NVL(NEGO.VRDESCITEM, 0)) AS VRDESCITEM,
                      DECODE(NVL(NEGO.VRPERCDESCITEM, 0), 0, 'ITENS', 'NEGOCIACAO') AS TIPODESCONTO,
                      DECODE(NVL(NEGO.VRPERCDESCITEM, 0), 0, ITNEGO.CDSERVORCA, NULL) CDSERVORCA,
                      DECODE(NVL(NEGO.VRPERCDESCITEM, 0), 0, ITNEGO.NRITEMORCAMENTO, NULL) NRITEMORCAMENTO,
                      DECODE(NVL(NEGO.VRPERCDESCITEM, 0), 0, ITNEGO.NRITEMORCAMENTOPAI, NULL) NRITEMORCAMENTOPAI
                    FROM FCT_NEGOCIACAOPROPOSTA NEGO
                    INNER JOIN FCT_ITNEGOCIACAOPROPOSTA ITNEGO
                      ON NEGO.NRNEGOCIACAOPROPOSTA = ITNEGO.NRNEGOCIACAOPROPOSTA
                      AND NEGO.NRORG               = ITNEGO.NRORG
                ) DESCONTOS
                WHERE DESCONTOS.VRPERCDESCITEM       <> 0
                  AND DESCONTOS.NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                  AND DESCONTOS.NRORG                = :NRORG";
                  
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Retorna o nritemorcamento de um item segundo as informações do elemento, serviço e medida
    **/
    public function getNovoItemCorrespondente(array $params) {
        $sql = "SELECT NRITEMORCAMENTO, NRITEMORCAMENTOPAI, QTMEDIDACALCULADA
                FROM FCT_ITEMORCAMENTO
                WHERE IDELEMENTO         = :IDELEMENTO
                  AND CDSERVORCA         = :CDSERVORCA
                  AND CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL
                  AND NRSEQORCAMENTO     = :NRSEQORCAMENTO
                  AND NRORG              = :NRORG";
                  
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
   /**
     * Retorna os dados da tabela de preço ligados a medida de um serviço 
    **/
    public function getSomatorioMedidaServico(array $params) {
        $sql = "SELECT SUM(RELACSERV.QTMEDIDASERV) AS QTMEDIDASERV
                  FROM FCT_RELACSERVICO RELACSERV
                 INNER JOIN FCT_SERVICOORCA SERV
                    ON RELACSERV.CDSERVORCA = SERV.CDSERVORCA
                   AND RELACSERV.NRORG     = SERV.NRORG
                 INNER JOIN (SELECT IDELEMENTO
                               FROM (SELECT DISTINCT IDELEMENTO, IDELEMENTOPAI
                                       FROM FCT_TRABSERVORCAMENTO 
                                      WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                        AND NRORG          = :NRORG
                                        AND IDESCOLHIDO    = 'S'
                                        AND IDATIVO        = 'S')
                            CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                              START WITH IDELEMENTO     = :IDELEMENTO) TRAB
                    ON SERV.IDELEMENTO = TRAB.IDELEMENTO
                 WHERE RELACSERV.CDSERVORCARELAC = :CDSERVORCA
                   AND RELACSERV.NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Atualiza o valor de um item
    **/
    public function atualizarValorItensOrcamento(array $params){
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET VRUNITITEMORCA         = :VRUNITITEMORCA,
                    VRTOTALITEMORCA        = :VRTOTALITEMORCA,
                    VRUNITITEMORCATBLPRECO = :VRUNITITEMORCATBLPRECO,
                    QTMEDIDAORCA           = :QTMEDIDAORCA
                WHERE NRITEMORCAMENTO = :NRITEMORCAMENTO
                    AND NRORG         = :NRORG";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna os dados de uma negociacao a partir das informações de faturamento
     * fornecidas
    **/
    public function getNovaNegociacaoCorrespondente(array $params) {
        $sql = "SELECT NRNEGOCIACAOPROPOSTA,
                       VRTOTITEMCOMDESC
                FROM FCT_NEGOCIACAOPROPOSTA
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                  AND IDTPFATURA        = :IDTPFATURA
                  AND CDSERIENF         = :CDSERIENF
                  AND CDTIPOOPERFATURA  = :CDTIPOOPERFATURA
                  AND CDFILIALFATURA    = :CDFILIALFATURA
                  AND NRORG             = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * insere um desconto por período
    **/
    public function inserirDescontoPorPeriodo(array $params) {
        $sql = "INSERT INTO FCT_DESCPERIODONEGOPROP (
                    NRDESCPERIODONEGOPROP, IDATIVO,       NRORG,
                    VRPERCDESCITEM,        VRDESCITEM,    VRTOTITEMCOMDESC,
                    NRNEGOCIACAOPROPOSTA,  QTMESESDESC,   DTFINALVALIDADEDESC,
                    DTINCLUSAO,            NRORGINCLUSAO, CDOPERINCLUSAO,
                    DTULTATU,              NRORGULTATU,   CDOPERULTATU,
                    NRORDEMAPLICADESC,     IDTPDESCONTO
                ) VALUES (
                    :NRDESCPERIODONEGOPROP, :IDATIVO,       :NRORG,
                    :VRPERCDESCITEM,        :VRDESCITEM,    :VRTOTITEMCOMDESC,
                    :NRNEGOCIACAOPROPOSTA,  :QTMESESDESC,   :DTFINALVALIDADEDESC,
                    :DTINCLUSAO,            :NRORGINCLUSAO, :CDOPERINCLUSAO,
                    :DTULTATU,              :NRORGULTATU,   :CDOPERULTATU,
                    :NRORDEMAPLICADESC,     :IDTPDESCONTO
                )";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * Atualiza o valor dos itens da negociação 
    **/
    public function atualizarItensNegociaco(array $params, $sqlAux) {
        $sql = "UPDATE FCT_ITNEGOCIACAOPROPOSTA
                SET $sqlAux
                WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                    AND NRITEMORCAMENTO    = :NRITEMORCAMENTO
                    AND NRORG              = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * Atualiza o valor da negociação 
    **/
    public function atualizarNegociaco(array $params, $sqlAux) {
        $sql = "UPDATE FCT_NEGOCIACAOPROPOSTA
                SET $sqlAux
                WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                    AND NRORG              = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getIdMultiplicavariacaoItem(array $params) {
        $sql = "SELECT NVL(VAR.IDMULTIPVARIACAO, 'S') AS IDMULTIPVARIACAO,
                  DECODE(VAR.VRPRECOVARIACAO, NULL, ITVAR.VRPRECO, VAR.VRPRECOVARIACAO) AS VRPRECOVARIACAO
                FROM FCT_ITPRECBASESERV ITVAR
                LEFT JOIN FCT_VARIACAOPRECBASESERV VAR
                  ON VAR.NRSEQITPRECBSERV  = ITVAR.NRSEQITPRECBSERV
                  AND VAR.NRORG            = ITVAR.NRORG
                WHERE ITVAR.NRORG          = :NRORG
                  AND ITVAR.CDSERVORCA     = :CDSERVORCA
                  AND ITVAR.CDTPMEDIDASERV = :CDTPMEDIDASERV
                  AND (VAR.IDMULTIPVARIACAO IS NULL
                      OR VAR.CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL 
                      AND :QTMEDIDAORCA BETWEEN QTMEDIDAINIVARIACAO AND QTMEDIDAFIMVARIACAO)";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Retorna as parcelas das negociações de uma proposta
    **/
    public function getParcelasProposta(array $params) {
        $sql = "SELECT NRPARCELANEGOPROP,
                    QTDIASFATURAMENTO,
                    NRPARCELA,
                    NRDIAVENCIMENTO,
                    NRNEGOCIACAOPROPOSTA,
                    VRPARCELA,
                    NRORG,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO
                FROM FCT_PARCELANEGOPROP
                WHERE NRNEGOCIACAOPROPOSTA IN (SELECT NRNEGOCIACAOPROPOSTA
                                               FROM FCT_NEGOCIACAOPROPOSTA
                                               WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                                    AND NRORG       = :NRORG)";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function validaBloqAlteraValor(array $params) {
        $sql = "SELECT IDALTVRTABRPECPROP
                  FROM FCT_TIPOORCAMENTO
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG = :NRORG ";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getMedidaVariavelServicoProposta(array $params) {
        $sql = "SELECT NRRELACMEDIDAPROPOSTA, QTMEDIDAVARIAVEL
                  FROM FCT_RELACMEDIDAPROPOSTA
                 WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                   AND CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL
                   AND IDELEMENTORAIZ     IS NULL
                   AND CDSERVORCA         = :CDSERVORCA
                   AND NRORG              = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function atualizarMedidaVariavelServico(array $params) {
        $sql = "UPDATE FCT_RELACMEDIDAPROPOSTA
                   SET QTMEDIDAVARIAVEL = :QTMEDIDAVARIAVEL,
                       DTULTATU         = SYSDATE,
                       NRORGULTATU      = :NRORGULTATU,
                       CDOPERULTATU     = :CDOPERULTATU
                 WHERE NRRELACMEDIDAPROPOSTA = :NRRELACMEDIDAPROPOSTA
                   AND NRORG                 = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function inserirMedidaVariavelServico(array $params) {
        $sql = "INSERT INTO FCT_RELACMEDIDAPROPOSTA (
            NRRELACMEDIDAPROPOSTA,  NRSEQORCAMENTO,  CDTPMEDIDAVARIAVEL,
            QTMEDIDAVARIAVEL,       NRORG,           DTINCLUSAO,
            NRORGINCLUSAO,          CDOPERINCLUSAO,  IDATIVO,
            IDELEMENTORAIZ,         QTFRANQUIA,      CDSERVORCA,
            NRTIPOORCAMENTO
        )
        VALUES (
            :NRRELACMEDIDAPROPOSTA,  :NRSEQORCAMENTO,  :CDTPMEDIDAVARIAVEL,
            :QTMEDIDAVARIAVEL,       :NRORG,           SYSDATE,
            :NRORGINCLUSAO,          :CDOPERINCLUSAO,  :IDATIVO,
            :IDELEMENTORAIZ,         :QTFRANQUIA,      :CDSERVORCA,
            :NRTIPOORCAMENTO
        )";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getSomaQuantidadeMedidaServicoProposta(array $params) {
        $sql = "SELECT SUM(QTMEDIDAORCA) AS QTMEDIDAORCA
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                   AND CDSERVORCA         = :CDSERVORCA
                   AND NRORG              = :NRORG
                   AND NRITEMORCAMENTOPAI IS NOT NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**

     * realiza o update da proposta para incluir / alterar o campo "DSALTERACONTRATUAIS"
    **/
    public function updateDsAlteracoesContr(array $params) {
        $sql = "UPDATE FCT_ORCAMENTO
                   SET DSALTERACONTRATUAIS = :DSALTERACONTRATUAIS,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR,
                       NRORGULTATU = :NRORG
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    /**
     * Valida se o operador tem acesso à página "Alterações Contratuais"
    **/
    public function validaAlteracoesContratuais(array $params) {
        $sql = "SELECT PERFACESSPARC.NRPERFACESSPARC, PERFACESSPARC.NRPARCNEGOCIO, OPERADOR.CDOPERADOR
                  FROM PERFACESSPARC
                  JOIN OPERADOR 
                    ON OPERADOR.NRPARCNEGOCIO = PERFACESSPARC.NRPARCNEGOCIO
                   AND OPERADOR.NRORG = PERFACESSPARC.NRORG
                 WHERE PERFACESSPARC.NRPERFILACESSO IN (SELECT NRPERFILACESSO 
                                                          FROM FCT_ALTPROPCONCPERF)
                   AND PERFACESSPARC.NRORG = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     *recebe os dados de "Alterações Contratuais"
    **/
    public function getAlteracaoContratual(array $params) {
        $sql = "SELECT DSALTERACONTRATUAIS
                  FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * valida se o Token do Certsign está preenchido
    **/
    public function validaTokenCertsign(array $params) {
        $sql = "SELECT DSTOKENCERTSIGN 
                  FROM FCT_PARAMETROGERAL
                 WHERE NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca os signatários padrões do modelo de proposta 
    **/
    public function buscaSignatariosPadroes(array $params) {
        $sql = "SELECT FCT_SIGNATPADRMODORCA.NRSIGNATPADRORCA, FCT_SIGNATPADRMODORCA.NRORCAMENTOMODELO, FCT_SIGNATPADRMODORCA.NRPARCNEGOCIO, 
                       PARCNEGOCIO.NRINSCRICAOPARC , FCT_SIGNATPADRMODORCA.IDTIPOASSINATURA, FCT_SIGNATPADRMODORCA.IDTESTEMUNHA, 
                       FCT_SIGNATPADRMODORCA.NRORDENACAO, FCT_SIGNATPADRMODORCA.DSEMAILNOTSIGNENV, FCT_SIGNATPADRMODORCA.DSEMAILNOTIFOUTROS, FCT_SIGNATPADRMODORCA.NRORG 
                  FROM FCT_SIGNATPADRMODORCA
                  JOIN PARCNEGOCIO 
                    ON PARCNEGOCIO.NRPARCNEGOCIO = FCT_SIGNATPADRMODORCA.NRPARCNEGOCIO
                    AND PARCNEGOCIO.NRORG = FCT_SIGNATPADRMODORCA.NRORG
                  WHERE FCT_SIGNATPADRMODORCA.NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                    AND FCT_SIGNATPADRMODORCA.NRORG = :NRORG
                    AND FCT_SIGNATPADRMODORCA.IDTPSIGNATARIO = 'SE'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    /**
     * insere os signatários padrões do modelo de proposta 
    **/
    public function insereAssinatura(array $params) {
        
        $sql = "INSERT INTO FCT_ASSINATDIGITALPROP (NRASSDIGITALORCA, NRSEQORCAMENTO, NRSEQINTERLOCUTOR, NRPARCNEGOCIO, NRCPFSIGNATARIO, IDTIPOASSINATURA, IDTESTEMUNHA,
                                                    NRORDENACAO, DSEMAILNOTSIGNENV, DSEMAILNOTIFOUTROS, NRORG, DTINCLUSAO, CDOPERINCLUSAO, 
                                                    NRORGINCLUSAO, IDATIVO, IDSIGNATCERTSIGN)
                            VALUES (:NRASSDIGITALORCA, :NRSEQORCAMENTO, :NRSEQINTERLOCUTOR, :NRPARCNEGOCIO, :NRCPFSIGNATARIO, :IDTIPOASSINATURA, :IDTESTEMUNHA, 
                                    :NRORDENACAO, :DSEMAILNOTSIGNENV, :DSEMAILNOTIFOUTROS, :NRORG, SYSDATE, :OPERADOR, 
                                    :NRORG, 'S', :IDSIGNATCERTSIGN)";
                             
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * A função "validaSignatCadastrados" realiza a validação dos signatários padrões para evitar a inclusão repetida de um parceiro.
     */
    public function validaSignatCadastrados(array $params) {
        $sql = "SELECT COUNT (*) AS VALIDA
                  FROM FCT_ASSINATDIGITALPROP
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO 
                   AND ((NRPARCNEGOCIO = :NRPARCNEGOCIO) OR (NRPARCNEGOCIO IS NULL))
                   AND ((NRSEQINTERLOCUTOR = :NRSEQINTERLOCUTOR) OR (NRSEQINTERLOCUTOR IS NULL)) 
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["VALIDA"];
    }
    
    /**
     * A função "recebeIdAssEletrDigital" verifica se a proposta está apta para assinatura eletronica / digital
     */
    public function recebeIdAssEletrDigital(array $params) {
        
        $sql = "SELECT IDASSELETRDIGITAL 
                  FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "updateSituacaoAssManualParc" atualiza o signatario de parceiro de negócio como 
     * "Assinado Manualmente"
     */
    public function updateSituacaoAssManualParc(array $params) {
        
        $sql = "UPDATE FCT_ASSINATDIGITALPROP
                   SET IDSITUACAO = 'M',
                       DTULTATU = SYSDATE, 
                       CDOPERULTATU = :OPERADOR, 
                       NRORGULTATU = :NRORG
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /**
     * A função "updateSituacaoAssManualInte" atualiza o signatario de interlocutor como 
     * "Assinado Manualmente" 
     */
    public function updateSituacaoAssManualInte(array $params) {
        
        $sql = "UPDATE FCT_ASSINATDIGITALPROP
                   SET IDSITUACAO = 'M',
                       DTULTATU = SYSDATE, 
                       CDOPERULTATU = :OPERADOR, 
                       NRORGULTATU = :NRORG
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRSEQINTERLOCUTOR = :NRSEQINTERLOCUTOR 
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * A função "recebeassinaturas" recebe as assinaturas da proposta
     */
    public function recebeassinaturas(array $params) {
        
        $sql = "SELECT NRASSDIGITALORCA, NRSEQORCAMENTO, NRPARCNEGOCIO, NRCPFSIGNATARIO, IDTIPOASSINATURA, 
                	   IDTESTEMUNHA, NRORDENACAO, IDSITUACAO, NRORG
                  FROM FCT_ASSINATDIGITALPROP
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "buscarDadosInterlocutor" busca o CPF e o Tipo de Signatário do interlocutor
     */
    public function buscarDadosInterlocutor(array $params) {
        
        $sql = "SELECT NRCPFINTERLOC, IDTPSIGNATARIO 
                  FROM CRM_INTERLOCUTOR
                 WHERE NRSEQINTERLOCUTOR = :NRSEQINTERLOCUTOR
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
     /**
     * A função "getDadosSender" recebe os dados de responsável pelo envio do documento
     */
    public function getDadosSender(array $params) {
        $sql = "SELECT NMSENDERCERTSIGN, DSEMAILSENDERCERTSIGN, NRCPFSENDERCERTSIGN, IDPADRAOASSINACERTSIGN, NRTIPODOCCERTSIGN
                  FROM FCT_PARAMETROGERAL 
                 WHERE NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "getSignatarios" recebe os dados dos signatários para o envio do documento
     */
    public function getSignatarios(array $params) {
        
        $sql = "SELECT FCT_ASSINATDIGITALPROP.NRASSDIGITALORCA, FCT_ASSINATDIGITALPROP.NRSEQORCAMENTO, FCT_ASSINATDIGITALPROP.NRPARCNEGOCIO, 
                       FCT_ASSINATDIGITALPROP.NRCPFSIGNATARIO, FCT_ASSINATDIGITALPROP.IDTIPOASSINATURA, FCT_ASSINATDIGITALPROP.IDTESTEMUNHA, 
                       FCT_ASSINATDIGITALPROP.NRORDENACAO, FCT_ASSINATDIGITALPROP.IDSITUACAO, FCT_ASSINATDIGITALPROP.NRORG, 
                       FCT_ASSINATDIGITALPROP.NRSEQINTERLOCUTOR, OPERADOR.NMOPERADOR AS NOME, OPERADOR.DSEMAILOPER AS EMAIL
                  FROM FCT_ASSINATDIGITALPROP
                  JOIN OPERADOR 
                    ON OPERADOR.NRPARCNEGOCIO = FCT_ASSINATDIGITALPROP.NRPARCNEGOCIO
                  AND OPERADOR.NRORG = FCT_ASSINATDIGITALPROP.NRORG 
                 WHERE FCT_ASSINATDIGITALPROP.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND FCT_ASSINATDIGITALPROP.IDTIPOASSINATURA = :IDTIPOASSINATURA
                   AND FCT_ASSINATDIGITALPROP.NRORG = :NRORG
                    
                 UNION ALL
                 
                 SELECT FCT_ASSINATDIGITALPROP.NRASSDIGITALORCA, FCT_ASSINATDIGITALPROP.NRSEQORCAMENTO, FCT_ASSINATDIGITALPROP.NRPARCNEGOCIO, 
                        FCT_ASSINATDIGITALPROP.NRCPFSIGNATARIO, FCT_ASSINATDIGITALPROP.IDTIPOASSINATURA, FCT_ASSINATDIGITALPROP.IDTESTEMUNHA, 
                        FCT_ASSINATDIGITALPROP.NRORDENACAO, FCT_ASSINATDIGITALPROP.IDSITUACAO, FCT_ASSINATDIGITALPROP.NRORG, 
                        FCT_ASSINATDIGITALPROP.NRSEQINTERLOCUTOR, CRM_INTERLOCUTOR.NMINTERLOCUTOR AS NOME, CRM_INTERLOCUTOR.EMAILINTERLOC AS EMAIL
                  FROM FCT_ASSINATDIGITALPROP
                  JOIN CRM_INTERLOCUTOR
                    ON CRM_INTERLOCUTOR.NRSEQINTERLOCUTOR = FCT_ASSINATDIGITALPROP.NRSEQINTERLOCUTOR 
                   AND CRM_INTERLOCUTOR.NRORG = FCT_ASSINATDIGITALPROP.NRORG
                 WHERE FCT_ASSINATDIGITALPROP.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND FCT_ASSINATDIGITALPROP.IDTIPOASSINATURA = :IDTIPOASSINATURA
                   AND FCT_ASSINATDIGITALPROP.NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "getIdDocumentoCertsign" recebe os ids dos documentos do Certsign
     */
    public function getIdDocumentoCertsign(array $params) {
        
        /*$sql = "SELECT DISTINCT DOC.IDDOCUMCERTSIGN, DOC.CHAVEDOCUMCERTSIGN
                  FROM FCT_ORCAMENTODOC DOC
                  JOIN FCT_ORCAMENTO ORC
                    ON DOC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                 WHERE DOC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND DOC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND (DOC.IDSITUACAOORCAMENTO = ORC.IDSITUACAOORCAMENTO OR DOC.IDSITUACAOORCAMENTO IS NULL)
                   AND DOC.NRORG = :NRORG";*/
        
        $sql = "SELECT DISTINCT IDDOCUMCERTSIGN, CHAVEDOCUMCERTSIGN
                  FROM FCT_ORCAMENTODOC
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND (IDSITUACAOORCAMENTO = 'CONCLUIDO' OR IDSITUACAOORCAMENTO IS NULL)
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "updateDocumentoIdCertsign" atualiza o registro dos documentos da proposta 
     * para incluir o ID e a Chave do Certsing" 
     */
    public function updateDocumentoIdCertsign(array $params) {
        
        $sql = "UPDATE FCT_ORCAMENTODOC
                   SET IDDOCUMCERTSIGN = :IDDOCUMCERTSIGN,
                       CHAVEDOCUMCERTSIGN = :CHAVEDOCUMCERTSIGN,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR,
                       NRORGULTATU = :NRORG
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND ((IDSITUACAOORCAMENTO IN ('ACEITO','CONCLUIDO'))
                    OR (IDSITUACAOORCAMENTO IS NULL))
                   AND NRORG = :NRORG";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * A função "updateDocumentoIdCertsign" atualiza o registro dos signatários da proposta 
     * para incluir o ID do Certsing
     */
    public function updateSignatarioIdCertsign(array $params) {
        
        $sql = "UPDATE FCT_ASSINATDIGITALPROP
                   SET IDSIGNATCERTSIGN = :IDSIGNATCERTSIGN,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR,
                       NRORGULTATU = :NRORG
                 WHERE NRCPFSIGNATARIO = :NRCPFSIGNATARIO
                   AND NRORG = :NRORG";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function clearDocumentoIdCertsign(array $params) {
        
        $sql = "UPDATE FCT_ORCAMENTODOC
                   SET IDDOCUMCERTSIGN = NULL,
                       CHAVEDOCUMCERTSIGN = NULL,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR,
                       NRORGULTATU = :NRORG
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND ((IDSITUACAOORCAMENTO IN ('ACEITO','CONCLUIDO'))
                    OR (IDSITUACAOORCAMENTO IS NULL))
                   AND NRORG = :NRORG";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function clearSignatarioIdCertsign(array $params) {
        
        $sql = "UPDATE FCT_ASSINATDIGITALPROP
                   SET IDSIGNATCERTSIGN = NULL,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR,
                       NRORGULTATU = :NRORG
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND ((NRCPFSIGNATARIO = :NRCPFSIGNATARIO)
                    OR ('T' = :NRCPFSIGNATARIO))
                   AND NRORG = :NRORG";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * A função "validaExisteDocCertsign" verifica se existe um documento gerado para a proposta no Certsign
     */
    public function validaExisteDocCertsign(array $params) {
        
        $sql = "SELECT DECODE(IDDOCUMCERTSIGN, NULL, 'FALSE', 'TRUE') AS IDDOCUMCERTSIGN
                  FROM FCT_ORCAMENTODOC
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND ((IDSITUACAOORCAMENTO IN ('ACEITO','CONCLUIDO'))
                    OR (IDSITUACAOORCAMENTO IS NULL))
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getServicosComMedicao(array $params) {
        $sql = "SELECT REL.CDTPMEDIDAVARIAVEL,
                       REL.QTMEDIDAVARIAVEL,
                       REL.CDSERVORCA,
                       REL.NRTIPOORCAMENTO
                  FROM FCT_RELACMEDIDAPROPOSTA REL
                 INNER JOIN FCT_ITEMORCAMENTO IT
                    ON REL.NRSEQORCAMENTO = IT.NRSEQORCAMENTO
                   AND REL.CDSERVORCA     = IT.CDSERVORCA
                 WHERE REL.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                   AND REL.NRORG             = :NRORG
                   AND REL.IDELEMENTORAIZ    IS NULL
                   AND REL.CDSERVORCA        IS NOT NULL
                   AND IT.NRITEMORCAMENTOPAI IS NULL
                   AND IT.IDELEMENTO         IS NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function atualizarValorVariacaoPrecoServicoProposta(array $params) {
        $sql = "UPDATE FCT_VARPRECBASESERVORCA
                   SET CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL,
                       DTULTATU           = SYSDATE,
                       CDOPERULTATU       = :CDOPERULTATU,
                       NRORGULTATU        = :NRORGULTATU
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG
                   AND CDSERVORCA     = :CDSERVORCA";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function atualizarEscopoItemProposta(array $params) {
        $sql = "UPDATE FCT_ITEMORCAMENTO
                   SET NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                 WHERE NRITEMORCAMENTO = :NRITEMORCAMENTO
                   AND NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    
    public function recebeSeqDocOrca(array $params, $nrtipoorcamento) {
        $sql = "SELECT NRSEQORCADOC, NRTIPOORCAMENTO, IDDOCUMCERTSIGN, CHAVEDOCUMCERTSIGN
                  FROM FCT_ORCAMENTODOC
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRTIPOORCAMENTO IN ($nrtipoorcamento)
                   AND IDSITUACAOORCAMENTO = 'CONCLUIDO'
                   AND NRORG = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
 
    public function recebeSeqSignatario(array $params) {
        $sql = "SELECT NRASSDIGITALORCA 
                 FROM FCT_ASSINATDIGITALPROP
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRCPFSIGNATARIO = :NRCPFSIGNATARIO
                  AND NRORG = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }   
    
    public function insereDocAssinatura(array $params) {
        $sql = "INSERT INTO  FCT_DOCASSINATPROP 
            		(NRDOCASSINATPROP, NRASSDIGITALORCA, NRSEQORCADOC, IDSITUACAO, IDSIGNATCERTSIGN, NRSEQORCAMENTO,
            		NRORG, IDATIVO, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                 VALUES
                    (:NRDOCASSINATPROP,:NRASSDIGITALORCA,:NRSEQORCADOC,'P',:IDSIGNATCERTSIGN, :NRSEQORCAMENTO,
                    :NRORG, 'S', SYSDATE,:OPERADOR,:NRORG)";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }   
    
    
    /**
     * A função "getIdDocumentoCertsign" recebe os ids dos documentos do Certsign
     */
    public function getIdDocumentoCertsignDoc(array $params) {
        
        $sql = "SELECT IDDOCUMCERTSIGN, CHAVEDOCUMCERTSIGN 
                  FROM FCT_ORCAMENTODOC
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRSEQORCADOC = :NRSEQORCADOC
                   AND ((IDSITUACAOORCAMENTO IN ('ACEITO','CONCLUIDO'))
                    OR (IDSITUACAOORCAMENTO IS NULL))
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    /**
     * A função "getCpfSignatAssDoc" recebe o CPF dos signatários dos documentos do Certsign
     */
    public function getCpfSignatAssDoc(array $params) {
        
        $sql = "SELECT NRCPFSIGNATARIO 
                  FROM FCT_ASSINATDIGITALPROP
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function clearDocumentoIdCertsignDoc(array $params) {
        
        $sql = "UPDATE FCT_ORCAMENTODOC
                   SET IDDOCUMCERTSIGN = NULL,
                       CHAVEDOCUMCERTSIGN = NULL,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR,
                       NRORGULTATU = :NRORG
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRSEQORCADOC = :NRSEQORCADOC
                   AND ((IDSITUACAOORCAMENTO IN ('ACEITO','CONCLUIDO'))
                    OR (IDSITUACAOORCAMENTO IS NULL))
                   AND NRORG = :NRORG";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /**
     * A função "updateSituacaoAssManualDoc" atualiza o signatario como  "Assinado Manualmente"
     */
    public function updateSituacaoAssManualDoc(array $params) {
        
        $sql = "UPDATE FCT_DOCASSINATPROP
                   SET IDSIGNATCERTSIGN = NULL,
                       IDSITUACAO = 'M',
                       DTULTATU = SYSDATE, 
                       CDOPERULTATU = :OPERADOR, 
                       NRORGULTATU = :NRORG
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRSEQORCADOC = :NRSEQORCADOC
                   AND IDSITUACAO = 'P'
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /* Recebe situação e justificativa de recusa dos documentos de assinatura da proposta */
    public function recebeDadosAssDocProp (array $params){
    
        $sql = "SELECT IDSITUACAO, DSJUSTIFICARECUSA
                  FROM FCT_DOCASSINATPROP
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    /*Atualiza situação das assinaturas da proposta*/
    public function updateAssinaProposta (array $params) {
        $sql = "UPDATE FCT_ASSINATDIGITALPROP
                   SET IDSITUACAO = :IDSITUACAO,
                       DSJUSTIFICARECUSA = :DSJUSTIFICARECUSA,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR, 
                       NRORGULTATU = :NRORG
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    /* Recebe o sequencial do documento da assinatura*/
    public function recebeSeqDocumento (array $params){
    
        $sql = "SELECT NRSEQORCADOC
                  FROM FCT_DOCASSINATPROP
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND IDSITUACAO = 'P'
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getParcelasOrderQtDiasFatu(array $params){
        
        $sql = "SELECT NRPARCELANEGOPROP,
                       QTDIASFATURAMENTO,
                       NRPARCELA,
                       NRDIAVENCIMENTO,
                       NRNEGOCIACAOPROPOSTA,
                       VRPARCELA,
                       NRORG,
                       NRORGINCLUSAO,
                       CDOPERINCLUSAO
                  FROM FCT_PARCELANEGOPROP
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG = :NRORG
                 ORDER BY QTDIASFATURAMENTO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function atualizarNrParcelas(array $params){
        
        $sql = "UPDATE FCT_PARCELANEGOPROP
                   SET NRPARCELA = :NRPARCELA
                 WHERE NRPARCELANEGOPROP = :NRPARCELANEGOPROP
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    public function deleteInterlocutorInserido(array $params){
        
        $sql = "DELETE FROM FCT_ASSINATDIGITALPROP
                      WHERE NRSEQINTERLOCUTOR = :NRSEQINTERLOCUTOR
                        AND NRSEQORCAMENTO = :NRSEQORCAMENTO
                        AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function validaGeracaoDocCertisign(array $params) {
        
        $sql = "SELECT COUNT (*) AS VALIDA 
                  FROM FCT_ORCAMENTODOC
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG
                   AND IDDOCUMCERTSIGN IS NOT NULL
                   AND CHAVEDOCUMCERTSIGN IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["VALIDA"];
    }
    
    public function habilitaAssinatura(array $params) {
        
        $sql = "UPDATE FCT_ORCAMENTO
                   SET IDASSELETRDIGITAL = DECODE(IDASSELETRDIGITAL, 'S', 'N', 'S'),
                       NRORGULTATU = :NRORG,
                       DTULTATU = SYSDATE,
                       CDOPERULTATU = :OPERADOR
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /*public function getVariacaoPrecoContrato(array $params) {
        $sql = " SELECT CDTPMEDIDASERV,
                        CDTPMEDIDAVARIAVEL,
                        QTMEDIDAINIVARIACAO,
                        QTMEDIDAFIMVARIACAO,
                        VRPRECOVARIACAO,
                        IDMULTIPVARIACAO,
                        NULL AS DTFIMVALPBSERV,
                        CDSERVORCA
                   FROM FCT_VARPRECBASESERVCONT
                   WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }*/
    
     /**
     * Valida se é necessário filtrar os serviços na inserção de contratos manuais
    **/
    public function validaFiltroServico(array $params) {
        
        $sql = "SELECT IDFILTRASERVICO
                  FROM FCT_TIPOORCAMENTO
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG = :NRORG";
            
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["IDFILTRASERVICO"];
    }
    
    
     /**
     * Valida se é necessário filtrar os serviços na inserção de contratos manuais
    **/
    public function countServicos(array $params) {
        
        $sql = "SELECT COUNT(*) AS VALIDA
                  FROM FCT_ITESCOPOPROPOSTA
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG = :NRORG";
            
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["VALIDA"];
    }
    
    
    public function validaStatusAssinaturas(array $params) {
        
        $sql = "SELECT NRASSDIGITALORCA, IDSITUACAO 
                  FROM FCT_ASSINATDIGITALPROP
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

     /**
     * deleta os operadores de notificação de assinatura
    **/
    public function deletaOperadoresNotifica (array $params) {
        
        $sql = "DELETE FROM FCT_RELACOPERNOTIFASSELE
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRORG = :NRORG";
                             
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /**
     * insere os operadores de notificação de assinatura
    **/
    public function insereOperadoresNotifica (array $params) {
        $sql = "INSERT INTO FCT_RELACOPERNOTIFASSELE
                    (NRRELOPERNOTIFASSELE, NRASSDIGITALORCA, CDOPERADOR, NRORG, IDATIVO,            
                	 DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES 
                    (:NRRELOPERNOTIFASSELE, :NRASSDIGITALORCA, :CDOPERADOR, :NRORG, 'S',             
                	 SYSDATE, :OPERADOR, :NRORG)";
                             
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Busca os operadores padrões para notificação de modelo de proposta da parametrização SM 
    **/
    public function buscaOperadoresNotif(array $params) {
        $sql = "SELECT CDOPERADOR 
                  FROM FCT_RELACOPERNOTIFASSELE
                 WHERE NRSIGNATPADRORCA = :NRSIGNATPADRORCA
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca os signatários padrões do modelo de proposta 
    **/
    public function buscaSignatariosPadroesPorTipo(array $params) {
        $sql = "SELECT SIG.NRSIGNATPADRORCA,
                       SIG.NRORCAMENTOMODELO,
                       SIG.NRPARCNEGOCIO,
                       PARC.NRINSCRICAOPARC,
                       SIG.IDTIPOASSINATURA,
                       SIG.IDTESTEMUNHA,
                       SIG.NRORDENACAO,
                       SIG.NRORG,
                       SIG.DSEMAILNOTSIGNENV,
                       SIG.DSEMAILNOTIFOUTROS
                  FROM FCT_SIGNATPADRMODORCA SIG
                  LEFT JOIN PARCNEGOCIO PARC
                    ON PARC.NRPARCNEGOCIO = SIG.NRPARCNEGOCIO
                   AND PARC.NRORG         = SIG.NRORG
                 WHERE SIG.NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                   AND SIG.IDTPSIGNATARIO    = :IDTPSIGNATARIO
                   AND SIG.NRORG             = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    

    public function recebeOrdenacaoENotificacoes(array $params) {
        
        $sql = "SELECT NRSIGNATPADRORCA, NRORDENACAO, DSEMAILNOTSIGNENV, DSEMAILNOTIFOUTROS 
                  FROM FCT_SIGNATPADRMODORCA
                 WHERE NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                   AND IDTPSIGNATARIO = :IDTPSIGNATARIO
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    public function recebeOperdoresNotificacoes(array $params) {
        
        $sql = "SELECT CDOPERADOR
                  FROM FCT_RELACOPERNOTIFASSELE
                 WHERE NRSIGNATPADRORCA = :NRSIGNATPADRORCA
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function validaGeraDocCertisign(array $params) {
        
        $sql = "SELECT IDENVIADOCCONCLUSAO
                  FROM FCT_ORCAMENTOMODELO
                 WHERE NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                   AND NRORG = :NRORG";
            
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["IDENVIADOCCONCLUSAO"];
    }
    
    public function excluirSignatarioProposta(array $params) {
        $sql = "DELETE FROM FCT_ASSINATDIGITALPROP
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                  AND NRORG             = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getItensElementoProposta(array $params) {
        $sql = "SELECT NRITEMORCAMENTO,
                       NRITEMORCAMENTOPAI,
                       CDSERVORCA,
                       IDELEMENTO,
                       NMITEMORCAMENTO,
                       QTMEDIDAORCA,
                       CDTPMEDIDASERV,
                       CDTPMEDIDAVARIAVEL
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG
                   AND IDADICITEMORC  = 'N'
                   AND NRITEMORCAMENTOPAI IS NOT NULL
                   AND IDELEMENTO IS NOT NULL";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getUtilizaAssinaturaEletronicaProposta(array $params) {
        $sql = "SELECT IDASSELETRDIGITAL
                  FROM FCT_ORCAMENTO
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getTipoDocumento (array $params) {
        $sql = "SELECT NRTIPODOCCERTSIGN
                  FROM FCT_ORCAMENTOMODELO
                 WHERE NRORCAMENTOMODELO = (SELECT NRORCAMENTOMODELO
                                                FROM FCT_ORCAMENTO
                                            WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO)";
                                            
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();              
    }
    
    public function getEnviarDocAss (array $params) {
        $sql = "SELECT OP.DSEMAILOPER
                  FROM FCT_ORCAMENTO ORC
                  JOIN OPERADOR OP
                    ON ORC.CDOPERRESPONSAVEL = OP.CDOPERADOR
                 WHERE ORC.NRSEQORCAMENTO        = :NRSEQORCAMENTO
                   AND IDENVIADOCOPERRESPCONCASS = 'S'";
                                            
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();              
    }
}
?>
